@@ -7,7 +7,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: common.cpp 3024 2005-08-17 11:46:09Z mosu $
+   $Id: common.cpp 3524 2007-04-19 07:50:15Z mosu $
 
    helper functions, common variables
 
@@ -16,13 +16,11 @@
 
 #include "os.h"
 
-#include <assert.h>
+#include <cassert>
 #include <ctype.h>
 #include <errno.h>
 #include <iconv.h>
-#if defined(COMP_MSC)
-# include <libcharset.h>
-#elif HAVE_NL_LANGINFO
+#if HAVE_NL_LANGINFO
 # include <langinfo.h>
 #elif HAVE_LOCALE_CHARSET
 # include <libcharset.h>
@@ -660,7 +658,7 @@ remove_unique_uint32(uint32_t number,
 
   if (hack_engaged(ENGAGE_NO_VARIABLE_DATA))
     return true;
-  foreach(dit, ru_numbers[category])
+  mxforeach(dit, ru_numbers[category])
     if (*dit == number) {
       ru_numbers[category].erase(dit);
       return true;
@@ -740,6 +738,9 @@ _saferealloc(void *mem,
              size_t size,
              const char *file,
              int line) {
+  if (0 == size)
+    // Do this so realloc() may not return NULL on success.
+    size = 1;
   mem = realloc(mem, size);
   if (mem == NULL)
     die("common.cpp/saferealloc() called from file %s, line %d: realloc() "
@@ -757,7 +758,7 @@ safefree(void *p) {
 
 void
 dump_malloc_report() {
-  mxinfo("%llu bytes malloced, %llu bytes duplicated\n",
+  mxinfo("" LLU " bytes malloced, " LLU " bytes duplicated\n",
          _safemalloced - _safedupped, _safedupped);
 }
 
@@ -852,6 +853,30 @@ strip(vector<string> &v,
     strip(v[i], newlines);
 }
 
+string &
+shrink_whitespace(string &s) {
+  int i;
+  bool previous_was_whitespace;
+
+  i = 0;
+  previous_was_whitespace = false;
+  while (s.length() > i) {
+    if (!isblanktab(s[i])) {
+      previous_was_whitespace = false;
+      ++i;
+      continue;
+    }
+    if (previous_was_whitespace)
+      s.erase(i, 1);
+    else {
+      previous_was_whitespace = true;
+      ++i;
+    }
+  }
+
+  return s;
+}
+
 string
 escape(const string &source) {
   string dst;
@@ -901,8 +926,7 @@ unescape(const string &source) {
 }
 
 string
-escape_xml(const string &source,
-           bool escape_quotes) {
+escape_xml(const string &source) {
   string dst;
   string::const_iterator src;
 
@@ -914,7 +938,7 @@ escape_xml(const string &source,
       dst += "&gt;";
     else if (*src == '<')
       dst += "&lt;";
-    else if (escape_quotes && (*src == '"'))
+    else if (*src == '"')
       dst += "&quot;";
     else
       dst += *src;
@@ -933,7 +957,7 @@ create_xml_node_name(const char *name,
   node_name = string("<") + name;
   for (i = 0; (NULL != atts[i]) && (NULL != atts[i + 1]); i += 2)
     node_name += string(" ") + atts[i] + "=\"" +
-      escape_xml(atts[i + 1], true) + "\"";
+      escape_xml(atts[i + 1]) + "\"";
   node_name += ">";
 
   return node_name;
@@ -1012,6 +1036,18 @@ round_to_nearest_pow2(uint32_t value) {
   return best_value;
 }
 
+int
+int_log2(uint32_t value) {
+  uint32_t highest, bit, mask;
+
+  highest = 0;
+  for (mask = 1, bit = 0; 31 >= bit; mask <<= 1, ++bit)
+    if (value & mask)
+      highest = bit;
+
+  return highest;
+}
+
 bool
 parse_int(const char *s,
           int64_t &value) {
@@ -1046,6 +1082,7 @@ parse_int(const char *s,
   int64_t tmp;
   bool result;
 
+  tmp = 0;
   result = parse_int(s, tmp);
   value = tmp;
 
@@ -1081,7 +1118,7 @@ string
 to_string(int64_t i) {
   char buf[100];
 
-  sprintf(buf, "%lld", i);
+  sprintf(buf, "" LLD, i);
 
   return string(buf);
 }
@@ -1190,8 +1227,8 @@ debug_c::dump_info() {
   mxprint(stdout, "\nDBG> dumping time info:\n");
   for (i = 0; i < entries.size(); i++) {
     entry = entries[i];
-    mxprint(stdout, "DBG> function: %s, # calls: %llu, elapsed time: %.3fs, "
-            "time/call: %.3fms", entry->label, entry->number_of_calls,
+    mxprint(stdout, "DBG> function: %s, # calls: " LLU ", elapsed time: %.3fs,"
+            " time/call: %.3fms", entry->label, entry->number_of_calls,
             entry->elapsed_time / 1000000.0,
             entry->elapsed_time / (float)entry->number_of_calls / 1000.0);
     diff_calls = entry->number_of_calls - entry->last_number_of_calls;
@@ -1199,7 +1236,7 @@ debug_c::dump_info() {
     if ((entry->last_elapsed_time != 0) &&
         (entry->last_number_of_calls != 0) &&
         (diff_calls > 0)) {
-      mxprint(stdout, ", since the last call: # calls: %llu, elapsed time: "
+      mxprint(stdout, ", since the last call: # calls: " LLU ", elapsed time: "
               "%.3fs, time/call: %.3fms", diff_calls, diff_time / 1000000.0,
               diff_time / (float)diff_calls / 1000.0);
     }
@@ -1574,29 +1611,60 @@ set_tcp_error(const string &error) {
 
 bool
 parse_timecode(const string &src,
-               int64_t &timecode) {
+               int64_t &timecode,
+               bool allow_negative) {
   // Recognized format:
-  // 1. XXXXXXXs   with XXXXXX being a number of seconds followed
-  //    by the letter 's',
+  // 1. XXXXXXXu   with XXXXXX being a number followed
+  //    by one of the units 's', 'ms', 'us' or 'ns'
   // 2. HH:MM:SS.nnnnnnnnn  with up to nine digits 'n' for ns precision;
   // HH: is optional; HH, MM and SS can be either one or two digits.
+  // 2. HH:MM:SS:nnnnnnnnn  with up to nine digits 'n' for ns precision;
+  // HH: is optional; HH, MM and SS can be either one or two digits.
   int h, m, s, n, i, values[4], num_values, num_digits, num_colons;
+  int offset = 0, negative = 1;
   bool decimal_point_found;
 
   if (src.empty())
     return false;
 
-  // Number of seconds postfixed by 's'.
-  if ('s' == src[src.length() - 1]) {
-    string tmp = src;
+  if ('-' == src[0]) {
+    if (!allow_negative)
+      return false;
+    negative = -1;
+    offset = 1;
+  }
 
-    tmp.erase(tmp.length() - 1);
-    if (!parse_int(tmp, timecode))
-      return set_tcp_error("Non-digits found but expected a nunber of seconds "
-                           "before the 's' at the end");
+  try {
+    if (src.length() < (2 + offset))
+      throw false;
+
+    string unit = src.substr(src.length() - 2, 2);
+
+    int64_t multiplier = 1000000000;
+    int unit_length = 2;
+    int64_t value = 0;
+
+    if (unit == "ms")
+      multiplier = 1000000;
+    else if (unit == "us")
+      multiplier = 1000;
+    else if (unit == "ns")
+      multiplier = 1;
+    else if (unit.substr(1, 1) == "s")
+      unit_length = 1;
+    else
+      throw false;
+
+    if (src.length() < (unit_length + 1 + offset))
+      throw false;
+
+    if (!parse_int(src.substr(offset, src.length() - unit_length - offset), value))
+      throw false;
+
+    timecode = value * multiplier * negative;
 
-    timecode *= 1000000000ll;
     return true;
+  } catch (...) {
   }
 
   num_digits = 0;
@@ -1605,7 +1673,7 @@ parse_timecode(const string &src,
   decimal_point_found = false;
   memset(&values, 0, sizeof(int) * 4);
 
-  for (i = 0; src.length() > i; ++i) {
+  for (i = offset; src.length() > i; ++i) {
     if (isdigit(src[i])) {
       if (decimal_point_found && (9 == num_digits))
         return set_tcp_error("Invalid format: More than nine nano-second "
@@ -1613,6 +1681,18 @@ parse_timecode(const string &src,
       values[num_values - 1] = values[num_values - 1] * 10 + src[i] - '0';
       ++num_digits;
 
+    } else if (('.' == src[i]) ||
+               ((':' == src[i]) && (2 == num_colons))) {
+      if (decimal_point_found)
+        return set_tcp_error("Invalid format: Second decimal point after "
+                             "first decimal point");
+
+      if (0 == num_digits)
+        return set_tcp_error("Invalid format: No digits before decimal point");
+      ++num_values;
+      num_digits = 0;
+      decimal_point_found = true;
+
     } else if (':' == src[i]) {
       if (decimal_point_found)
         return set_tcp_error("Invalid format: Colon inside nano-second part");
@@ -1624,17 +1704,6 @@ parse_timecode(const string &src,
       ++num_values;
       num_digits = 0;
 
-    } else if ('.' == src[i]) {
-      if (decimal_point_found)
-        return set_tcp_error("Invalid format: Second decimal point after "
-                             "first decimal point");
-
-      if (0 == num_digits)
-        return set_tcp_error("Invalid format: No digits before decimal point");
-      ++num_values;
-      num_digits = 0;
-      decimal_point_found = true;
-
     } else
       return set_tcp_error(mxsprintf("Invalid format: unknown character '%c' "
                                      "found", src[i]));
@@ -1689,6 +1758,8 @@ parse_timecode(const string &src,
   timecode = ((int64_t)h * 60 * 60 + (int64_t)m * 60 + (int64_t)s) *
     1000000000ll + n;
 
+  timecode *= negative;
+
   timecode_parser_error = "no error";
   return true;
 }
@@ -1920,9 +1991,14 @@ usage(int exit_code) {
 }
 
 buffer_t::buffer_t():
-  buffer(NULL), size(0) {
+  m_buffer(NULL), m_size(0) {
+}
+
+buffer_t::buffer_t(unsigned char *buffer,
+                   int size):
+  m_buffer(buffer), m_size(size) {
 }
 
 buffer_t::~buffer_t() {
-  safefree(buffer);
+  safefree(m_buffer);
 }
