@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: common.h 3216 2006-04-28 14:52:55Z mosu $
+   $Id: common.h 3581 2007-08-19 09:24:10Z mosu $
 
    definitions used in all programs, helper functions
 
@@ -55,7 +55,7 @@ namespace libebml {
 
 using namespace libebml;
 
-#define VERSIONNAME "What Do You Take Me For"
+#define VERSIONNAME "Another Place To Fall"
 #define VERSIONINFO "mkvmerge v" VERSION " ('" VERSIONNAME "')"
 #define BUGMSG _("This should not have happened. Please contact the author " \
                  "Moritz Bunkus <moritz@bunkus.org> with this error/warning " \
@@ -70,6 +70,9 @@ using namespace std;
 #define MXMSG_INFO    15
 #define MXMSG_DEBUG   20
 
+#define MXMIN(a, b) ((a) < (b) ? (a) : (b))
+#define MXMAX(a, b) ((a) > (b) ? (a) : (b))
+
 #define FOURCC(a, b, c, d) (uint32_t)((((unsigned char)a) << 24) + \
                            (((unsigned char)b) << 16) + \
                            (((unsigned char)c) << 8) + \
@@ -94,7 +97,8 @@ using namespace std;
                             (int32_t)((t) % 1000000000)
 #define ARG_TIMECODEN(t) ARG_TIMECODENINT((int64_t)(t))
 extern string MTX_DLL_API timecode_parser_error;
-extern bool MTX_DLL_API parse_timecode(const string &s, int64_t &timecode);
+extern bool MTX_DLL_API parse_timecode(const string &s, int64_t &timecode,
+                                       bool allow_negative = false);
 
 extern bool MTX_DLL_API suppress_warnings;
 void MTX_DLL_API fix_format(const char *fmt, string &new_fmt);
@@ -226,7 +230,7 @@ void *MTX_DLL_API _saferealloc(void *mem, size_t size, const char *file,
 void MTX_DLL_API dump_malloc_report();
 
 vector<string> MTX_DLL_API split(const char *src, const char *pattern = ",",
-                              int max_num = -1);
+                                 int max_num = -1);
 inline vector<string>
 split(const string &src,
       const string &pattern = string(","),
@@ -278,6 +282,11 @@ parse_int(const string &s,
 }
 string MTX_DLL_API to_string(int64_t i);
 bool MTX_DLL_API parse_double(const char *s, double &value);
+inline bool
+parse_double(const string &s,
+             double &value) {
+  return parse_double(s.c_str(), value);
+}
 
 int MTX_DLL_API get_arg_len(const char *fmt, ...);
 int MTX_DLL_API get_varg_len(const char *fmt, va_list ap);
@@ -286,10 +295,10 @@ int MTX_DLL_API int_log2(uint32_t value);
 
 extern int MTX_DLL_API verbose;
 
-#define foreach(it, vec) for (it = (vec).begin(); it != (vec).end(); it++)
+#define mxforeach(it, vec) for (it = (vec).begin(); it != (vec).end(); it++)
 #define mxfind(value, cont) std::find(cont.begin(), cont.end(), value)
 #define mxfind2(it, value, cont) \
-  ((id = std::find((cont).begin(), (cont).end(), value)) != (cont).end())
+  ((it = std::find((cont).begin(), (cont).end(), value)) != (cont).end())
 #define map_has_key(m, k) ((m).end() != (m).find(k))
 
 class MTX_DLL_API bitvalue_c {
