@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: compression.h 2859 2005-03-19 11:03:12Z mosu $
+   $Id: compression.h 3155 2005-12-02 21:39:26Z mosu $
 
    Helper routines for various compression libs
 
@@ -20,9 +20,12 @@
 
 #include <memory>
 
+#include <matroska/KaxContentEncoding.h>
 #include <matroska/KaxTracks.h>
 
 #include "common.h"
+#include "common_memory.h"
+#include "error.h"
 #include "smart_pointers.h"
 
 using namespace libmatroska;
@@ -33,12 +36,19 @@ enum compression_method_e {
   COMPRESSION_ZLIB,
   COMPRESSION_BZ2,
   COMPRESSION_LZO,
+  COMPRESSION_HEADER_REMOVAL,
+  COMPRESSION_MPEG4_P2,
   COMPRESSION_NONE,
   COMPRESSION_NUM = COMPRESSION_NONE
 };
 
 extern const char *MTX_DLL_API xcompression_methods[];
 
+class MTX_DLL_API compression_error_c: public error_c {
+public:
+  compression_error_c(const string &error): error_c(error) { }
+};
+
 class compressor_c;
 typedef counted_ptr<compressor_c> compressor_ptr;
 
@@ -58,20 +68,25 @@ public:
     return method;
   };
 
-  virtual unsigned char *decompress(unsigned char *buffer, int &size) {
-    return (unsigned char *)safememdup(buffer, size);
+  virtual void decompress(memory_cptr &buffer) {
   };
 
-  virtual unsigned char *compress(unsigned char *buffer, int &size) {
-    return (unsigned char *)safememdup(buffer, size);
+  virtual void compress(memory_cptr &buffer) {
   };
 
+  virtual void set_track_headers(KaxContentEncoding &c_encoding);
+
   static compressor_ptr create(compression_method_e method);
   static compressor_ptr create(const char *method);
 };
 
-#if defined(HAVE_LZO1X_H)
-#include <lzo1x.h>
+#ifdef HAVE_LZO
+
+#ifdef HAVE_LZO_LZO1X_H
+# include <lzo/lzo1x.h>
+#else
+# include <lzo1x.h>
+#endif
 
 class MTX_DLL_API lzo_compressor_c: public compressor_c {
 protected:
@@ -81,10 +96,10 @@ public:
   lzo_compressor_c();
   virtual ~lzo_compressor_c();
 
-  virtual unsigned char *decompress(unsigned char *buffer, int &size);
-  virtual unsigned char *compress(unsigned char *buffer, int &size);
+  virtual void decompress(memory_cptr &buffer);
+  virtual void compress(memory_cptr &buffer);
 };
-#endif // HAVE_LZO1X_H
+#endif // HAVE_LZO
 
 #if defined(HAVE_ZLIB_H)
 #include <zlib.h>
@@ -94,8 +109,8 @@ public:
   zlib_compressor_c();
   virtual ~zlib_compressor_c();
 
-  virtual unsigned char *decompress(unsigned char *buffer, int &size);
-  virtual unsigned char *compress(unsigned char *buffer, int &size);
+  virtual void decompress(memory_cptr &buffer);
+  virtual void compress(memory_cptr &buffer);
 };
 #endif // HAVE_ZLIB_H
 
@@ -107,30 +122,56 @@ public:
   bzlib_compressor_c();
   virtual ~bzlib_compressor_c();
 
-  virtual unsigned char *decompress(unsigned char *buffer, int &size);
-  virtual unsigned char *compress(unsigned char *buffer, int &size);
+  virtual void decompress(memory_cptr &buffer);
+  virtual void compress(memory_cptr &buffer);
 };
 #endif // HAVE_BZLIB_H
 
-struct kax_content_encoding_t {
-  uint32_t order, type, scope;
-  uint32_t comp_algo;
-  unsigned char *comp_settings;
-  uint32_t comp_settings_len;
-  uint32_t enc_algo, sig_algo, sig_hash_algo;
-  unsigned char *enc_keyid, *sig_keyid, *signature;
-  uint32_t enc_keyid_len, sig_keyid_len, signature_len;
+class MTX_DLL_API header_removal_compressor_c: public compressor_c {
+protected:
+  memory_cptr m_bytes;
+
+public:
+  header_removal_compressor_c();
+
+  virtual void set_bytes(memory_cptr &bytes) {
+    m_bytes = bytes;
+    m_bytes->grab();
+  }
+
+  virtual void decompress(memory_cptr &buffer);
+  virtual void compress(memory_cptr &buffer);
+
+  virtual void set_track_headers(KaxContentEncoding &c_encoding);
 };
 
+class MTX_DLL_API mpeg4_p2_compressor_c: public header_removal_compressor_c {
+public:
+  mpeg4_p2_compressor_c();
+};
+
+// ------------------------------------------------------------------
+
 enum content_encoding_scope_e {
   CONTENT_ENCODING_SCOPE_BLOCK = 1,
   CONTENT_ENCODING_SCOPE_CODECPRIVATE = 2
 };
 
+struct kax_content_encoding_t {
+  uint32_t order, type, scope;
+  uint32_t comp_algo;
+  memory_cptr comp_settings;
+  uint32_t enc_algo, sig_algo, sig_hash_algo;
+  memory_cptr enc_keyid, sig_keyid, signature;
+
+  counted_ptr<compressor_c> compressor;
+
+  kax_content_encoding_t();
+};
+
 class MTX_DLL_API content_decoder_c {
 protected:
   vector<kax_content_encoding_t> encodings;
-  auto_ptr<compressor_c> zlib_compressor, bzlib_compressor, lzo1x_compressor;
   bool ok;
 
 public:
@@ -139,8 +180,7 @@ public:
   ~content_decoder_c();
 
   bool initialize(KaxTrackEntry &ktentry);
-  bool reverse(unsigned char *&data, uint32_t &size,
-               content_encoding_scope_e scope);
+  void reverse(memory_cptr &data, content_encoding_scope_e scope);
   bool is_ok() {
     return ok;
   }
