@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: hacks.cpp 3414 2007-01-03 10:23:22Z mosu $
+   $Id: hacks.cpp 3511 2007-03-05 19:18:24Z mosu $
 
    hacks :)
 
@@ -37,9 +37,12 @@ static const char *mosu_hacks[] = {
   ENGAGE_NO_DEFAULT_HEADER_VALUES,
   ENGAGE_FORCE_PASSTHROUGH_PACKETIZER,
   ENGAGE_WRITE_HEADERS_TWICE,
+  ENGAGE_ALLOW_AVC_IN_VFW_MODE,
   ENGAGE_KEEP_BITSTREAM_AR_INFO,
   ENGAGE_USE_SIMPLE_BLOCK,
   ENGAGE_OLD_AAC_CODECID,
+  ENGAGE_USE_CODEC_STATE,
+  ENGAGE_ENABLE_TIMECODE_WARNING,
   NULL
 };
 static vector<string> engaged_hacks;
