@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: hacks.h 3414 2007-01-03 10:23:22Z mosu $
+   $Id: hacks.h 3511 2007-03-05 19:18:24Z mosu $
 
    some hacks that the author might want to use
 
@@ -34,9 +34,12 @@ using namespace std;
 #define ENGAGE_NO_DEFAULT_HEADER_VALUES "no_default_header_values"
 #define ENGAGE_FORCE_PASSTHROUGH_PACKETIZER "force_passthrough_packetizer"
 #define ENGAGE_WRITE_HEADERS_TWICE "write_headers_twice"
+#define ENGAGE_ALLOW_AVC_IN_VFW_MODE "allow_avc_in_vfw_mode"
 #define ENGAGE_KEEP_BITSTREAM_AR_INFO "keep_bitstream_ar_info"
 #define ENGAGE_USE_SIMPLE_BLOCK "use_simpleblock"
 #define ENGAGE_OLD_AAC_CODECID "old_aac_codecid"
+#define ENGAGE_USE_CODEC_STATE "use_codec_state"
+#define ENGAGE_ENABLE_TIMECODE_WARNING "enable_timecode_warning"
 
 void MTX_DLL_API engage_hacks(const string &hacks);
 bool MTX_DLL_API hack_engaged(const string &hack);
