@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: matroska.h 2711 2005-01-12 15:03:47Z mosu $
+   $Id: matroska.h 3503 2007-03-02 11:17:25Z mosu $
 
    Definitions for the various Codec IDs
 
@@ -28,7 +28,9 @@
 #define MKV_A_AAC_4SSR   "A_AAC/MPEG4/SSR"
 #define MKV_A_AAC_4LTP   "A_AAC/MPEG4/LTP"
 #define MKV_A_AAC_4SBR   "A_AAC/MPEG4/LC/SBR"
+#define MKV_A_AAC        "A_AAC"
 #define MKV_A_AC3        "A_AC3"
+#define MKV_A_EAC3       "A_EAC3"
 #define MKV_A_DTS        "A_DTS"
 #define MKV_A_MP3        "A_MPEG/L3"
 #define MKV_A_PCM        "A_PCM/INT/LIT"
@@ -46,18 +48,20 @@
 #define MKV_A_TTA        "A_TTA1"
 #define MKV_A_WAVPACK4   "A_WAVPACK4"
 
-#define MKV_V_MPEG1      "V_MPEG1"
-#define MKV_V_MPEG2      "V_MPEG2"
-#define MKV_V_MPEG4_SP   "V_MPEG4/ISO/SP"
-#define MKV_V_MPEG4_ASP  "V_MPEG4/ISO/ASP"
-#define MKV_V_MPEG4_AP   "V_MPEG4/ISO/AP"
-#define MKV_V_MPEG4_AVC  "V_MPEG4/ISO/AVC"
-#define MKV_V_MSCOMP     "V_MS/VFW/FOURCC"
-#define MKV_V_REALV1     "V_REAL/RV10"
-#define MKV_V_REALV2     "V_REAL/RV20"
-#define MKV_V_REALV3     "V_REAL/RV30"
-#define MKV_V_REALV4     "V_REAL/RV40"
-#define MKV_V_QUICKTIME  "V_QUICKTIME"
+#define MKV_V_MPEG1       "V_MPEG1"
+#define MKV_V_MPEG2       "V_MPEG2"
+#define MKV_V_MPEG4_SP    "V_MPEG4/ISO/SP"
+#define MKV_V_MPEG4_ASP   "V_MPEG4/ISO/ASP"
+#define MKV_V_MPEG4_AP    "V_MPEG4/ISO/AP"
+#define MKV_V_MPEG4_AVC   "V_MPEG4/ISO/AVC"
+#define MKV_V_MSCOMP      "V_MS/VFW/FOURCC"
+#define MKV_V_REALV1      "V_REAL/RV10"
+#define MKV_V_REALV2      "V_REAL/RV20"
+#define MKV_V_REALV3      "V_REAL/RV30"
+#define MKV_V_REALV4      "V_REAL/RV40"
+#define MKV_V_QUICKTIME   "V_QUICKTIME"
+#define MKV_V_THEORA      "V_THEORA"
+#define MKV_V_COREPICTURE "V_CIPC"
 
 #define MKV_S_TEXTUTF8   "S_TEXT/UTF8"
 #define MKV_S_TEXTSSA    "S_TEXT/SSA"
