@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mm_io.cpp 3029 2005-08-17 15:28:47Z mosu $
+   $Id: mm_io.cpp 3571 2007-07-10 20:13:14Z mosu $
 
    IO callback class implementation
 
@@ -27,8 +27,11 @@
 #include <stdlib.h>
 #if defined(SYS_WINDOWS)
 # include <windows.h>
+# include <direct.h>
 #else
-#include <unistd.h>
+# include <unistd.h>
+# include <sys/stat.h>
+# include <sys/types.h>
 #endif // SYS_WINDOWS
 
 #include "error.h"
@@ -39,6 +42,11 @@ using namespace std;
 
 #if !defined(SYS_WINDOWS)
 
+static string
+get_errno_msg() {
+  return to_utf8(cc_local_utf8, strerror(errno));
+}
+
 # if HAVE_POSIX_FADVISE
 static const unsigned long read_using_willneed = 16 * 1024 * 1024;
 static const unsigned long write_before_dontneed = 8 * 1024 * 1024;
@@ -56,6 +64,7 @@ mm_file_io_c::mm_file_io_c(const string &path,
 
   string local_path;
   char *cmode;
+  struct stat st;
 # if HAVE_POSIX_FADVISE
   int advise;
 
@@ -64,7 +73,7 @@ mm_file_io_c::mm_file_io_c(const string &path,
   write_count = 0;
   use_posix_fadvise_here = use_posix_fadvise;
 # endif
-  
+
   switch (mode) {
     case MODE_READ:
       cmode = "rb";
@@ -94,7 +103,13 @@ mm_file_io_c::mm_file_io_c(const string &path,
       throw mm_io_error_c("Unknown open mode");
   }
 
+  if ((MODE_WRITE == mode) || (MODE_CREATE == mode))
+    prepare_path(path);
   local_path = from_utf8(cc_local_utf8, path);
+
+  if ((0 == stat(local_path.c_str(), &st)) && S_ISDIR(st.st_mode))
+    throw mm_io_open_error_c();
+
   file = (FILE *)fopen(local_path.c_str(), cmode);
 
   if (file == NULL)
@@ -104,7 +119,7 @@ mm_file_io_c::mm_file_io_c(const string &path,
   if (use_posix_fadvise && use_posix_fadvise_here &&
       (0 != posix_fadvise(fileno((FILE *)file), 0, read_using_willneed,
                           advise))) {
-    mxverb(2, FADVISE_WARNING, path.c_str(), errno, strerror(errno));
+    mxverb(2, FADVISE_WARNING, path.c_str(), errno, get_errno_msg().c_str());
     use_posix_fadvise_here = false;
   }
 # endif
@@ -140,8 +155,8 @@ mm_file_io_c::write(const void *buffer,
 
   bwritten = fwrite(buffer, 1, size, (FILE *)file);
   if (ferror((FILE *)file) != 0)
-    mxerror("Cound not write to the output file: %d (%s)\n", errno,
-            strerror(errno));
+    mxerror("Could not write to the output file: %d (%s)\n", errno,
+            get_errno_msg().c_str());
 
 # if HAVE_POSIX_FADVISE
   write_count += bwritten;
@@ -151,7 +166,8 @@ mm_file_io_c::write(const void *buffer,
     write_count = 0;
     if (0 != posix_fadvise(fileno((FILE *)file), 0, pos,
                            POSIX_FADV_DONTNEED)) {
-      mxverb(2, FADVISE_WARNING, file_name.c_str(), errno, strerror(errno));
+      mxverb(2, FADVISE_WARNING, file_name.c_str(), errno,
+             get_errno_msg().c_str());
       use_posix_fadvise_here = false;
     }
   }
@@ -167,7 +183,7 @@ mm_file_io_c::read(void *buffer,
                    size_t size) {
   int64_t bread;
 
-  bread = fread(buffer, 1, size, (FILE *)file); 
+  bread = fread(buffer, 1, size, (FILE *)file);
 
 # if HAVE_POSIX_FADVISE
   if (use_posix_fadvise && use_posix_fadvise_here && (0 <= bread)) {
@@ -177,13 +193,15 @@ mm_file_io_c::read(void *buffer,
       int fd = fileno((FILE *)file);
       read_count = 0;
       if (0 != posix_fadvise(fd, 0, pos, POSIX_FADV_DONTNEED)) {
-        mxverb(2, FADVISE_WARNING, file_name.c_str(), errno, strerror(errno));
+        mxverb(2, FADVISE_WARNING, file_name.c_str(), errno,
+               get_errno_msg().c_str());
         use_posix_fadvise_here = false;
       }
-      if (use_posix_fadvise_here && 
+      if (use_posix_fadvise_here &&
           (0 != posix_fadvise(fd, pos, pos + read_using_willneed,
                               POSIX_FADV_WILLNEED))) {
-        mxverb(2, FADVISE_WARNING, file_name.c_str(), errno, strerror(errno));
+        mxverb(2, FADVISE_WARNING, file_name.c_str(), errno,
+               get_errno_msg().c_str());
         use_posix_fadvise_here = false;
       }
     }
@@ -278,6 +296,8 @@ mm_file_io_c::mm_file_io_c(const string &path,
       throw mm_io_error_c("Unknown open mode");
   }
 
+  if ((MODE_WRITE == mode) || (MODE_CREATE == mode))
+    prepare_path(path);
   file = (void *)CreateFileUtf8(path.c_str(), access_mode, share_mode, NULL,
                                 disposition, 0, NULL);
   _eof = false;
@@ -370,6 +390,7 @@ mm_file_io_c::write(const void *buffer,
   if (bytes_written != size) {
     DWORD error;
     char *error_msg;
+    string error_msg_utf8;
 
     error = GetLastError();
     error_msg = NULL;
@@ -388,9 +409,11 @@ mm_file_io_c::write(const void *buffer,
         error_msg[idx] = 0;
         idx--;
       }
-    }
-    mxerror("Cound not write to the output file: %d (%s)\n", (int)error,
-            error_msg != NULL ? error_msg : "unknown");
+      error_msg_utf8 = to_utf8(cc_local_utf8, error_msg);
+    } else
+      error_msg_utf8 = "unknown";
+    mxerror("Could not write to the output file: %d (%s)\n", (int)error,
+            error_msg_utf8.c_str());
     if (error_msg != NULL)
       LocalFree(error_msg);
   }
@@ -427,6 +450,42 @@ mm_file_io_c::setup() {
 
 #endif // SYS_UNIX
 
+void
+mm_file_io_c::prepare_path(const string &path) {
+  string local_path = path; // contains copy of given path
+
+#if defined(SYS_WINDOWS)
+  bool first_part = true;
+  const string SEPARATOR ("\\");
+  // convert separators for current OS
+  std::replace(local_path.begin(), local_path.end(), '/', '\\');
+#else  // SYS_WINDOWS
+  const string SEPARATOR ("/");
+  // convert separators for current OS
+  std::replace(local_path.begin(), local_path.end(), '\\', '/');
+#endif // SYS_WINDOWS
+
+  // current position:
+  string::size_type position = local_path.find_first_of(SEPARATOR, 0);
+
+  while (position != string::npos) {
+    string subpath = local_path.substr(0, position);
+
+    if ((subpath.size() != 0) && !fs_entry_exists(subpath.c_str())
+#if defined(SYS_WINDOWS)
+        && (!first_part || (subpath.find_first_of(':') == string::npos))
+#endif // SYS_WINDOWS
+        )
+      create_directory(subpath.c_str());
+
+    position = local_path.find_first_of(SEPARATOR, position + 1);
+
+#if defined(SYS_WINDOWS)
+    first_part = false;
+#endif // SYS_WINDOWS
+  }
+}
+
 uint64
 mm_file_io_c::getFilePointer() {
   return m_current_position;
@@ -609,6 +668,20 @@ mm_io_c::read_uint64_be() {
   return get_uint64_be(buffer);
 }
 
+uint32_t
+mm_io_c::read(memory_cptr &buffer,
+              size_t size,
+              int offset) {
+  if (-1 == offset)
+    offset = buffer->get_size();
+  if (buffer->get_size() <= (size + offset))
+    buffer->resize(size + offset);
+  if (read(buffer->get() + offset, size) != size)
+    throw mm_io_eof_error_c();
+  buffer->set_size(size + offset);
+  return size;
+}
+
 int
 mm_io_c::write_uint8(unsigned char value) {
   return write(&value, 1);
@@ -662,6 +735,17 @@ mm_io_c::write_uint64_be(uint64_t value) {
   return write(&buffer, sizeof(uint64_t));
 }
 
+uint32_t
+mm_io_c::write(const memory_cptr &buffer,
+               int size,
+               int offset) {
+  if (-1 == size)
+    size = buffer->get_size();
+  if (write(buffer->get() + offset, size) != size)
+    throw mm_io_eof_error_c();
+  return size;
+}
+
 void
 mm_io_c::skip(int64 num_bytes) {
   int64_t pos;
@@ -968,6 +1052,12 @@ mm_mem_io_c::eof() {
   return pos >= mem_size;
 }
 
+unsigned char *
+mm_mem_io_c::get_and_lock_buffer() {
+  free_mem = false;
+  return mem;
+}
+
 /*
    Class for handling UTF-8/UTF-16/UTF-32 text files.
 */
