@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_base.cpp 3000 2005-08-02 08:20:01Z mosu $
+   $Id: xtr_base.cpp 3577 2007-08-15 19:43:36Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -27,6 +27,8 @@
 #include "xtr_avc.h"
 #include "xtr_avi.h"
 #include "xtr_base.h"
+#include "xtr_cpic.h"
+#include "xtr_mpeg1_2.h"
 #include "xtr_ogg.h"
 #include "xtr_rmff.h"
 #include "xtr_textsubs.h"
@@ -39,9 +41,12 @@ using namespace libmatroska;
 
 xtr_base_c::xtr_base_c(const string &_codec_id,
                        int64_t _tid,
-                       track_spec_t &tspec):
-  codec_id(_codec_id), file_name(tspec.out_name), master(NULL), out(NULL),
-  tid(_tid), default_duration(0), bytes_written(0) {
+                       track_spec_t &tspec,
+                       const char *_container_name):
+  codec_id(_codec_id), file_name(tspec.out_name),
+  container_name(NULL == _container_name ? "raw data" : _container_name),
+  master(NULL), out(NULL), tid(_tid), default_duration(0), bytes_written(0),
+  content_decoder_initialized(false) {
 }
 
 xtr_base_c::~xtr_base_c() {
@@ -53,12 +58,13 @@ void
 xtr_base_c::create_file(xtr_base_c *_master,
                         KaxTrackEntry &track) {
   if (NULL != _master)
-    mxerror("Cannot write track %lld with the CodecID '%s' to the file '%s' "
-            "because track %lld with the CodecID '%s' is already being "
-            "written to the same file.\n", tid, codec_id.c_str(),
+    mxerror("Cannot write track " LLD " with the CodecID '%s' to the file "
+            "'%s' because track " LLD " with the CodecID '%s' is already "
+            "being written to the same file.\n", tid, codec_id.c_str(),
             file_name.c_str(), _master->tid, _master->codec_id.c_str());
 
   try {
+    init_content_decoder(track);
     out = new mm_file_io_c(file_name, MODE_CREATE);
   } catch(...) {
     mxerror("Failed to create the file '%s': %d (%s)\n", file_name.c_str(),
@@ -69,19 +75,18 @@ xtr_base_c::create_file(xtr_base_c *_master,
 }
 
 void
-xtr_base_c::handle_block(KaxBlock &block,
+xtr_base_c::handle_frame(memory_cptr &frame,
                          KaxBlockAdditions *additions,
                          int64_t timecode,
                          int64_t duration,
                          int64_t bref,
-                         int64_t fref) {
-  int i;
-
-  for (i = 0; i < block.NumberFrames(); i++) {
-    DataBuffer &data = block.GetBuffer(i);
-    out->write(data.Buffer(), data.Size());
-    bytes_written += data.Size();
-  }
+                         int64_t fref,
+                         bool keyframe,
+                         bool discardable,
+                         bool references_valid) {
+  content_decoder.reverse(frame, CONTENT_ENCODING_SCOPE_BLOCK);
+  out->write(frame->get(), frame->get_size());
+  bytes_written += frame->get_size();
 }
 
 void
@@ -96,6 +101,26 @@ void
 xtr_base_c::headers_done() {
 }
 
+memory_cptr
+xtr_base_c::decode_codec_private(KaxCodecPrivate *priv) {
+  memory_cptr mpriv(new memory_c(priv->GetBuffer(), priv->GetSize()));
+  content_decoder.reverse(mpriv, CONTENT_ENCODING_SCOPE_CODECPRIVATE);
+
+  return mpriv;
+}
+
+void
+xtr_base_c::init_content_decoder(KaxTrackEntry &track) {
+  if (content_decoder_initialized)
+    return;
+
+  if (!content_decoder.initialize(track))
+    mxerror("Tracks with unsupported content encoding schemes (compression "
+            "or encryption) cannot be extracted.\n");
+
+  content_decoder_initialized = true;
+}
+
 xtr_base_c *
 xtr_base_c::create_extractor(const string &new_codec_id,
                              int64_t new_tid,
@@ -106,10 +131,16 @@ xtr_base_c::create_extractor(const string &new_codec_id,
   else if (2 == tspec.extract_raw)
     return new xtr_fullraw_c(new_codec_id, new_tid, tspec);
   // Audio formats
-  else if ((new_codec_id == MKV_A_AC3) ||
-           starts_with_case(new_codec_id, "A_MPEG/L") ||
-           (new_codec_id == MKV_A_DTS))
-    return new xtr_base_c(new_codec_id, new_tid, tspec);
+  else if (new_codec_id == MKV_A_AC3)
+    return new xtr_base_c(new_codec_id, new_tid, tspec, "Dolby Digital (AC3)");
+  else if (new_codec_id == MKV_A_EAC3)
+    return new xtr_base_c(new_codec_id, new_tid, tspec, "Dolby Digital Plus (EAC3)");
+  else if (starts_with_case(new_codec_id, "A_MPEG/L"))
+    return new xtr_base_c(new_codec_id, new_tid, tspec,
+                          "MPEG-1 Audio Layer 2/3");
+  else if (new_codec_id == MKV_A_DTS)
+    return new xtr_base_c(new_codec_id, new_tid, tspec,
+                          "Digital Theater System (DTS)");
   else if (new_codec_id == MKV_A_PCM)
     return new xtr_wav_c(new_codec_id, new_tid, tspec);
   else if (new_codec_id == MKV_A_FLAC) {
@@ -119,7 +150,7 @@ xtr_base_c::create_extractor(const string &new_codec_id,
       return new xtr_flac_c(new_codec_id, new_tid, tspec);
   } else if (new_codec_id == MKV_A_VORBIS)
     return new xtr_oggvorbis_c(new_codec_id, new_tid, tspec);
-  else if (starts_with_case(new_codec_id, "A_AAC/"))
+  else if (starts_with_case(new_codec_id, "A_AAC"))
     return new xtr_aac_c(new_codec_id, new_tid, tspec);
   else if (starts_with_case(new_codec_id, "A_REAL/"))
     return new xtr_rmff_c(new_codec_id, new_tid, tspec);
@@ -135,6 +166,9 @@ xtr_base_c::create_extractor(const string &new_codec_id,
     return new xtr_avc_c(new_codec_id, new_tid, tspec);
   else if (starts_with_case(new_codec_id, "V_REAL/"))
     return new xtr_rmff_c(new_codec_id, new_tid, tspec);
+  else if ((new_codec_id == MKV_V_MPEG1) ||
+           (new_codec_id == MKV_V_MPEG2))
+    return new xtr_mpeg1_2_video_c(new_codec_id, new_tid, tspec);
 
   // Subtitle formats
   else if ((new_codec_id == MKV_S_TEXTUTF8) ||
@@ -149,6 +183,8 @@ xtr_base_c::create_extractor(const string &new_codec_id,
     return new xtr_vobsub_c(new_codec_id, new_tid, tspec);
   else if (new_codec_id == MKV_S_TEXTUSF)
     return new xtr_usf_c(new_codec_id, new_tid, tspec);
+  else if (new_codec_id == MKV_V_COREPICTURE)
+    return new xtr_cpic_c(new_codec_id, new_tid, tspec);
 
   return NULL;
 }
@@ -162,6 +198,15 @@ xtr_fullraw_c::create_file(xtr_base_c *_master,
 
   priv = FINDFIRST(&track, KaxCodecPrivate);
 
-  if ((NULL != priv) && (0 != priv->GetSize()))
-    out->write(priv->GetBuffer(), priv->GetSize());
+  if ((NULL != priv) && (0 != priv->GetSize())) {
+    memory_cptr mem(new memory_c(priv->GetBuffer(), priv->GetSize(), false));
+    content_decoder.reverse(mem, CONTENT_ENCODING_SCOPE_CODECPRIVATE);
+    out->write(mem->get(), mem->get_size());
+  }
+}
+
+void
+xtr_fullraw_c::handle_codec_state(memory_cptr &codec_state) {
+  content_decoder.reverse(codec_state, CONTENT_ENCODING_SCOPE_CODECPRIVATE);
+  out->write(codec_state->get(), codec_state->get_size());
 }
