@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_ogg.cpp 2903 2005-05-02 13:50:23Z mosu $
+   $Id: xtr_ogg.cpp 3532 2007-06-23 09:16:13Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -35,12 +35,14 @@ xtr_flac_c::create_file(xtr_base_c *_master,
 
   priv = FINDFIRST(&track, KaxCodecPrivate);
   if (NULL == priv)
-    mxerror("Track %lld with the CodecID '%s' is missing the \"codec private"
-            "\" element and cannot be extracted.\n", tid, codec_id.c_str());
+    mxerror("Track " LLD " with the CodecID '%s' is missing the \"codec "
+            "private\" element and cannot be extracted.\n", tid,
+            codec_id.c_str());
 
   xtr_base_c::create_file(_master, track);
 
-  out->write(priv->GetBuffer(), priv->GetSize());
+  memory_cptr mpriv = decode_codec_private(priv);
+  out->write(mpriv->get(), mpriv->get_size());
 }
 
 // ------------------------------------------------------------------------
@@ -49,7 +51,7 @@ xtr_oggbase_c::xtr_oggbase_c(const string &_codec_id,
                              int64_t _tid,
                              track_spec_t &tspec):
   xtr_base_c(_codec_id, _tid, tspec),
-  packetno(2), buffered_data(NULL), buffered_size(0), previous_end(0) {
+  packetno(2), previous_end(0) {
 }
 
 void
@@ -66,40 +68,33 @@ xtr_oggbase_c::create_file(xtr_base_c *_master,
 }
 
 void
-xtr_oggbase_c::handle_block(KaxBlock &block,
+xtr_oggbase_c::handle_frame(memory_cptr &frame,
                             KaxBlockAdditions *additions,
                             int64_t timecode,
                             int64_t duration,
                             int64_t bref,
-                            int64_t fref) {
-  int i;
+                            int64_t fref,
+                            bool keyframe,
+                            bool discardable,
+                            bool references_valid) {
+  content_decoder.reverse(frame, CONTENT_ENCODING_SCOPE_BLOCK);
 
-  if (-1 == duration)
-    duration = default_duration * block.NumberFrames();
+  if (NULL != buffered_data.get()) {
+    ogg_packet op;
 
-  for (i = 0; i < block.NumberFrames(); i++) {
-    DataBuffer &data = block.GetBuffer(i);
-    if (NULL != buffered_data) {
-      ogg_packet op;
+    op.b_o_s = 0;
+    op.e_o_s = 0;
+    op.packetno = packetno;
+    op.packet = buffered_data->get();
+    op.bytes = buffered_data->get_size();
+    op.granulepos = timecode * sfreq / 1000000000;
+    ogg_stream_packetin(&os, &op);
+    write_pages();
 
-      op.b_o_s = 0;
-      op.e_o_s = 0;
-      op.packetno = packetno;
-      op.packet = buffered_data;
-      op.bytes = buffered_size;
-//       op.granulepos = (timecode / 1000000) * sfreq / 1000;
-      op.granulepos = (timecode + i * duration / block.NumberFrames()) *
-        sfreq / 1000000000;
-      ogg_stream_packetin(&os, &op);
-      write_pages();
-      safefree(buffered_data);
-
-      packetno++;
-    }
-
-    buffered_data = (unsigned char *)safememdup(data.Buffer(), data.Size());
-    buffered_size = data.Size();
+    packetno++;
   }
+
+  buffered_data = memory_cptr(frame->clone());
   previous_end = timecode + duration;
 }
 
@@ -107,7 +102,7 @@ void
 xtr_oggbase_c::finish_file() {
   ogg_packet op;
 
-  if (NULL == buffered_data)
+  if (NULL == buffered_data.get())
     return;
 
   // Set the "end of stream" marker on the last packet, handle it
@@ -115,12 +110,11 @@ xtr_oggbase_c::finish_file() {
   op.b_o_s = 0;
   op.e_o_s = 1;
   op.packetno = packetno;
-  op.packet = buffered_data;
-  op.bytes = buffered_size;
+  op.packet = buffered_data->get();
+  op.bytes = buffered_data->get_size();
 //   op.granulepos = (previous_end / 1000000) * sfreq / 1000;
   op.granulepos = previous_end * sfreq / 1000000000;
   ogg_stream_packetin(&os, &op);
-  safefree(buffered_data);
   flush_pages();
   ogg_stream_clear(&os);
 }
@@ -163,13 +157,16 @@ xtr_oggflac_c::create_file(xtr_base_c *_master,
 
   priv = FINDFIRST(&track, KaxCodecPrivate);
   if (NULL == priv)
-    mxerror("Track %lld with the CodecID '%s' is missing the \"codec private"
-            "\" element and cannot be extracted.\n", tid, codec_id.c_str());
+    mxerror("Track " LLD " with the CodecID '%s' is missing the \"codec "
+            "private\" element and cannot be extracted.\n", tid,
+            codec_id.c_str());
 
   sfreq = (int)kt_get_a_sfreq(track);
 
   xtr_base_c::create_file(_master, track);
 
+  memory_cptr mpriv = decode_codec_private(priv);
+
   if (no_variable_data)
     ogg_stream_init(&os, 1804289383);
   else
@@ -187,13 +184,13 @@ xtr_oggflac_c::create_file(xtr_base_c *_master,
   flush_pages();
   op.b_o_s = 0;
   op.packetno = 1;
-  ptr = priv->GetBuffer();
-  if ((priv->GetSize() >= 4) && (ptr[0] == 'f') &&
+  ptr = mpriv->get();
+  if ((mpriv->get_size() >= 4) && (ptr[0] == 'f') &&
       (ptr[1] == 'L') && (ptr[2] == 'a') && (ptr[3] == 'C')) {
     ptr += 4;
-    op.bytes = priv->GetSize() - 4;
+    op.bytes = mpriv->get_size() - 4;
   } else
-    op.bytes = priv->GetSize();
+    op.bytes = mpriv->get_size();
   op.packet = (unsigned char *)safememdup(ptr, op.bytes);
   ogg_stream_packetin(&os, &op);
   safefree(op.packet);
@@ -219,24 +216,28 @@ xtr_oggvorbis_c::create_file(xtr_base_c *_master,
 
   priv = FINDFIRST(&track, KaxCodecPrivate);
   if (NULL == priv)
-    mxerror("Track %lld with the CodecID '%s' is missing the \"codec private"
-            "\" element and cannot be extracted.\n", tid, codec_id.c_str());
+    mxerror("Track " LLD " with the CodecID '%s' is missing the \"codec "
+            "private\" element and cannot be extracted.\n", tid,
+            codec_id.c_str());
 
-  c = (const unsigned char *)priv->GetBuffer();
-  if ((priv->GetSize() == 0) || (c[0] != 2))
-    mxerror("Track %lld with the CodecID '%s' does not contain valid headers."
-            "\n", tid, codec_id.c_str());
+  init_content_decoder(track);
+  memory_cptr mpriv = decode_codec_private(priv);
+
+  c = (const unsigned char *)mpriv->get();
+  if ((mpriv->get_size() == 0) || (c[0] != 2))
+    mxerror("Track " LLD " with the CodecID '%s' does not contain valid "
+            "headers.\n", tid, codec_id.c_str());
 
   offset = 1;
   for (packetno = 0; packetno < 2; packetno++) {
     int length = 0;
 
-    while ((offset < priv->GetSize()) && ((unsigned char)255 == c[offset])) {
+    while ((offset < mpriv->get_size()) && ((unsigned char)255 == c[offset])) {
       length += 255;
       offset++;
     }
-    if ((priv->GetSize() - 1) <= offset)
-      mxerror("Track %lld with the CodecID '%s' does not contain valid "
+    if ((mpriv->get_size() - 1) <= offset)
+      mxerror("Track " LLD " with the CodecID '%s' does not contain valid "
               "headers.\n", tid, codec_id.c_str());
     length += c[offset];
     offset++;
@@ -246,7 +247,7 @@ xtr_oggvorbis_c::create_file(xtr_base_c *_master,
   headers[0] = &c[offset];
   headers[1] = &c[offset + header_sizes[0]];
   headers[2] = &c[offset + header_sizes[0] + header_sizes[1]];
-  header_sizes[2] = priv->GetSize() - offset - header_sizes[0] -
+  header_sizes[2] = mpriv->get_size() - offset - header_sizes[0] -
     header_sizes[1];
 
   xtr_oggbase_c::create_file(_master, track);
