@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_tta.cpp 2791 2005-02-11 23:29:39Z mosu $
+   $Id: xtr_tta.cpp 3532 2007-06-23 09:16:13Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -14,6 +14,8 @@
 
 #include "os.h"
 
+#include <time.h>
+
 #include "checksums.h"
 #include "commonebml.h"
 #include "tta_common.h"
@@ -28,7 +30,7 @@ xtr_tta_c::xtr_tta_c(const string &_codec_id,
   xtr_base_c(_codec_id, _tid, tspec),
   last_duration(0), bps(0), channels(0), sfreq(0) {
 
-  temp_file_name = mxsprintf("mkvextract-%lld-temp-tta-%u", tid,
+  temp_file_name = mxsprintf("mkvextract-" LLD "-temp-tta-%u", tid,
                              (uint32_t)time(NULL));
 }
 
@@ -48,20 +50,19 @@ xtr_tta_c::create_file(xtr_base_c *_master,
 }
 
 void
-xtr_tta_c::handle_block(KaxBlock &block,
+xtr_tta_c::handle_frame(memory_cptr &frame,
                         KaxBlockAdditions *additions,
                         int64_t timecode,
                         int64_t duration,
                         int64_t bref,
-                        int64_t fref) {
-  int i;
+                        int64_t fref,
+                        bool keyframe,
+                        bool discardable,
+                        bool references_valid) {
+  content_decoder.reverse(frame, CONTENT_ENCODING_SCOPE_BLOCK);
 
-  for (i = 0; i < block.NumberFrames(); i++) {
-    DataBuffer &data = block.GetBuffer(i);
-
-    frame_sizes.push_back(data.Size());
-    out->write(data.Buffer(), data.Size());
-  }
+  frame_sizes.push_back(frame->get_size());
+  out->write(frame->get(), frame->get_size());
 
   if (0 < duration)
     last_duration = duration;
@@ -123,7 +124,7 @@ xtr_tta_c::finish_file() {
   out->write_uint32_le(crc);
   safefree(buffer);
 
-  mxinfo("\nThe temporary TTA file for track ID %lld is being "
+  mxinfo("\nThe temporary TTA file for track ID " LLD " is being "
          "copied into the final TTA file. This may take some time.\n", tid);
 
   buffer = (unsigned char *)safemalloc(128000);
