@@ -6,13 +6,15 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_ac3.cpp 2932 2005-05-25 21:31:26Z mosu $
+   $Id: r_ac3.cpp 3503 2007-03-02 11:17:25Z mosu $
 
    AC3 demultiplexer module
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
+#include "os.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -71,8 +73,8 @@ ac3_reader_c::create_packetizer(int64_t) {
     return;
   add_packetizer(new ac3_packetizer_c(this, ac3header.sample_rate,
                                       ac3header.channels, ac3header.bsid, ti));
-  mxinfo(FMT_TID "Using the AC3 output module.\n", ti.fname.c_str(),
-         (int64_t)0);
+  mxinfo(FMT_TID "Using the %sAC3 output module.\n", ti.fname.c_str(),
+         (int64_t)0, 16 == ac3header.bsid ? "E" : "");
 }
 
 file_status_e
@@ -99,8 +101,8 @@ ac3_reader_c::get_progress() {
 
 void
 ac3_reader_c::identify() {
-  mxinfo("File '%s': container: AC3\nTrack ID 0: audio (AC3)\n",
-         ti.fname.c_str());
+  mxinfo("File '%s': container: AC3\nTrack ID 0: audio (%sAC3)\n",
+         ti.fname.c_str(), 16 == ac3header.bsid ? "E" : "");
 }
 
 int
