@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_avc.cpp 3448 2007-01-08 20:36:27Z mosu $
+   $Id: r_avc.cpp 3534 2007-06-23 16:05:01Z mosu $
 
    AVC/h2.64 ES demultiplexer module
 
@@ -28,6 +28,7 @@
 #include "byte_buffer.h"
 #include "common.h"
 #include "error.h"
+#include "output_control.h"
 #include "r_avc.h"
 #include "p_avc.h"
 
@@ -49,6 +50,7 @@ avc_es_reader_c::probe_file(mm_io_c *io,
     memory_cptr buf(new memory_c((unsigned char *)safemalloc(READ_SIZE),
                                  READ_SIZE));
     int num_read, i;
+    bool first = true;
 
     avc_es_parser_c parser;
     parser.ignore_nalu_size_length_errors();
@@ -61,6 +63,11 @@ avc_es_reader_c::probe_file(mm_io_c *io,
       if (num_read < 4)
         return 0;
 
+      // MPEG TS starts with 0x47.
+      if (first && (0x47 == buf->get()[0]))
+        return 0;
+      first = false;
+
       parser.add_bytes(buf->get(), num_read);
 
       if (parser.headers_parsed())
@@ -89,6 +96,10 @@ avc_es_reader_c::avc_es_reader_c(track_info_c &n_ti)
     avc_es_parser_c parser;
     parser.ignore_nalu_size_length_errors();
     parser.enable_timecode_generation(40000000);
+    if (map_has_key(ti.nalu_size_lengths, 0))
+      parser.set_nalu_size_length(ti.nalu_size_lengths[0]);
+    else if (map_has_key(ti.nalu_size_lengths, -1))
+      parser.set_nalu_size_length(ti.nalu_size_lengths[-1]);
 
     int num_read, i;
 
@@ -145,7 +156,7 @@ avc_es_reader_c::read(generic_packetizer_c *,
 
   m_bytes_processed += num_read;
 
-  if (READ_SIZE != num_read)
+  if ((READ_SIZE != num_read) || (m_bytes_processed >= m_size))
     PTZR0->flush();
 
   return READ_SIZE == num_read ? FILE_STATUS_MOREDATA : FILE_STATUS_DONE;
@@ -158,7 +169,12 @@ avc_es_reader_c::get_progress() {
 
 void
 avc_es_reader_c::identify() {
+  string info;
+
+  if (identify_verbose)
+    info = " [packetizer:mpeg4_p10_es_video]";
   mxinfo("File '%s': container: AVC/h.264 elementary stream (ES)\n"
-         "Track ID 0: video (MPEG-4 part 10 ES)\n", ti.fname.c_str());
+         "Track ID 0: video (MPEG-4 part 10 ES)%s\n", ti.fname.c_str(),
+         info.c_str());
 }
 
