@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_avi.cpp 2967 2005-06-26 10:04:02Z mosu $
+   $Id: r_avi.cpp 3534 2007-06-23 16:05:01Z mosu $
 
    AVI demultiplexer module
 
@@ -31,18 +31,19 @@ extern "C" {
 #include "aac_common.h"
 #include "common.h"
 #include "error.h"
+#include "hacks.h"
 #include "mpeg4_common.h"
 #include "output_control.h"
 #include "r_avi.h"
 #include "p_aac.h"
 #include "p_ac3.h"
+#include "p_avc.h"
 #include "p_dts.h"
 #include "p_mp3.h"
 #include "p_pcm.h"
 #include "p_video.h"
 
 #define PFX "avi_reader: "
-#define MIN_CHUNK_SIZE 128000
 
 // }}}
 
@@ -75,15 +76,21 @@ avi_reader_c::probe_file(mm_io_c *io,
 
 avi_reader_c::avi_reader_c(track_info_c &_ti)
   throw (error_c):
-  generic_reader_c(_ti) {
-  long fsize, i;
+  generic_reader_c(_ti),
+  divx_type(DIVX_TYPE_NONE),
+  avi(NULL), vptzr(-1),
+  fps(1.0), video_frames_read(0), max_video_frames(0), dropped_video_frames(0),
+  act_wchar(0), rederive_keyframes(false),
+  bytes_to_process(0), bytes_processed(0) {
+
   int64_t size;
 
   try {
-    io = new mm_file_io_c(ti.fname);
-    size = io->get_size();
-    if (!avi_reader_c::probe_file(io, size))
+    mm_file_io_c io(ti.fname);
+    size = io.get_size();
+    if (!avi_reader_c::probe_file(&io, size))
       throw error_c(PFX "Source is not a valid AVI file.");
+
   } catch (...) {
     throw error_c(PFX "Could not read the source file.");
   }
@@ -93,36 +100,13 @@ avi_reader_c::avi_reader_c(track_info_c &_ti)
            "may take some time depending on the file's size.\n",
            ti.fname.c_str());
 
-  fsize = 0;
-  rederive_keyframes = 0;
-  vptzr = -1;
-
-  frames = 0;
-  delete io;
   if ((avi = AVI_open_input_file(ti.fname.c_str(), 1)) == NULL) {
     const char *msg = PFX "Could not initialize AVI source. Reason: ";
     throw error_c(mxsprintf("%s%s", msg, AVI_strerror()));
   }
 
   fps = AVI_frame_rate(avi);
-  maxframes = AVI_video_frames(avi);
-  for (i = 0; i < maxframes; i++)
-    if (AVI_frame_size(avi, i) > fsize)
-      fsize = AVI_frame_size(avi, i);
-  max_frame_size = fsize;
-
-  if (video_fps < 0)
-    video_fps = fps;
-  chunk_size = max_frame_size < MIN_CHUNK_SIZE ? MIN_CHUNK_SIZE :
-    max_frame_size;
-  chunk = (unsigned char *)safemalloc(chunk_size);
-  act_wchar = 0;
-  old_key = 0;
-  old_chunk = NULL;
-  video_done = 0;
-  dropped_frames = 0;
-  bytes_to_process = 0;
-  bytes_processed = 0;
+  max_video_frames = AVI_video_frames(avi);
 }
 
 // }}}
@@ -133,11 +117,9 @@ avi_reader_c::~avi_reader_c() {
   if (avi != NULL)
     AVI_close(avi);
 
-  safefree(chunk);
-  safefree(old_chunk);
   ti.private_data = NULL;
 
-  mxverb(2, "avi_reader_c: Dropped video frames: %d\n", dropped_frames);
+  mxverb(2, "avi_reader_c: Dropped video frames: %d\n", dropped_video_frames);
 }
 
 // }}}
@@ -147,28 +129,38 @@ avi_reader_c::create_packetizer(int64_t tid) {
   char *codec;
 
   if ((tid == 0) && demuxing_requested('v', 0) && (vptzr == -1)) {
-    int i, maxframes;
+    int i;
 
-    maxframes = AVI_video_frames(avi);
-    for (i = 0; i < maxframes; i++)
+    mxverb(4, FMT_TID "frame sizes:\n", ti.fname.c_str(), (int64_t)0);
+    for (i = 0; i < max_video_frames; i++) {
       bytes_to_process += AVI_frame_size(avi, i);
+      mxverb(4, "  %d: %ld\n", i, AVI_frame_size(avi, i));
+    }
 
     codec = AVI_video_compressor(avi);
     if (!strcasecmp(codec, "DIV3") ||
         !strcasecmp(codec, "AP41") || // Angel Potion
         !strcasecmp(codec, "MPG3") ||
         !strcasecmp(codec, "MP43"))
-      is_divx = RAVI_DIVX3;
-    else if (is_mpeg4_p2_fourcc(codec))
-      is_divx = RAVI_MPEG4;
-    else
-      is_divx = 0;
+      divx_type = DIVX_TYPE_V3;
+    else if (mpeg4::p2::is_fourcc(codec))
+      divx_type = DIVX_TYPE_MPEG4;
 
     ti.private_data = (unsigned char *)avi->bitmap_info_header;
     if (ti.private_data != NULL)
       ti.private_size = get_uint32_le(&avi->bitmap_info_header->bi_size);
+
+    mxverb(4, "track extra data size: %d\n",
+           (int)(ti.private_size - sizeof(alBITMAPINFOHEADER)));
+    if (sizeof(alBITMAPINFOHEADER) < ti.private_size) {
+      mxverb(4, "  ");
+      for (i = sizeof(alBITMAPINFOHEADER); i < ti.private_size; ++i)
+        mxverb(4, "%02x ", ti.private_data[i]);
+      mxverb(4, "\n");
+    }
+
     ti.id = 0;                 // ID for the video track.
-    if (is_divx == RAVI_MPEG4) {
+    if (divx_type == DIVX_TYPE_MPEG4) {
       vptzr =
         add_packetizer(new mpeg4_p2_video_packetizer_c(this,
                                                        AVI_frame_rate(avi),
@@ -179,6 +171,26 @@ avi_reader_c::create_packetizer(int64_t tid) {
       if (verbose)
         mxinfo(FMT_TID "Using the MPEG-4 part 2 video output module.\n",
                ti.fname.c_str(), (int64_t)0);
+    } else if (mpeg4::p10::is_avc_fourcc(codec) &&
+               !hack_engaged(ENGAGE_ALLOW_AVC_IN_VFW_MODE)) {
+      try {
+        memory_cptr avcc = extract_avcc();
+        mpeg4_p10_es_video_packetizer_c *ptzr =
+          new mpeg4_p10_es_video_packetizer_c(this, avcc, AVI_video_width(avi),
+                                              AVI_video_height(avi), ti);
+        vptzr = add_packetizer(ptzr);
+        ptzr->enable_timecode_generation(false);
+        ptzr->set_track_default_duration((int64_t)(1000000000 /
+                                                   AVI_frame_rate(avi)));
+        if (verbose)
+          mxinfo(FMT_TID "Using the MPEG-4 part 10 ES video output module.\n",
+                 ti.fname.c_str(), (int64_t)0);
+
+      } catch (...) {
+        mxerror(FMT_TID "Could not extract the decoder specific config data "
+                "(AVCC) from this AVC/h.264 track.\n",
+                ti.fname.c_str(), (int64_t)0);
+      }
     } else {
       vptzr = add_packetizer(new video_packetizer_c(this, NULL,
                                                     AVI_frame_rate(avi),
@@ -199,22 +211,46 @@ avi_reader_c::create_packetizer(int64_t tid) {
 
 void
 avi_reader_c::create_packetizers() {
-  uint32_t i, bps;
-  vector<avi_demuxer_t>::const_iterator demuxer;
+  int i;
 
   create_packetizer(0);
 
   for (i = 0; i < AVI_audio_tracks(avi); i++)
     create_packetizer(i + 1);
+}
 
-  foreach(demuxer, ademuxers) {
-    bps = demuxer->samples_per_second * demuxer->channels *
-      demuxer->bits_per_sample / 8;
-    if (bps > chunk_size) {
-      chunk_size = bps;
-      chunk = (unsigned char *)saferealloc(chunk, chunk_size);
+memory_cptr
+avi_reader_c::extract_avcc() {
+  avc_es_parser_c parser;
+  int i, size, key;
+
+  parser.ignore_nalu_size_length_errors();
+  if (map_has_key(ti.nalu_size_lengths, 0))
+    parser.set_nalu_size_length(ti.nalu_size_lengths[0]);
+  else if (map_has_key(ti.nalu_size_lengths, -1))
+    parser.set_nalu_size_length(ti.nalu_size_lengths[-1]);
+
+  for (i = 0; i < max_video_frames; ++i) {
+    size = AVI_frame_size(avi, i);
+    if (0 == size)
+      continue;
+
+    memory_cptr buffer(new memory_c((unsigned char *)safemalloc(size),
+                                    size, true));
+    AVI_set_video_position(avi, i);
+    size = AVI_read_frame(avi, (char *)buffer->get(), &key);
+    if (0 < size) {
+      parser.add_bytes(buffer->get(), size);
+      if (parser.headers_parsed()) {
+        AVI_set_video_position(avi, 0);
+        return parser.get_avcc();
+      }
     }
   }
+
+  AVI_set_video_position(avi, 0);
+
+  throw false;
 }
 
 // {{{ FUNCTION avi_reader_c::add_audio_demuxer
@@ -228,7 +264,7 @@ avi_reader_c::add_audio_demuxer(int aid) {
   uint32_t audio_format;
   int maxchunks, i;
 
-  foreach(it, ademuxers)
+  mxforeach(it, ademuxers)
     if (it->aid == aid) // Demuxer already added?
       return;
 
@@ -352,11 +388,11 @@ avi_reader_c::is_keyframe(unsigned char *data,
   if (!rederive_keyframes)
     return suggestion;
 
-  switch (is_divx) {
-    case RAVI_DIVX3:
+  switch (divx_type) {
+    case DIVX_TYPE_V3:
       i = *((int *)data);
       return ((i & 0x40000000) ? 0 : 1);
-    case RAVI_MPEG4:
+    case DIVX_TYPE_MPEG4:
       for (i = 0; i < size - 5; i++) {
         if ((data[i] == 0x00) && (data[i + 1] == 0x00) &&
             (data[i + 2] == 0x01)) {
@@ -382,161 +418,125 @@ avi_reader_c::is_keyframe(unsigned char *data,
 
 // {{{ FUNCTION avi_reader_c::read
 
+file_status_e
+avi_reader_c::read_video() {
+  unsigned char *chunk;
+  int size, key, nread, i;
+  bool need_more_data;
+  int64_t timestamp, duration;
+
+  if (video_frames_read >= max_video_frames)
+    return FILE_STATUS_DONE;
+
+  need_more_data = false;
+
+  chunk = NULL;
+  key = 0;
+
+  do {
+    safefree(chunk);
+    size = AVI_frame_size(avi, video_frames_read);
+    chunk = (unsigned char *)safemalloc(size);
+    nread = AVI_read_frame(avi, (char *)chunk, &key);
+
+    ++video_frames_read;
+
+    if (0 > nread) {
+      // Error reading the frame: abort
+      video_frames_read = max_video_frames;
+      safefree(chunk);
+      return FILE_STATUS_DONE;
+
+    } else if (0 == nread)
+      ++dropped_video_frames;
+
+  } while ((0 == nread) && (video_frames_read < max_video_frames));
+
+
+  if (0 == nread) {
+    // This is only the case if the AVI contains dropped frames only.
+    safefree(chunk);
+    return FILE_STATUS_DONE;
+  }
+
+  for (i = video_frames_read; i < max_video_frames; ++i) {
+    int dummy_key;
+
+    if (0 != AVI_frame_size(avi, i))
+      break;
+
+    AVI_read_frame(avi, NULL, &dummy_key);
+  }
+
+  timestamp = (int64_t)(((int64_t)video_frames_read - 1) * 1000000000ll / fps);
+  duration = (int64_t)
+    (((int64_t)i - video_frames_read + 1) * 1000000000ll / fps);
+
+  dropped_video_frames += i - video_frames_read;
+  video_frames_read = i;
+
+  PTZR(vptzr)->process(new packet_t(new memory_c(chunk, nread, true),
+                                    timestamp, duration,
+                                    key ? VFT_IFRAME : VFT_PFRAMEAUTOMATIC,
+                                    VFT_NOBFRAME));
+  bytes_processed += nread;
+
+  if (video_frames_read >= max_video_frames) {
+    PTZR(vptzr)->flush();
+    return FILE_STATUS_DONE;
+  }
+
+  return FILE_STATUS_MOREDATA;
+}
+
+file_status_e
+avi_reader_c::read_audio(avi_demuxer_t &demuxer) {
+  unsigned char *chunk;
+  bool need_more_data;
+  int size;
+
+  AVI_set_audio_track(avi, demuxer.aid);
+  size = AVI_read_audio_chunk(avi, NULL);
+
+  if (0 >= size) {
+    PTZR(demuxer.ptzr)->flush();
+    return FILE_STATUS_DONE;
+  }
+
+  chunk = (unsigned char *)safemalloc(size);
+  size = AVI_read_audio_chunk(avi, (char *)chunk);
+
+  if (0 >= size) {
+    safefree(chunk);
+    PTZR(demuxer.ptzr)->flush();
+    return FILE_STATUS_DONE;
+  }
+
+  need_more_data = 0 != AVI_read_audio_chunk(avi, NULL);
+
+  PTZR(demuxer.ptzr)->add_avi_block_size(size);
+  PTZR(demuxer.ptzr)->process(new packet_t(new memory_c(chunk, size, true)));
+  bytes_processed += size;
+
+  if (need_more_data)
+    return FILE_STATUS_MOREDATA;
+  else {
+    PTZR(demuxer.ptzr)->flush();
+    return FILE_STATUS_DONE;
+  }
+}
+
 file_status_e
 avi_reader_c::read(generic_packetizer_c *ptzr,
                    bool force) {
-  vector<avi_demuxer_t>::const_iterator demuxer;
-  int key, last_frame, frames_read, size;
-  long nread;
-  bool need_more_data, done;
-  int64_t duration;
+  vector<avi_demuxer_t>::iterator demuxer;
 
-  key = 0;
-  nread = 0;
-  if ((vptzr != -1) && !video_done && (PTZR(vptzr) == ptzr)) {
-    debug_enter("avi_reader_c::read (video)");
+  if ((vptzr != -1) && (PTZR(vptzr) == ptzr))
+    return read_video();
 
-    need_more_data = false;
-    last_frame = 0;
-    done = false;
-
-    // Make sure we have a frame to work with.
-    while (NULL == old_chunk) {
-      debug_enter("AVI_read_frame");
-      nread = AVI_read_frame(avi, (char *)chunk, &key);
-      debug_leave("AVI_read_frame");
-      if (nread < 0) {
-        frames = maxframes + 1;
-        done = true;
-        break;
-      } else if (nread > 0) {
-        old_chunk = chunk;
-        chunk = (unsigned char *)safemalloc(chunk_size);
-        old_key = key;
-        old_nread = nread;
-        frames++;
-      } else {
-        ++frames;
-        ++dropped_frames;
-      }
-    }
-    if (!done) {
-      frames_read = 1;
-      // Check whether we have identical frames
-      while (!done && (frames <= (maxframes - 1))) {
-        debug_enter("AVI_read_frame");
-        nread = AVI_read_frame(avi, (char *)chunk, &key);
-        debug_leave("AVI_read_frame");
-        if (nread < 0) {
-          PTZR(vptzr)->process(new packet_t(new memory_c(old_chunk, old_nread,
-                                                         true),
-                                            -1, -1,
-                                            old_key ? VFT_IFRAME :
-                                            VFT_PFRAMEAUTOMATIC,
-                                            VFT_NOBFRAME));
-          bytes_processed += old_nread;
-          old_chunk = NULL;
-          mxwarn(PFX "Reading frame number %d resulted in an error. "
-                 "Aborting this track.\n", frames);
-          frames = maxframes + 1;
-          break;
-        }
-        if (frames == (maxframes - 1)) {
-          last_frame = 1;
-          done = true;
-        }
-        if (nread == 0) {
-          frames_read++;
-          dropped_frames++;
-        }
-        else if (nread > 0)
-          done = true;
-        frames++;
-      }
-      duration = (int64_t)(1000000000.0 * frames_read / fps);
-      if (nread > 0) {
-        PTZR(vptzr)->process(new packet_t(new memory_c(old_chunk, old_nread,
-                                                       true),
-                                          -1, duration,
-                                          old_key ? VFT_IFRAME :
-                                          VFT_PFRAMEAUTOMATIC,
-                                          VFT_NOBFRAME));
-        bytes_processed += old_nread;
-        old_chunk = NULL;
-        if (! last_frame) {
-          if (old_chunk != NULL)
-            safefree(old_chunk);
-          old_chunk = chunk;
-          chunk = (unsigned char *)safemalloc(chunk_size);
-          old_key = key;
-          old_nread = nread;
-        } else if (nread > 0) {
-          PTZR(vptzr)->process(new packet_t(new memory_c(chunk, nread, false),
-                                            -1, duration,
-                                            key ? VFT_IFRAME :
-                                            VFT_PFRAMEAUTOMATIC,
-                                            VFT_NOBFRAME));
-          bytes_processed += nread;
-        }
-      }
-    }
-
-    if (last_frame) {
-      frames = maxframes + 1;
-      video_done = 1;
-    } else if (frames != (maxframes + 1))
-      need_more_data = true;
-
-    debug_leave("avi_reader_c::read (video)");
-
-    if (need_more_data)
-      return FILE_STATUS_MOREDATA;
-    else {
-      PTZR(vptzr)->flush();
-      return FILE_STATUS_DONE;
-    }
-  }
-
-  foreach(demuxer, ademuxers) {
-    unsigned char *audio_chunk;
-
-    if (PTZR(demuxer->ptzr) != ptzr)
-      continue;
-
-    debug_enter("avi_reader_c::read (audio)");
-
-    need_more_data = false;
-
-    AVI_set_audio_track(avi, demuxer->aid);
-    size = AVI_read_audio_chunk(avi, NULL);
-    if (size > 0) {
-      audio_chunk = (unsigned char *)safemalloc(size);
-
-      debug_enter("AVI_read_audio");
-      nread = AVI_read_audio_chunk(avi, (char *)audio_chunk);
-      debug_leave("AVI_read_audio");
-
-      if (nread > 0) {
-        size = AVI_read_audio_chunk(avi, NULL);
-        if (size > 0)
-          need_more_data = true;
-        PTZR(demuxer->ptzr)->add_avi_block_size(nread);
-        PTZR(demuxer->ptzr)->process(new packet_t(new memory_c(audio_chunk,
-                                                               nread, true)));
-        bytes_processed += nread;
-      } else
-        safefree(audio_chunk);
-    }
-
-    debug_leave("avi_reader_c::read (audio)");
-
-    if (need_more_data)
-      return FILE_STATUS_MOREDATA;
-    else {
-      PTZR(demuxer->ptzr)->flush();
-      return FILE_STATUS_DONE;
-    }
-  }
+  mxforeach(demuxer, ademuxers)
+    if ((-1 != demuxer->ptzr) && (PTZR(demuxer->ptzr) == ptzr))
+      return read_audio(*demuxer);
 
   return FILE_STATUS_DONE;
 }
@@ -557,49 +557,63 @@ avi_reader_c::identify() {
   int i;
   const char *type;
   uint32_t par_num, par_den;
-  bool extended_info_shown;
+  vector<string> extended_info;
 
   mxinfo("File '%s': container: AVI\n", ti.fname.c_str());
-  extended_info_shown = false;
   type = AVI_video_compressor(avi);
-  if (!strncasecmp(type, "MP42", 4) ||
-      !strncasecmp(type, "DIV2", 4) ||
-      !strncasecmp(type, "DIVX", 4) ||
-      !strncasecmp(type, "XVID", 4) ||
-      !strncasecmp(type, "DX50", 4)) {
+
+  if (identify_verbose &&
+      (!strncasecmp(type, "MP42", 4) ||
+       !strncasecmp(type, "DIV2", 4) ||
+       !strncasecmp(type, "DIVX", 4) ||
+       !strncasecmp(type, "XVID", 4) ||
+       !strncasecmp(type, "DX50", 4))) {
     unsigned char *buffer;
     uint32_t width, height, disp_width, disp_height;
     float aspect_ratio;
     int size, key;
-    string extended_info;
 
     size = AVI_frame_size(avi, 0);
     if (size > 0) {
       buffer = (unsigned char *)safemalloc(size);
       AVI_read_frame(avi, (char *)buffer, &key);
-      if (mpeg4_p2_extract_par(buffer, size, par_num, par_den)) {
-        width = AVI_video_width(avi);
-        height = AVI_video_height(avi);
-        aspect_ratio = (float)width / (float)height * (float)par_num /
-          (float)par_den;
+
+      if (mpeg4::p2::extract_par(buffer, size, par_num, par_den)) {
+        width        = AVI_video_width(avi);
+        height       = AVI_video_height(avi);
+        aspect_ratio = (float)width / (float)height * (float)par_num / (float)par_den;
+
         if (aspect_ratio > ((float)width / (float)height)) {
-          disp_width = irnd(height * aspect_ratio);
+          disp_width  = irnd(height * aspect_ratio);
           disp_height = height;
+
         } else {
-          disp_width = width;
+          disp_width  = width;
           disp_height = irnd(width / aspect_ratio);
         }
-        if (identify_verbose)
-          extended_info = mxsprintf(" [display_dimensions:%ux%u ]",
-                                    disp_width, disp_height);
-        mxinfo("Track ID 0: video (%s)%s\n", type, extended_info.c_str());
-        extended_info_shown = true;
+
+        extended_info.push_back(mxsprintf("display_dimensions:%ux%u", disp_width, disp_height));
       }
       safefree(buffer);
     }
   }
-  if (!extended_info_shown)
-    mxinfo("Track ID 0: video (%s)\n", AVI_video_compressor(avi));
+
+  if (mpeg4::p10::is_avc_fourcc(type))
+    extended_info.push_back("packetizer:mpeg4_p10_es_video");
+
+  mxinfo("Track ID 0: video (%s)", type);
+
+  if (identify_verbose && !extended_info.empty()) {
+    vector<string>::iterator ei;
+
+    mxinfo("[");
+    mxforeach (ei, extended_info)
+      mxinfo("%s ", ei->c_str());
+    mxinfo("]");
+  }
+
+  mxinfo("\n");
+
   for (i = 0; i < AVI_audio_tracks(avi); i++) {
     AVI_set_audio_track(avi, i);
     switch (AVI_audio_format(avi)) {
@@ -622,7 +636,7 @@ avi_reader_c::identify() {
         type = "AAC";
         break;
       default:
-        type = "unknown";
+        type = "unsupported";
     }
     mxinfo("Track ID %d: audio (%s)\n", i + 1, type);
   }
