@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_avi.h 2809 2005-02-24 21:05:35Z mosu $
+   $Id: r_avi.h 3409 2007-01-02 12:17:12Z mosu $
 
    class definitions for the AVI demultiplexer module
 
@@ -31,10 +31,6 @@ extern "C" {
 #include "common.h"
 #include "error.h"
 
-#define RAVI_UNKNOWN 0
-#define RAVI_DIVX3   1
-#define RAVI_MPEG4   2
-
 typedef struct avi_demuxer_t {
   int ptzr;
   int channels, bits_per_sample, samples_per_second, aid;
@@ -43,14 +39,18 @@ typedef struct avi_demuxer_t {
 
 class avi_reader_c: public generic_reader_c {
 private:
-  unsigned char *chunk, *old_chunk;
+  enum divx_type_e {
+    DIVX_TYPE_NONE,
+    DIVX_TYPE_V3,
+    DIVX_TYPE_MPEG4
+  } divx_type;
+
   avi_t *avi;
-  mm_io_c *io;
   int vptzr;
   vector<avi_demuxer_t> ademuxers;
   double fps;
-  int frames, max_frame_size, act_wchar, old_key, old_nread, dropped_frames;
-  int video_done, maxframes, is_divx, rederive_keyframes, chunk_size;
+  int video_frames_read, max_video_frames, dropped_video_frames, act_wchar;
+  bool is_divx, rederive_keyframes;
 
   int64_t bytes_to_process, bytes_processed;
 
@@ -70,6 +70,9 @@ public:
 protected:
   virtual void add_audio_demuxer(int aid);
   virtual int is_keyframe(unsigned char *data, long size, int suggestion);
+  virtual file_status_e read_video();
+  virtual file_status_e read_audio(avi_demuxer_t &demuxer);
+  virtual memory_cptr extract_avcc();
 };
 
 #endif  // __R_AVI_H
