@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_mpeg.cpp 2951 2005-06-10 07:12:50Z mosu $
+   $Id: r_mpeg.cpp 3572 2007-07-10 20:18:46Z mosu $
 
    MPEG ES demultiplexer module
 
@@ -27,7 +27,9 @@
 #include "mp3_common.h"
 #include "r_mpeg.h"
 #include "smart_pointers.h"
+#include "output_control.h"
 #include "p_ac3.h"
+#include "p_avc.h"
 #include "p_dts.h"
 #include "p_mp3.h"
 #include "p_video.h"
@@ -82,7 +84,7 @@ mpeg_es_reader_c::probe_file(mm_io_c *io,
       return 0;
 
     // Let's try to read one frame.
-    if (!read_frame(parser, *io, READ_SIZE))
+    if (!read_frame(parser, *io, READ_SIZE, true))
       return 0;
 
   } catch (...) {
@@ -183,7 +185,8 @@ mpeg_es_reader_c::read(generic_packetizer_c *,
 bool
 mpeg_es_reader_c::read_frame(M2VParser &parser,
                              mm_io_c &in,
-                             int64_t max_size) {
+                             int64_t max_size,
+                             bool flush) {
   int bytes_probed;
 
   bytes_probed = 0;
@@ -210,6 +213,7 @@ mpeg_es_reader_c::read_frame(M2VParser &parser,
       bytes_probed += bytes_read;
 
       parser.WriteData(buffer, bytes_read);
+      parser.SetEOS();
       delete [] buffer;
 
     } else if (state == MPV_PARSER_STATE_FRAME)
@@ -236,14 +240,20 @@ mpeg_es_reader_c::identify() {
 
 // ------------------------------------------------------------------------
 
-#define PS_PROBE_SIZE 1024 * 1024
+#define PS_PROBE_SIZE 1 * 1024 * 1024
+
+static bool
+operator <(const mpeg_ps_track_ptr &a,
+           const mpeg_ps_track_ptr &b) {
+  return a->sort_key < b->sort_key;
+}
 
 int
 mpeg_ps_reader_c::probe_file(mm_io_c *io,
                              int64_t size) {
   try {
-    autofree_ptr<unsigned char> af_buf(safemalloc(PS_PROBE_SIZE));
-    unsigned char *buf = af_buf;
+    memory_c af_buf((unsigned char *)safemalloc(PS_PROBE_SIZE), 0, true);
+    unsigned char *buf = af_buf.get();
     int num_read;
 
     io->setFilePointer(0, seek_beginning);
@@ -265,20 +275,30 @@ mpeg_ps_reader_c::probe_file(mm_io_c *io,
 mpeg_ps_reader_c::mpeg_ps_reader_c(track_info_c &_ti)
   throw (error_c):
   generic_reader_c(_ti) {
+
+  int i;
+
+  try {
+    io = new mm_file_io_c(ti.fname);
+    size = io->get_size();
+    file_done = false;
+
+  } catch (...) {
+    throw error_c("mpeg_ps_reader: Could not open the file.");
+  }
+
   try {
     uint32_t header;
     uint8_t byte;
     bool done;
-    int i;
-
-    io = new mm_file_io_c(ti.fname);
-    size = io->get_size();
-    file_done = false;
 
     bytes_processed = 0;
 
     for (i = 0; i < 512; i++)
       id2idx[i] = -1;
+    memset(es_map, 0, sizeof(uint32_t) * NUM_ES_MAP_ENTRIES);
+    memset(blacklisted_ids, 0, sizeof(bool) * 512);
+
     header = io->read_uint32_be();
     done = io->eof();
     version = -1;
@@ -289,7 +309,7 @@ mpeg_ps_reader_c::mpeg_ps_reader_c(track_info_c &_ti)
 
       switch (header) {
         case MPEGVIDEO_PACKET_START_CODE:
-          mxverb(3, "mpeg_ps: packet start at %lld\n",
+          mxverb(3, "mpeg_ps: packet start at " LLD "\n",
                  io->getFilePointer() - 4);
 
           if (version == -1) {
@@ -311,7 +331,7 @@ mpeg_ps_reader_c::mpeg_ps_reader_c(track_info_c &_ti)
           break;
 
         case MPEGVIDEO_SYSTEM_HEADER_START_CODE:
-          mxverb(3, "mpeg_ps: system header start code at %lld\n",
+          mxverb(3, "mpeg_ps: system header start code at " LLD "\n",
                  io->getFilePointer() - 4);
 
           io->skip(2 * 4);   // system header
@@ -328,9 +348,13 @@ mpeg_ps_reader_c::mpeg_ps_reader_c(track_info_c &_ti)
           done = true;
           break;
 
+        case MPEGVIDEO_PROGRAM_STREAM_MAP_START_CODE:
+          parse_program_stream_map();
+          break;
+
         default:
           if (!mpeg_is_start_code(header)) {
-            mxverb(3, "mpeg_ps: unknown header 0x%08x at %lld\n",
+            mxverb(3, "mpeg_ps: unknown header 0x%08x at " LLD "\n",
                    header, io->getFilePointer() - 4);
             done = true;
             break;
@@ -341,7 +365,7 @@ mpeg_ps_reader_c::mpeg_ps_reader_c(track_info_c &_ti)
           found_new_stream(stream_id);
           io->restore_pos();
           pes_packet_length = io->read_uint16_be();
-          mxverb(3, "mpeg_ps: id 0x%02x len %u at %lld\n", stream_id,
+          mxverb(3, "mpeg_ps: id 0x%02x len %u at " LLD "\n", stream_id,
                  pes_packet_length, io->getFilePointer() - 4 - 2);
 
           io->skip(pes_packet_length);
@@ -354,41 +378,14 @@ mpeg_ps_reader_c::mpeg_ps_reader_c(track_info_c &_ti)
       done |= io->eof() || (io->getFilePointer() >= PS_PROBE_SIZE);
     } // while (!done)
 
-    mxverb(2, "mpeg_ps: Streams found: ");
-    for (i = 0; i < 256; i++)
-      if (id2idx[i] != -1)
-        mxverb(2, "%02x ", i);
-    for (i = 256 ; i < 512; i++)
-      if (id2idx[i] != -1)
-        mxverb(2, "bd(%02x) ", i - 256);
-    mxverb(2, "\n");
-
-    // Calculate by how much the timecodes have to be offset
-    if (tracks.size() > 0) {
-      int64_t min_timecode;
-
-      min_timecode = tracks[0]->timecode_offset;
-      for (i = 1; i < tracks.size(); i++)
-        if (tracks[i]->timecode_offset < min_timecode)
-          min_timecode = tracks[i]->timecode_offset;
-      for (i = 0; i < tracks.size(); i++)
-        tracks[i]->timecode_offset -= min_timecode;
-
-      mxverb(2, "mpeg_ps: Timecode offset: min was %lld ", min_timecode);
-      for (i = 0; i < tracks.size(); i++)
-        if (tracks[i]->id > 0xff)
-          mxverb(2, "bd(%02x)=%lld ", tracks[i]->id - 256,
-                 tracks[i]->timecode_offset);
-        else
-          mxverb(2, "%02x=%lld ", tracks[i]->id, tracks[i]->timecode_offset);
-      mxverb(2, "\n");
-    }
-
-    io->setFilePointer(0, seek_beginning);
-
   } catch (...) {
-    throw error_c("mpeg_ps_reader: Could not open the file.");
   }
+
+  sort_tracks();
+  calculate_global_timecode_offset();
+
+  io->setFilePointer(0, seek_beginning);
+
   if (verbose)
     mxinfo(FMT_FN "Using the MPEG PS demultiplexer.\n", ti.fname.c_str());
 }
@@ -397,6 +394,56 @@ mpeg_ps_reader_c::~mpeg_ps_reader_c() {
   delete io;
 }
 
+void
+mpeg_ps_reader_c::sort_tracks() {
+  int i;
+
+  for (i = 0; tracks.size() > i; ++i)
+    tracks[i]->sort_key =
+      ('v' == tracks[i]->type ? 0x00000 :
+       'a' == tracks[i]->type ? 0x10000 :
+       's' == tracks[i]->type ? 0x20000 : 0x30000) +
+      (256 > tracks[i]->id ? tracks[i]->id + 256 : tracks[i]->id);
+  sort(tracks.begin(), tracks.end());
+  for (i = 0; tracks.size() > i; ++i)
+    id2idx[tracks[i]->id] = i;
+
+  mxverb(2, "mpeg_ps: Supported streams, sorted by ID: ");
+  for (i = 0; tracks.size() > i; ++i)
+    if (256 > tracks[i]->id)
+      mxverb(2, "%02x ", tracks[i]->id);
+    else
+      mxverb(2, "bd(%02x) ", tracks[i]->id - 256);
+  mxverb(2, "\n");
+}
+
+void
+mpeg_ps_reader_c::calculate_global_timecode_offset() {
+  // Calculate by how much the timecodes have to be offset
+  if (tracks.empty())
+    return;
+
+  int64_t min_timecode;
+  int i;
+
+  min_timecode = tracks[0]->timecode_offset;
+  for (i = 1; i < tracks.size(); i++)
+    if (tracks[i]->timecode_offset < min_timecode)
+      min_timecode = tracks[i]->timecode_offset;
+  for (i = 0; i < tracks.size(); i++)
+    tracks[i]->timecode_offset -= min_timecode;
+
+  mxverb(2, "mpeg_ps: Timecode offset: min was " LLD " ", min_timecode);
+  for (i = 0; i < tracks.size(); i++)
+    if (tracks[i]->id > 0xff)
+      mxverb(2, "bd(%02x)=" LLD " ", tracks[i]->id - 256,
+             tracks[i]->timecode_offset);
+    else
+      mxverb(2, "%02x=" LLD " ", tracks[i]->id,
+             tracks[i]->timecode_offset);
+  mxverb(2, "\n");
+}
+
 bool
 mpeg_ps_reader_c::read_timestamp(int c,
                                  int64_t &timestamp) {
@@ -414,14 +461,85 @@ mpeg_ps_reader_c::read_timestamp(int c,
   return true;
 }
 
+void
+mpeg_ps_reader_c::parse_program_stream_map() {
+  int64_t pos;
+  int len = 0, prog_len, es_map_len, type, id, id_offset, plen;
+
+  pos = io->getFilePointer();
+  try {
+    len = io->read_uint16_be();
+
+    if (!len || (1018 < len))
+      throw false;
+
+    if (0x00 == (io->read_uint8() & 0x80))
+      throw false;
+
+    io->skip(1);
+
+    prog_len = io->read_uint16_be();
+    io->skip(prog_len);
+
+    es_map_len = io->read_uint16_be();
+    es_map_len = MXMIN(es_map_len, len - prog_len - 8);
+
+    while (0 < es_map_len) {
+      type = io->read_uint8();
+      id = io->read_uint8();
+
+      if ((0xb0 <= id) && (0xef >= id)) {
+        id_offset = id - 0xb0;
+
+        switch(type) {
+          case 0x01:
+            es_map[id_offset] = FOURCC('M', 'P', 'G', '1');
+            break;
+          case 0x02:
+            es_map[id_offset] = FOURCC('M', 'P', 'G', '2');
+            break;
+          case 0x03:
+          case 0x04:
+            es_map[id_offset] = FOURCC('M', 'P', '2', ' ');
+            break;
+          case 0x0f:
+          case 0x11:
+            es_map[id_offset] = FOURCC('A', 'A', 'C', ' ');
+            break;
+          case 0x10:
+            es_map[id_offset] = FOURCC('M', 'P', 'G', '4');
+            break;
+          case 0x1b:
+            es_map[id_offset] = FOURCC('A', 'V', 'C', '1');
+            break;
+          case 0x81:
+            es_map[id_offset] = FOURCC('A', 'C', '3', ' ');
+            break;
+        }
+      }
+
+      plen = io->read_uint16_be();
+      plen = MXMIN(plen, es_map_len);
+      io->skip(plen);
+      es_map_len -= 4 + plen;
+    }
+
+  } catch (...) {
+  }
+
+  io->setFilePointer(pos + len);
+}
+
 bool
 mpeg_ps_reader_c::parse_packet(int id,
                                int64_t &timestamp,
                                int &length,
+                               int &full_length,
                                int &aid) {
   uint8_t c;
 
   length = io->read_uint16_be();
+  full_length = length;
   if ((id < 0xbc) || (id >= 0xf0) ||
       (id == 0xbe) ||           // padding stream
       (id == 0xbf)) {           // private 2 stream
@@ -478,7 +596,7 @@ mpeg_ps_reader_c::parse_packet(int id,
     if (hdrlen > length)
       return false;
 
-    if ((pts_flags & 2) == 2) {
+    if (2 == (pts_flags & 0x02)) {
       if (hdrlen < 5)
         return false;
       c = io->read_uint8();
@@ -488,7 +606,9 @@ mpeg_ps_reader_c::parse_packet(int id,
       hdrlen -= 5;
 
     }
-    if (pts_flags == 3) {
+
+    // DTS
+    if (1 == (pts_flags & 0x01)) {
       if (hdrlen < 5)
         return false;
       io->skip(5);
@@ -507,13 +627,17 @@ mpeg_ps_reader_c::parse_packet(int id,
       aid = io->read_uint8();
       length--;
 
-      if ((aid >= 0x80) && (aid <= 0xbf)) {
+      if ((aid & 0xe0) == 0x20) {
+        // Subtitles, not supported yet.
+        return false;
+
+      } else if (   ((0x80 <= aid) && (0x8f >= aid))
+                 || ((0x98 <= aid) && (0xaf >= aid))
+                 || ((0xc0 <= aid) && (0xcf >= aid))) {
         io->skip(3);         // number of frames, startpos
         length -= 3;
 
-        if (aid >= 0xa0) {      // LPCM
-          if (length < 3)
-            return false;
+        if ((0xa0 == (aid & 0xe0)) && (3 <= length)) { // LPCM
           io->skip(3);
           length -= 3;
         }
@@ -529,6 +653,219 @@ mpeg_ps_reader_c::parse_packet(int id,
   return true;
 }
 
+void
+mpeg_ps_reader_c::new_stream_v_mpeg_1_2(int id,
+                                        unsigned char *buf,
+                                        int length,
+                                        mpeg_ps_track_ptr &track) {
+  int64_t timecode;
+  int state, aid, full_length;
+  auto_ptr<M2VParser> m2v_parser(new M2VParser);
+  MPEG2SequenceHeader seq_hdr;
+  MPEGChunk *raw_seq_hdr;
+
+  m2v_parser->WriteData(buf, length);
+
+  state = m2v_parser->GetState();
+  while ((MPV_PARSER_STATE_FRAME != state) &&
+         (PS_PROBE_SIZE >= io->getFilePointer())) {
+    if (!find_next_packet_for_id(id, PS_PROBE_SIZE))
+      break;
+
+    if (!parse_packet(id, timecode, length, full_length, aid))
+      break;
+    memory_c new_buf((unsigned char *)safemalloc(length), 0, true);
+    if (io->read(new_buf.get(), length) != length)
+      break;
+
+    m2v_parser->WriteData(new_buf.get(), length);
+
+    state = m2v_parser->GetState();
+  }
+
+  if (MPV_PARSER_STATE_FRAME != state) {
+    mxverb(3, "MPEG PS: blacklisting id %d for supposed type MPEG1/2\n",
+           id);
+    blacklisted_ids[id] = true;
+    return;
+  }
+
+  seq_hdr = m2v_parser->GetSequenceHeader();
+  auto_ptr<MPEGFrame> frame(m2v_parser->ReadFrame());
+  if (frame.get() == NULL)
+    throw false;
+
+  track->v_version = m2v_parser->GetMPEGVersion();
+  track->v_width = seq_hdr.width;
+  track->v_height = seq_hdr.height;
+  track->v_frame_rate = seq_hdr.frameRate;
+  track->v_aspect_ratio = seq_hdr.aspectRatio;
+  if ((track->v_aspect_ratio <= 0) || (track->v_aspect_ratio == 1))
+    track->v_dwidth = track->v_width;
+  else
+    track->v_dwidth = (int)(track->v_height * track->v_aspect_ratio);
+  track->v_dheight = track->v_height;
+  raw_seq_hdr = m2v_parser->GetRealSequenceHeader();
+  if (raw_seq_hdr != NULL) {
+    track->raw_seq_hdr = (unsigned char *)
+      safememdup(raw_seq_hdr->GetPointer(), raw_seq_hdr->GetSize());
+    track->raw_seq_hdr_size = raw_seq_hdr->GetSize();
+  }
+  track->fourcc = FOURCC('M', 'P', 'G', '0' + track->v_version);
+  track->use_buffer(128000);
+}
+
+void
+mpeg_ps_reader_c::new_stream_v_avc(int id,
+                                   unsigned char *buf,
+                                   int length,
+                                   mpeg_ps_track_ptr &track) {
+  mpeg4::p10::avc_es_parser_c parser;
+  int aid, full_length;
+  int64_t timecode;
+
+  parser.ignore_nalu_size_length_errors();
+  if (map_has_key(ti.nalu_size_lengths, tracks.size()))
+    parser.set_nalu_size_length(ti.nalu_size_lengths[0]);
+  else if (map_has_key(ti.nalu_size_lengths, -1))
+    parser.set_nalu_size_length(ti.nalu_size_lengths[-1]);
+
+  parser.add_bytes(buf, length);
+
+  while (!parser.headers_parsed() &&
+         (PS_PROBE_SIZE >= io->getFilePointer())) {
+    if (!find_next_packet_for_id(id, PS_PROBE_SIZE))
+      break;
+
+    if (!parse_packet(id, timecode, length, full_length, aid))
+      break;
+    memory_c new_buf((unsigned char *)safemalloc(length), 0, true);
+    if (io->read(new_buf.get(), length) != length)
+      break;
+
+    parser.add_bytes(new_buf.get(), length);
+  }
+
+  if (!parser.headers_parsed())
+    throw false;
+
+  track->v_avcc = parser.get_avcc();
+
+  try {
+    mm_mem_io_c avcc(track->v_avcc->get(), track->v_avcc->get_size());
+    mm_mem_io_c new_avcc(NULL, track->v_avcc->get_size(), 1024);
+    memory_cptr nalu(new memory_c());
+    int num_sps, sps, length;
+    sps_info_t sps_info;
+
+    avcc.read(nalu, 5);
+    new_avcc.write(nalu);
+
+    num_sps = avcc.read_uint8();
+    new_avcc.write_uint8(num_sps);
+    num_sps &= 0x1f;
+
+    for (sps = 0; sps < num_sps; sps++) {
+      bool abort;
+
+      length = avcc.read_uint16_be();
+      if ((length + avcc.getFilePointer()) >= track->v_avcc->get_size())
+        length = track->v_avcc->get_size() - avcc.getFilePointer();
+      avcc.read(nalu, length);
+
+      abort = false;
+      if ((0 < length) && ((nalu->get()[0] & 0x1f) == 7)) {
+        nalu_to_rbsp(nalu);
+        if (!mpeg4::p10::parse_sps(nalu, sps_info, true))
+          throw false;
+        rbsp_to_nalu(nalu);
+        abort = true;
+      }
+
+      new_avcc.write_uint16_be(nalu->get_size());
+      new_avcc.write(nalu);
+
+      if (abort) {
+        avcc.read(nalu, track->v_avcc->get_size() - avcc.getFilePointer());
+        new_avcc.write(nalu);
+        break;
+      }
+    }
+
+    track->v_avcc =
+      memory_cptr(new memory_c(new_avcc.get_and_lock_buffer(),
+                               new_avcc.getFilePointer(), true));
+    track->v_width = sps_info.width;
+    track->v_height = sps_info.height;
+    track->fourcc = FOURCC('A', 'V', 'C', '1');
+
+    if (sps_info.ar_found) {
+      float aspect_ratio = (float)sps_info.width / (float)sps_info.height *
+        (float)sps_info.par_num / (float)sps_info.par_den;
+      track->v_aspect_ratio = aspect_ratio;
+
+      if (aspect_ratio > ((float)track->v_width / (float)track->v_height)) {
+        track->v_dwidth = irnd(track->v_height * aspect_ratio);
+        track->v_dheight = track->v_height;
+
+      } else {
+        track->v_dwidth = track->v_width;
+        track->v_dheight = irnd(track->v_width / aspect_ratio);
+      }
+    }
+
+    track->use_buffer(256000);
+
+  } catch (...) {
+    throw false;
+  }
+}
+
+void
+mpeg_ps_reader_c::new_stream_a_mpeg(int id,
+                                    unsigned char *buf,
+                                    int length,
+                                    mpeg_ps_track_ptr &track) {
+  mp3_header_t header;
+
+  if (-1 == find_mp3_header(buf, length))
+    throw false;
+
+  decode_mp3_header(buf, &header);
+  track->a_channels = header.channels;
+  track->a_sample_rate = header.sampling_frequency;
+  track->fourcc = FOURCC('M', 'P', '0' + header.layer, ' ');
+}
+
+void
+mpeg_ps_reader_c::new_stream_a_ac3(int id,
+                                   unsigned char *buf,
+                                   int length,
+                                   mpeg_ps_track_ptr &track) {
+  ac3_header_t header;
+
+  if (-1 == find_ac3_header(buf, length, &header))
+    throw false;
+
+  mxverb(2, "first ac3 header bsid %d channels %d sample_rate %d "
+         "bytes %d samples %d\n",
+         header.bsid, header.channels, header.sample_rate, header.bytes,
+         header.samples);
+
+  track->a_channels = header.channels;
+  track->a_sample_rate = header.sample_rate;
+  track->a_bsid = header.bsid;
+}
+
+void
+mpeg_ps_reader_c::new_stream_a_dts(int id,
+                                   unsigned char *buf,
+                                   int length,
+                                   mpeg_ps_track_ptr &track) {
+  if (-1 == find_dts_header(buf, length, &track->dts_header))
+    throw false;
+}
+
 void
 mpeg_ps_reader_c::found_new_stream(int id) {
   if (((id < 0xc0) || (id > 0xef)) && (id != 0xbd))
@@ -536,10 +873,10 @@ mpeg_ps_reader_c::found_new_stream(int id) {
 
   try {
     int64_t timecode;
-    int length, aid;
+    int length, aid, full_length;
     unsigned char *buf;
 
-    if (!parse_packet(id, timecode, length, aid))
+    if (!parse_packet(id, timecode, length, full_length, aid))
       throw false;
 
     if ((id == 0xbd) && (aid == -1))
@@ -548,134 +885,91 @@ mpeg_ps_reader_c::found_new_stream(int id) {
     if (id == 0xbd)             // DVD audio substream
       id = 256 + aid;
 
-    if (id2idx[id] != -1)
+    if ((-1 != id2idx[id]) || blacklisted_ids[id])
       return;
 
+    mxverb(2, "MPEG PS: new stream id 0x%04x\n", id);
+
     mpeg_ps_track_ptr track(new mpeg_ps_track_t);
     track->timecode_offset = timecode;
 
+    track->type = '?';
     if (id > 0xff) {
+      track->type = 'a';
+
       if ((aid >= 0x20) && (aid <= 0x3f)) {
         track->type = 's';
         track->fourcc = FOURCC('V', 'S', 'U', 'B');
-      } else if ((aid >= 0x80) && (aid <= 0x87)) {
-        track->type = 'a';
+
+      } else if (((aid >= 0x80) && (aid <= 0x87)) ||
+                 ((aid >= 0xc0) && (aid <= 0xc7)))
         track->fourcc = FOURCC('A', 'C', '3', ' ');
-      } else if ((aid >= 0x88) && (aid <= 0x90)) {
-        track->type = 'a';
+      else if ((aid >= 0x88) && (aid <= 0x8f))
         track->fourcc = FOURCC('D', 'T', 'S', ' ');
-      } else if ((aid >= 0xa0) && (aid <= 0xa8)) {
-        track->type = 'a';
+      else if ((aid >= 0xa0) && (aid <= 0xa7))
         track->fourcc = FOURCC('P', 'C', 'M', ' ');
-      }
+      else
+        track->type = '?';
+
     } else if (id < 0xe0) {
       track->type = 'a';
       track->fourcc = FOURCC('M', 'P', '2', ' ');
-    } else {
+
+    } else if ((0xe0 <= id) && (0xef >= id)) {
       track->type = 'v';
-      track->fourcc = FOURCC('m', 'p', 'g', '0' + version);
+      track->fourcc = FOURCC('M', 'P', 'G', '0' + version);
     }
 
     if (track->type == '?')
       return;
 
-    autofree_ptr<unsigned char> af_buf(safemalloc(length));
-    buf = af_buf;
+    memory_c af_buf((unsigned char *)safemalloc(length), 0, true);
+    buf = af_buf.get();
     if (io->read(buf, length) != length)
       throw false;
 
-    if (track->type == 'v') {
-      if ((track->fourcc == FOURCC('m', 'p', 'g', '1')) ||
-          (track->fourcc == FOURCC('m', 'p', 'g', '2'))) {
-        int state;
-        auto_ptr<M2VParser> m2v_parser(new M2VParser);
-        MPEG2SequenceHeader seq_hdr;
-        MPEGChunk *raw_seq_hdr;
+    if ((track->fourcc == FOURCC('M', 'P', 'G', '1')) ||
+        (track->fourcc == FOURCC('M', 'P', 'G', '2'))) {
+      if (4 > length)
+        throw false;
 
-        m2v_parser->WriteData(buf, length);
+      uint32_t code = get_uint32_be(buf);
 
-        state = m2v_parser->GetState();
-        while (state != MPV_PARSER_STATE_FRAME) {
-          if (!find_next_packet_for_id(id))
-            throw false;
+      if (0x00000001 == code)
+        new_stream_v_avc(id, buf, length, track);
 
-          if (!parse_packet(id, timecode, length, aid))
-            throw false;
-          autofree_ptr<unsigned char> new_buf(safemalloc(length));
-          if (io->read(new_buf, length) != length)
-            throw false;
+      else if ((MPEGVIDEO_SEQUENCE_START_CODE == code) ||
+               (MPEGVIDEO_PACKET_START_CODE == code) ||
+               (MPEGVIDEO_SYSTEM_HEADER_START_CODE == code))
+        new_stream_v_mpeg_1_2(id, buf, length, track);
 
-          m2v_parser->WriteData(new_buf, length);
+      else
+        throw false;
 
-          state = m2v_parser->GetState();
-        }
+    } else if (track->fourcc == FOURCC('M', 'P', '2', ' '))
+      new_stream_a_mpeg(id, buf, length, track);
 
-        seq_hdr = m2v_parser->GetSequenceHeader();
-        auto_ptr<MPEGFrame> frame(m2v_parser->ReadFrame());
-        if (frame.get() == NULL)
-          throw false;
+    else if (track->fourcc == FOURCC('A', 'C', '3', ' '))
+      new_stream_a_ac3(id, buf, length, track);
 
-        track->v_version = m2v_parser->GetMPEGVersion();
-        track->v_width = seq_hdr.width;
-        track->v_height = seq_hdr.height;
-        track->v_frame_rate = seq_hdr.frameRate;
-        track->v_aspect_ratio = seq_hdr.aspectRatio;
-        if ((track->v_aspect_ratio <= 0) || (track->v_aspect_ratio == 1))
-          track->v_dwidth = track->v_width;
-        else
-          track->v_dwidth = (int)(track->v_height * track->v_aspect_ratio);
-        track->v_dheight = track->v_height;
-        raw_seq_hdr = m2v_parser->GetRealSequenceHeader();
-        if (raw_seq_hdr != NULL) {
-          track->raw_seq_hdr = (unsigned char *)
-            safememdup(raw_seq_hdr->GetPointer(), raw_seq_hdr->GetSize());
-          track->raw_seq_hdr_size = raw_seq_hdr->GetSize();
-        }
-        track->fourcc = FOURCC('m', 'p', 'g', '0' + track->v_version);
+//     else if (track->fourcc == FOURCC('D', 'T', 'S', ' '))
+//       new_stream_a_dts(id, buf, length, track);
 
-      } else                    // if (track->fourcc == ...)
-        // Unsupported video track type
-        return;
-
-    } else if (track->type == 'a') { // if (track->type == 'v')
-      if (track->fourcc == FOURCC('M', 'P', '2', ' ')) {
-        mp3_header_t header;
-
-        if (find_mp3_header(buf, length) != 0)
-          throw "Error parsing the first MP2/MP3 audio frame.";
-        decode_mp3_header(buf, &header);
-        track->a_channels = header.channels;
-        track->a_sample_rate = header.sampling_frequency;
-        track->fourcc = FOURCC('M', 'P', '0' + header.layer, ' ');
-
-      } else if (track->fourcc == FOURCC('A', 'C', '3', ' ')) {
-        ac3_header_t header;
-
-        if (find_ac3_header(buf, length, &header) != 0)
-          throw "Error parsing the first AC3 audio frame.";
-        track->a_channels = header.channels;
-        track->a_sample_rate = header.sample_rate;
-        track->a_bsid = header.bsid;
-
-      } else if (track->fourcc == FOURCC('D', 'T', 'S', ' ')) {
-        if (find_dts_header(buf, length, &track->dts_header) != 0)
-          throw "Error parsing the first DTS audio frame.";
-
-//       } else if (track->fourcc == FOURCC('P', 'C', 'M', ' ')) {
-
-      } else
-        // Unsupported audio track type
-        return;
-    } else
+    else
       // Unsupported track type
-      return;
+      throw false;
 
     track->id = id;
     id2idx[id] = tracks.size();
     tracks.push_back(track);
-  } catch(const char *msg) {
+
+  } catch (bool err) {
+    blacklisted_ids[id] = true;
+
+  } catch (const char *msg) {
     mxerror(FMT_FN "%s\n", ti.fname.c_str(), msg);
-  } catch(...) {
+
+  } catch (...) {
     mxerror(FMT_FN "Error parsing a MPEG PS packet during the "
             "header reading phase. This stream seems to be badly damaged.\n",
             ti.fname.c_str());
@@ -683,7 +977,8 @@ mpeg_ps_reader_c::found_new_stream(int id) {
 }
 
 bool
-mpeg_ps_reader_c::find_next_packet(int &id) {
+mpeg_ps_reader_c::find_next_packet(int &id,
+                                   int64_t max_file_pos) {
   try {
     uint32_t header;
 
@@ -691,6 +986,9 @@ mpeg_ps_reader_c::find_next_packet(int &id) {
     while (1) {
       uint8_t byte;
 
+      if ((-1 != max_file_pos) && (io->getFilePointer() > max_file_pos))
+        return false;
+
       switch (header) {
         case MPEGVIDEO_PACKET_START_CODE:
           if (version == -1) {
@@ -725,8 +1023,12 @@ mpeg_ps_reader_c::find_next_packet(int &id) {
         case MPEGVIDEO_MPEG_PROGRAM_END_CODE:
           return false;
 
+        case MPEGVIDEO_PROGRAM_STREAM_MAP_START_CODE:
+          parse_program_stream_map();
+          break;
+
         default:
-          if (!mpeg_is_start_code(header))
+          if (!mpeg_is_start_code(header) && !resync_stream(header))
             return false;
 
           id = header & 0xff;
@@ -741,11 +1043,12 @@ mpeg_ps_reader_c::find_next_packet(int &id) {
 }
 
 bool
-mpeg_ps_reader_c::find_next_packet_for_id(int id) {
+mpeg_ps_reader_c::find_next_packet_for_id(int id,
+                                          int64_t max_file_pos) {
   int new_id;
 
   try {
-    while (find_next_packet(new_id)) {
+    while (find_next_packet(new_id, max_file_pos)) {
       if (id == new_id)
         return true;
       io->skip(io->read_uint16_be());
@@ -755,6 +1058,29 @@ mpeg_ps_reader_c::find_next_packet_for_id(int id) {
   return false;
 }
 
+bool
+mpeg_ps_reader_c::resync_stream(uint32_t &header) {
+  mxverb(2, "MPEG PS: synchronisation lost at " LLD
+         "; looking for start code\n", io->getFilePointer());
+
+  try {
+    while (1) {
+      header <<= 8;
+      header |= io->read_uint8();
+      if (mpeg_is_start_code(header))
+        break;
+    }
+
+    mxverb(2, "resync succeeded at " LLD ", header 0x%08x\n",
+           io->getFilePointer() - 4, header);
+    return true;
+
+  } catch (...) {
+    mxverb(2, "resync failed: exception caught\n");
+    return false;
+  }
+}
+
 void
 mpeg_ps_reader_c::create_packetizer(int64_t id) {
   if ((id < 0) || (id >= tracks.size()))
@@ -783,8 +1109,8 @@ mpeg_ps_reader_c::create_packetizer(int64_t id) {
                                             track->a_channels,
                                             track->a_bsid, ti));
       if (verbose)
-        mxinfo(FMT_TID "Using the AC3 output module.\n", ti.fname.c_str(),
-               id);
+        mxinfo(FMT_TID "Using the %sAC3 output module.\n", ti.fname.c_str(),
+               id, 16 == track->a_bsid ? "E" : "");
 
     } else if (track->fourcc == FOURCC('D', 'T', 'S', ' ')) {
       track->ptzr =
@@ -797,8 +1123,8 @@ mpeg_ps_reader_c::create_packetizer(int64_t id) {
       mxerror("mpeg_ps_reader: Should not have happened #1. %s", BUGMSG);
 
   } else {                      // if (track->type == 'a')
-    if ((track->fourcc == FOURCC('m', 'p', 'g', '1')) ||
-        (track->fourcc == FOURCC('m', 'p', 'g', '2'))) {
+    if ((track->fourcc == FOURCC('M', 'P', 'G', '1')) ||
+        (track->fourcc == FOURCC('M', 'P', 'G', '2'))) {
       mpeg1_2_video_packetizer_c *ptzr;
 
       ti.private_data = track->raw_seq_hdr;
@@ -817,6 +1143,17 @@ mpeg_ps_reader_c::create_packetizer(int64_t id) {
         mxinfo(FMT_TID "Using the MPEG-1/2 video output module.\n",
                ti.fname.c_str(), id);
 
+    } else if (track->fourcc == FOURCC('A', 'V', 'C', '1')) {
+      mpeg4_p10_es_video_packetizer_c *ptzr =
+        new mpeg4_p10_es_video_packetizer_c(this, track->v_avcc,
+                                            track->v_width, track->v_height,
+                                            ti);
+      track->ptzr = add_packetizer(ptzr);
+
+      if (verbose)
+        mxinfo(FMT_TID "Using the MPEG-4 part 10 ES video output module.\n",
+               ti.fname.c_str(), id);
+
     } else
       mxerror("mpeg_ps_reader: Should not have happened #2. %s", BUGMSG);
   }
@@ -842,7 +1179,7 @@ file_status_e
 mpeg_ps_reader_c::read(generic_packetizer_c *,
                        bool) {
   int64_t timecode, packet_pos;
-  int new_id, length, aid;
+  int new_id, length, aid, full_length;
   unsigned char *buf;
 
   if (file_done)
@@ -858,11 +1195,11 @@ mpeg_ps_reader_c::read(generic_packetizer_c *,
       }
 
       packet_pos = io->getFilePointer() - 4;
-      if (!parse_packet(new_id, timecode, length, aid)) {
-        file_done = true;
-        flush_packetizers();
-        mxverb(2, "mpeg_ps: file_done: !parse_packet\n");
-        return FILE_STATUS_DONE;
+      if (!parse_packet(new_id, timecode, length, full_length, aid)) {
+        mxverb(2, "mpeg_ps: packet_parse failed @ " LLD ", skipping %d\n",
+               packet_pos, full_length);
+        io->setFilePointer(packet_pos + 4 + 2 + full_length);
+        continue;
       }
 
       if (new_id == 0xbd)
@@ -874,31 +1211,74 @@ mpeg_ps_reader_c::read(generic_packetizer_c *,
         continue;
       }
 
-      mxverb(3, "mpeg_ps: packet for %d length %d at %lld\n", new_id, length,
-             packet_pos);
+      mpeg_ps_track_t *track = tracks[id2idx[new_id]].get();
 
-      buf = (unsigned char *)safemalloc(length);
-      if (io->read(buf, length) != length) {
-        safefree(buf);
-        file_done = true;
-        flush_packetizers();
-        mxverb(2, "mpeg_ps: file_done: io->read\n");
-        return FILE_STATUS_DONE;
+      mxverb(3, "mpeg_ps: packet for %d length %d at " LLD " timecode " LLD "\n", new_id,
+             length, packet_pos, timecode);
+
+      if (0 < track->buffer_size) {
+        if (//(-1 != timecode)
+             ((track->buffer_usage + length) > track->buffer_size)) {
+          PTZR(track->ptzr)->
+            process(new packet_t(new memory_c(track->buffer,
+                                              track->buffer_usage, false)));
+          track->buffer_usage = 0;
+        }
+
+        track->assert_buffer_size(length);
+
+        if (io->read(&track->buffer[track->buffer_usage], length) != length) {
+          mxverb(2, "mpeg_ps: file_done: io->read\n");
+          return finish();
+        }
+
+        track->buffer_usage += length;
+
+      } else {
+        buf = (unsigned char *)safemalloc(length);
+
+        if (io->read(buf, length) != length) {
+          safefree(buf);
+          mxverb(2, "mpeg_ps: file_done: io->read\n");
+          return finish();
+        }
+
+        PTZR(track->ptzr)->process(new packet_t(new memory_c(buf, length,
+                                                             true)));
       }
 
-      PTZR(tracks[id2idx[new_id]]->ptzr)->
-        process(new packet_t(new memory_c(buf, length, true)));
-
       return FILE_STATUS_MOREDATA;
     }
+    mxverb(2, "mpeg_ps: file_done: !find_next_packet\n");
+
   } catch(...) {
+    mxverb(2, "mpeg_ps: file_done: exception\n");
   }
+
+  return finish();
+}
+
+file_status_e
+mpeg_ps_reader_c::finish() {
+  if (file_done)
+    return FILE_STATUS_DONE;
+
+  vector<mpeg_ps_track_ptr>::iterator track;
+
+  mxforeach(track, tracks) {
+    if (0 < (*track)->buffer_usage) {
+      memory_c *mem = new memory_c((*track)->buffer, (*track)->buffer_usage);
+      PTZR((*track)->ptzr)->process(new packet_t(mem));
+    }
+  }
+
   file_done = true;
   flush_packetizers();
-  mxverb(2, "mpeg_ps: file_done: exception\n");
+
   return FILE_STATUS_DONE;
 }
 
+
 int
 mpeg_ps_reader_c::get_progress() {
   return 100 * io->getFilePointer() / size;
@@ -912,18 +1292,84 @@ mpeg_ps_reader_c::identify() {
          ti.fname.c_str(), version);
   for (i = 0; i < tracks.size(); i++) {
     mpeg_ps_track_ptr &track = tracks[i];
-    mxinfo("Track ID %d: %s (%s)\n", i,
+    string info;
+
+    if (identify_verbose) {
+      if (FOURCC('A', 'V', 'C', '1') == track->fourcc) {
+        if (0 != track->v_aspect_ratio)
+          info += mxsprintf("display_dimensions:%dx%d ",
+                            track->v_dwidth, track->v_dheight);
+      }
+      info += "packetizer:mpeg4_p10_es_video ";
+    }
+
+    info += mxsprintf("stream_id:%#x", track->id);
+
+    if (!info.empty()) {
+      info = mxsprintf(" [%s]", info.c_str());
+    }
+
+    mxinfo("Track ID %d: %s (%s)%s\n", i,
            track->type == 'a' ? "audio" : "video",
-           track->fourcc == FOURCC('m', 'p', 'g', '1') ? "MPEG-1" :
-           track->fourcc == FOURCC('m', 'p', 'g', '2') ? "MPEG-2" :
+           track->fourcc == FOURCC('M', 'P', 'G', '1') ? "MPEG-1" :
+           track->fourcc == FOURCC('M', 'P', 'G', '2') ? "MPEG-2" :
+           track->fourcc == FOURCC('A', 'V', 'C', '1') ? "AVC/h.264" :
            track->fourcc == FOURCC('M', 'P', '1', ' ') ? "MPEG-1 layer 1" :
            track->fourcc == FOURCC('M', 'P', '2', ' ') ? "MPEG-1 layer 2" :
            track->fourcc == FOURCC('M', 'P', '3', ' ') ? "MPEG-1 layer 3" :
-           track->fourcc == FOURCC('A', 'C', '3', ' ') ? "AC3" :
+           track->fourcc == FOURCC('A', 'C', '3', ' ') ?
+           (16 == track->a_bsid ? "EAC3" : "AC3") :
            track->fourcc == FOURCC('D', 'T', 'S', ' ') ? "DTS" :
            track->fourcc == FOURCC('P', 'C', 'M', ' ') ? "PCM" :
            track->fourcc == FOURCC('L', 'P', 'C', 'M') ? "LPCM" :
-           "unknown");
+           "unknown",
+           info.c_str());
   }
 }
 
+// ------------------------------------------------------------------------
+
+#define TS_CONSECUTIVE_PACKETS 16
+#define TS_PROBE_SIZE (2 * TS_CONSECUTIVE_PACKETS * 204)
+
+int mpeg_ts_reader_c::potential_packet_sizes[] = { 188, 192, 204, 0 };
+
+bool
+mpeg_ts_reader_c::probe_file(mm_io_c *io,
+                             int64_t size) {
+  try {
+    vector<int> positions;
+    size = size > TS_PROBE_SIZE ? TS_PROBE_SIZE : size;
+    memory_cptr buffer(new memory_c(safemalloc(size), size, true));
+    unsigned char *mem = buffer->get();
+    int i, k;
+
+    io->setFilePointer(0, seek_beginning);
+    size = io->read(mem, size);
+
+    for (i = 0; i < size; ++i)
+      if (0x47 == mem[i])
+        positions.push_back(i);
+
+    for (i = 0; positions.size() > i; ++i) {
+      int pos = positions[i];
+
+      for (k = 0; 0 != potential_packet_sizes[k]; ++k) {
+        int packet_size = potential_packet_sizes[k];
+        int num_startcodes = 1;
+
+        while ((TS_CONSECUTIVE_PACKETS > num_startcodes) && (pos < size) && (0x47 == mem[pos])) {
+          pos += packet_size;
+          ++num_startcodes;
+        }
+
+        if (TS_CONSECUTIVE_PACKETS <= num_startcodes)
+          return true;
+      }
+    }
+
+  } catch (...) {
+  }
+
+  return false;
+}
