@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_ogm.cpp 3210 2006-04-28 08:48:49Z mosu $
+   $Id: r_ogm.cpp 3578 2007-08-16 16:30:05Z mosu $
 
    OGG media stream reader
 
@@ -22,7 +22,12 @@
 #include <ogg/ogg.h>
 #include <vorbis/codec.h>
 #if defined(HAVE_FLAC_FORMAT_H)
-#include <FLAC/stream_decoder.h>
+# include <FLAC/stream_decoder.h>
+# if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT < 8
+#  define LEGACY_FLAC
+# else
+#  undef LEGACY_FLAC
+# endif
 #endif
 #if defined(SYS_WINDOWS)
 #include <windows.h>
@@ -37,6 +42,7 @@ extern "C" {                    // for BITMAPINFOHEADER
 #include "chapters.h"
 #include "common.h"
 #include "commonebml.h"
+#include "hacks.h"
 #include "iso639.h"
 #include "matroska.h"
 #include "ogmstreams.h"
@@ -44,6 +50,7 @@ extern "C" {                    // for BITMAPINFOHEADER
 #include "pr_generic.h"
 #include "p_aac.h"
 #include "p_ac3.h"
+#include "p_avc.h"
 #if defined(HAVE_FLAC_FORMAT_H)
 #include "p_flac.h"
 #endif
@@ -102,17 +109,21 @@ extract_vorbis_comments(const memory_cptr &mem) {
 static FLAC__StreamDecoderReadStatus
 fhe_read_cb(const FLAC__StreamDecoder *decoder,
             FLAC__byte buffer[],
+#ifdef LEGACY_FLAC
             unsigned *bytes,
+#else
+            size_t *bytes,
+#endif
             void *client_data) {
   flac_header_extractor_c *fhe;
   ogg_packet op;
 
   fhe = (flac_header_extractor_c *)client_data;
   if (fhe->done)
-    return FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM;
+    return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
   if (ogg_stream_packetout(&fhe->os, &op) != 1) {
     if (!fhe->read_page() || (ogg_stream_packetout(&fhe->os, &op) != 1))
-      return FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM;
+      return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
   }
 
   if (*bytes < op.bytes)
@@ -193,6 +204,7 @@ flac_header_extractor_c::flac_header_extractor_c(const string &file_name,
   decoder = FLAC__stream_decoder_new();
   if (decoder == NULL)
     mxerror(FPFX "FLAC__stream_decoder_new() failed.\n");
+#ifdef LEGACY_FLAC
   FLAC__stream_decoder_set_client_data(decoder, this);
   if (!FLAC__stream_decoder_set_read_callback(decoder, fhe_read_cb))
     mxerror(FPFX "Could not set the read callback.\n");
@@ -202,10 +214,16 @@ flac_header_extractor_c::flac_header_extractor_c(const string &file_name,
     mxerror(FPFX "Could not set the metadata callback.\n");
   if (!FLAC__stream_decoder_set_error_callback(decoder, fhe_error_cb))
     mxerror(FPFX "Could not set the error callback.\n");
+#endif
   if (!FLAC__stream_decoder_set_metadata_respond_all(decoder))
     mxerror(FPFX "Could not set metadata_respond_all.\n");
+#ifdef LEGACY_FLAC
   if (FLAC__stream_decoder_init(decoder) !=
       FLAC__STREAM_DECODER_SEARCH_FOR_METADATA)
+#else
+  if (FLAC__stream_decoder_init_stream(decoder, fhe_read_cb, NULL, NULL, NULL, NULL, fhe_write_cb, fhe_metadata_cb, fhe_error_cb, this) !=
+      FLAC__STREAM_DECODER_INIT_STATUS_OK)
+#endif
     mxerror(FPFX "Could not initialize the FLAC decoder.\n");
   ogg_sync_init(&oy);
 }
@@ -413,7 +431,7 @@ ogm_reader_c::create_packetizer(int64_t tid) {
 
     ptzr = NULL;
     switch (dmx->stype) {
-      case OGM_STREAM_TYPE_VIDEO:
+      case OGM_STREAM_TYPE_V_MSCOMP:
         // AVI compatibility mode. Fill in the values we've got and guess
         // the others.
         put_uint32_le(&bih.bi_size, sizeof(alBITMAPINFOHEADER));
@@ -430,22 +448,44 @@ ogm_reader_c::create_packetizer(int64_t tid) {
         fps = (double)10000000.0 / get_uint64_le(&sth->time_unit);
         width = get_uint32_le(&sth->sh.video.width);
         height = get_uint32_le(&sth->sh.video.height);
-        if (is_mpeg4_p2_fourcc(sth->subtype)) {
+
+        if (mpeg4::p2::is_fourcc(sth->subtype)) {
           ptzr = new mpeg4_p2_video_packetizer_c(this, fps, width, height,
                                                  false, ti);
           mxinfo(FMT_TID "Using the MPEG-4 part 2 video output module.\n",
                  ti.fname.c_str(), (int64_t)tid);
+
+        } else if (dmx->is_avc && !hack_engaged(ENGAGE_ALLOW_AVC_IN_VFW_MODE)) {
+          try {
+            ti.private_data       = NULL;
+            ti.private_size       = 0;
+            dmx->default_duration = 100 * get_uint64_le(&sth->time_unit);
+            memory_cptr avcc      = extract_avcc(dmx, tid);
+
+            mpeg4_p10_es_video_packetizer_c *vptzr = new mpeg4_p10_es_video_packetizer_c(this, avcc, width, height, ti);
+
+            vptzr->enable_timecode_generation(false);
+            vptzr->set_track_default_duration(dmx->default_duration);
+
+            if (verbose)
+              mxinfo(FMT_TID "Using the MPEG-4 part 10 ES video output module.\n", ti.fname.c_str(), (int64_t)tid);
+
+            ptzr = vptzr;
+
+          } catch (...) {
+            mxerror(FMT_TID "Could not extract the decoder specific config data (AVCC) from this AVC/h.264 track.\n", ti.fname.c_str(), (int64_t)tid);
+          }
+
         } else {
           ptzr = new video_packetizer_c(this, NULL, fps, width, height, ti);
-          mxinfo(FMT_TID "Using the video output module.\n", ti.fname.c_str(),
-                 (int64_t)tid);
+          mxinfo(FMT_TID "Using the video output module.\n", ti.fname.c_str(), (int64_t)tid);
         }
 
         ti.private_data = NULL;
 
         break;
 
-      case OGM_STREAM_TYPE_PCM:
+      case OGM_STREAM_TYPE_A_PCM:
         ptzr = new pcm_packetizer_c(this,
                                     get_uint64_le(&sth->samples_per_unit),
                                     get_uint16_le(&sth->sh.audio.channels),
@@ -455,16 +495,17 @@ ogm_reader_c::create_packetizer(int64_t tid) {
                (int64_t)tid);
         break;
 
-      case OGM_STREAM_TYPE_MP3:
+      case OGM_STREAM_TYPE_A_MP2:
+      case OGM_STREAM_TYPE_A_MP3:
         ptzr = new mp3_packetizer_c(this,
                                     get_uint64_le(&sth->samples_per_unit),
                                     get_uint16_le(&sth->sh.audio.channels),
                                     true, ti);
-        mxinfo(FMT_TID "Using the MP3 output module.\n", ti.fname.c_str(),
+        mxinfo(FMT_TID "Using the MPEG audio output module.\n", ti.fname.c_str(),
                (int64_t)tid);
         break;
 
-      case OGM_STREAM_TYPE_AC3:
+      case OGM_STREAM_TYPE_A_AC3:
         ptzr = new ac3_packetizer_c(this,
                                     get_uint64_le(&sth->samples_per_unit),
                                     get_uint16_le(&sth->sh.audio.channels), 0,
@@ -474,7 +515,7 @@ ogm_reader_c::create_packetizer(int64_t tid) {
 
         break;
 
-      case OGM_STREAM_TYPE_AAC:
+      case OGM_STREAM_TYPE_A_AAC:
         aac_info_extracted = false;
         if (dmx->packet_data[0]->get_size() >= (sizeof(stream_header) + 5)) {
           if (parse_aac_data(dmx->packet_data[0]->get() +
@@ -509,7 +550,7 @@ ogm_reader_c::create_packetizer(int64_t tid) {
 
         break;
 
-      case OGM_STREAM_TYPE_VORBIS:
+      case OGM_STREAM_TYPE_A_VORBIS:
         vorbis_info_init(&vi);
         vorbis_comment_init(&vc);
         memset(&op, 0, sizeof(ogg_packet));
@@ -535,7 +576,7 @@ ogm_reader_c::create_packetizer(int64_t tid) {
 
         break;
 
-      case OGM_STREAM_TYPE_TEXT:
+      case OGM_STREAM_TYPE_S_TEXT:
         ptzr = new textsubs_packetizer_c(this, MKV_S_TEXTUTF8, NULL, 0, true,
                                          false, ti);
         mxinfo(FMT_TID "Using the text subtitle output module.\n",
@@ -544,7 +585,7 @@ ogm_reader_c::create_packetizer(int64_t tid) {
         break;
 
 #if defined(HAVE_FLAC_FORMAT_H)
-      case OGM_STREAM_TYPE_FLAC:
+      case OGM_STREAM_TYPE_A_FLAC:
         unsigned char *buf;
         int size, i;
 
@@ -566,7 +607,7 @@ ogm_reader_c::create_packetizer(int64_t tid) {
 
         break;
 #endif
-      case OGM_STREAM_TYPE_THEORA:
+      case OGM_STREAM_TYPE_V_THEORA:
         codecprivate = lace_memory_xiph(dmx->packet_data);
         ti.private_data = codecprivate->get();
         ti.private_size = codecprivate->get_size();
@@ -671,7 +712,7 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
     if (!demuxing_requested('a', sdemuxers.size() - 1))
       return;
 
-    dmx->stype = OGM_STREAM_TYPE_VORBIS;
+    dmx->stype = OGM_STREAM_TYPE_A_VORBIS;
     dmx->num_header_packets = 3;
     dmx->in_use = true;
 
@@ -682,7 +723,7 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
     if (!demuxing_requested('v', sdemuxers.size() - 1))
       return;
 
-    dmx->stype = OGM_STREAM_TYPE_THEORA;
+    dmx->stype = OGM_STREAM_TYPE_V_THEORA;
     dmx->num_header_packets = 3;
     dmx->in_use = true;
 
@@ -702,7 +743,7 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
     if (!demuxing_requested('a', sdemuxers.size() - 1))
       return;
 
-    dmx->stype = OGM_STREAM_TYPE_FLAC;
+    dmx->stype = OGM_STREAM_TYPE_A_FLAC;
     dmx->in_use = true;
 
 #if !defined(HAVE_FLAC_FORMAT_H)
@@ -733,11 +774,20 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
       if (!demuxing_requested('v', sdemuxers.size() - 1))
         return;
 
-      dmx->stype = OGM_STREAM_TYPE_VIDEO;
+      dmx->stype = OGM_STREAM_TYPE_V_MSCOMP;
       if (video_fps < 0)
         video_fps = 10000000.0 / (float)get_uint64_le(&sth->time_unit);
       dmx->in_use = true;
 
+      sth = (stream_header *)&op.packet[1];
+      memcpy(buf, (char *)sth->subtype, 4);
+      buf[4] = 0;
+
+      if (mpeg4::p10::is_avc_fourcc(buf)) {
+        dmx->is_avc                 = true;
+        dmx->num_non_header_packets = 3;
+      }
+
       return;
     }
 
@@ -751,13 +801,15 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
       codec_id = strtol(buf, (char **)NULL, 16);
 
       if (codec_id == 0x0001)
-        dmx->stype = OGM_STREAM_TYPE_PCM;
+        dmx->stype = OGM_STREAM_TYPE_A_PCM;
+      else if (codec_id == 0x0050)
+        dmx->stype = OGM_STREAM_TYPE_A_MP2;
       else if (codec_id == 0x0055)
-        dmx->stype = OGM_STREAM_TYPE_MP3;
+        dmx->stype = OGM_STREAM_TYPE_A_MP3;
       else if (codec_id == 0x2000)
-        dmx->stype = OGM_STREAM_TYPE_AC3;
+        dmx->stype = OGM_STREAM_TYPE_A_AC3;
       else if (codec_id == 0x00ff)
-        dmx->stype = OGM_STREAM_TYPE_AAC;
+        dmx->stype = OGM_STREAM_TYPE_A_AAC;
       else {
         mxwarn("ogm_reader: Unknown audio stream type %u. "
                "Ignoring stream id %u.\n", codec_id,
@@ -774,7 +826,7 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
       if (!demuxing_requested('s', sdemuxers.size() - 1))
         return;
 
-      dmx->stype = OGM_STREAM_TYPE_TEXT;
+      dmx->stype = OGM_STREAM_TYPE_S_TEXT;
       dmx->in_use = true;
 
       return;
@@ -826,7 +878,7 @@ ogm_reader_c::process_page(ogg_page *og) {
     eos = op.e_o_s;
 
 #if defined(HAVE_FLAC_FORMAT_H)
-    if (dmx->stype == OGM_STREAM_TYPE_FLAC) {
+    if (dmx->stype == OGM_STREAM_TYPE_A_FLAC) {
       dmx->units_processed++;
       if (dmx->units_processed <= dmx->flac_header_packets)
         continue;
@@ -858,7 +910,7 @@ ogm_reader_c::process_page(ogg_page *og) {
     }
 #endif
 
-    if (dmx->stype == OGM_STREAM_TYPE_THEORA) {
+    if (dmx->stype == OGM_STREAM_TYPE_V_THEORA) {
       int keyframe_number, non_keyframe_number;
       int64_t timecode, duration, bref;
 
@@ -901,16 +953,21 @@ ogm_reader_c::process_page(ogg_page *og) {
     if (((*op.packet & 3) != PACKET_TYPE_HEADER) &&
         ((*op.packet & 3) != PACKET_TYPE_COMMENT)) {
 
-      if (dmx->stype == OGM_STREAM_TYPE_VIDEO) {
-        memory_c *mem = new memory_c(&op.packet[duration_len + 1],
-                                     op.bytes - 1 - duration_len, false);
-        PTZR(dmx->ptzr)->
-          process(new packet_t(mem, -1, -1,
-                               (*op.packet & PACKET_IS_SYNCPOINT ?
-                                VFT_IFRAME : VFT_PFRAMEAUTOMATIC)));
-        dmx->units_processed += (duration_len > 0 ? duration : 1);
+      if (dmx->stype == OGM_STREAM_TYPE_V_MSCOMP) {
+        if (!duration_len || !duration)
+          duration = 1;
 
-      } else if (dmx->stype == OGM_STREAM_TYPE_TEXT) {
+        int64_t pos = ogg_page_granulepos(og);
+
+        if (!dmx->units_processed && (1 == pos))
+          dmx->first_granulepos = 1;
+        pos -= dmx->first_granulepos;
+
+        memory_c *mem = new memory_c(&op.packet[duration_len + 1], op.bytes - 1 - duration_len, false);
+        PTZR(dmx->ptzr)->process(new packet_t(mem, pos * dmx->default_duration, (int64_t)duration * dmx->default_duration));
+        dmx->units_processed += duration;
+
+      } else if (dmx->stype == OGM_STREAM_TYPE_S_TEXT) {
         dmx->units_processed++;
         if (((op.bytes - 1 - duration_len) > 2) ||
             ((op.packet[duration_len + 1] != ' ') &&
@@ -923,7 +980,7 @@ ogm_reader_c::process_page(ogg_page *og) {
                                  (int64_t)duration * 1000000));
         }
 
-      } else if (dmx->stype == OGM_STREAM_TYPE_VORBIS)
+      } else if (dmx->stype == OGM_STREAM_TYPE_A_VORBIS)
         PTZR(dmx->ptzr)->process(new packet_t(new memory_c(op.packet, op.bytes,
                                                            false)));
 
@@ -971,7 +1028,7 @@ ogm_reader_c::process_header_packets(ogm_demuxer_t *dmx) {
   if (dmx->headers_read)
     return;
 
-  if (dmx->stype == OGM_STREAM_TYPE_FLAC) {
+  if (dmx->stype == OGM_STREAM_TYPE_A_FLAC) {
 #if defined HAVE_FLAC_FORMAT_H
     while ((dmx->packet_data.size() < dmx->flac_header_packets) &&
            (ogg_stream_packetout(&dmx->os, &op) == 1)) {
@@ -997,27 +1054,34 @@ ogm_reader_c::process_header_packets(ogm_demuxer_t *dmx) {
   while (ogg_stream_packetout(&dmx->os, &op) == 1) {
     bool is_header_packet;
 
-    if (OGM_STREAM_TYPE_THEORA == dmx->stype)
+    if (OGM_STREAM_TYPE_V_THEORA == dmx->stype)
       is_header_packet = ((0x80 <= op.packet[0]) && (0x82 >= op.packet[0]));
     else
       is_header_packet = op.packet[0] & 1;
 
     if (!is_header_packet) {
+      if (dmx->nh_packet_data.size() != dmx->num_non_header_packets) {
+        memory_c *mem = new memory_c(safememdup(op.packet, op.bytes), op.bytes, true);
+        dmx->nh_packet_data.push_back(memory_cptr(mem));
+
+        continue;
+      }
+
       mxwarn("ogm_reader: Missing header/comment packets for stream %d in "
              "'%s'. This file is broken but should be muxed correctly. If "
-             "not please contact the author Moritz Bunkus "
-             "<moritz@bunkus.org>.\n", dmx->serialno, ti.fname.c_str());
+             "not please contact the author Moritz Bunkus <moritz@bunkus.org>.\n",
+             dmx->serialno, ti.fname.c_str());
       dmx->headers_read = true;
       ogg_stream_reset(&dmx->os);
       return;
     }
-    memory_c *mem = new memory_c((unsigned char *)
-                                 safememdup(op.packet, op.bytes),
-                                 op.bytes, true);
+
+    memory_c *mem = new memory_c(safememdup(op.packet, op.bytes), op.bytes, true);
     dmx->packet_data.push_back(memory_cptr(mem));
   }
 
-  if (dmx->packet_data.size() == dmx->num_header_packets)
+  if (   (dmx->packet_data.size()    == dmx->num_header_packets)
+      && (dmx->nh_packet_data.size() >= dmx->num_non_header_packets))
     dmx->headers_read = true;
 }
 
@@ -1117,14 +1181,14 @@ ogm_reader_c::identify() {
   if (identify_verbose)
     for (i = 0; i < sdemuxers.size(); i++)
       if ((sdemuxers[i]->title != "") &&
-          (sdemuxers[i]->stype == OGM_STREAM_TYPE_VIDEO)) {
+          (sdemuxers[i]->stype == OGM_STREAM_TYPE_V_MSCOMP)) {
         info = string(" [title:") + escape(sdemuxers[i]->title) + string("]");
         break;
       }
 
   mxinfo("File '%s': container: Ogg/OGM%s\n", ti.fname.c_str(), info.c_str());
   for (i = 0; i < sdemuxers.size(); i++) {
-    if (sdemuxers[i]->stype == OGM_STREAM_TYPE_VIDEO) {
+    if (sdemuxers[i]->stype == OGM_STREAM_TYPE_V_MSCOMP) {
       sth = (stream_header *)(sdemuxers[i]->packet_data[0]->get() + 1);
       memcpy(fourcc, sth->subtype, 4);
       fourcc[4] = 0;
@@ -1135,32 +1199,34 @@ ogm_reader_c::identify() {
         info += string("language:") + escape(sdemuxers[i]->language) +
           string(" ");
       if ((sdemuxers[i]->title != "") &&
-          (sdemuxers[i]->stype != OGM_STREAM_TYPE_VIDEO))
+          (sdemuxers[i]->stype != OGM_STREAM_TYPE_V_MSCOMP))
         info += string("track_name:") + escape(sdemuxers[i]->title) +
           string(" ");
       info += "]";
     } else
       info = "";
     mxinfo("Track ID %d: %s (%s)%s\n", i,
-           (sdemuxers[i]->stype == OGM_STREAM_TYPE_VORBIS ||
-            sdemuxers[i]->stype == OGM_STREAM_TYPE_PCM ||
-            sdemuxers[i]->stype == OGM_STREAM_TYPE_MP3 ||
-            sdemuxers[i]->stype == OGM_STREAM_TYPE_AC3 ||
-            sdemuxers[i]->stype == OGM_STREAM_TYPE_AAC ||
-            sdemuxers[i]->stype == OGM_STREAM_TYPE_FLAC) ? "audio" :
-           (sdemuxers[i]->stype == OGM_STREAM_TYPE_VIDEO ||
-            sdemuxers[i]->stype == OGM_STREAM_TYPE_THEORA) ? "video" :
-           sdemuxers[i]->stype == OGM_STREAM_TYPE_TEXT ? "subtitles" :
+           (sdemuxers[i]->stype == OGM_STREAM_TYPE_A_AAC    ||
+            sdemuxers[i]->stype == OGM_STREAM_TYPE_A_AC3    ||
+            sdemuxers[i]->stype == OGM_STREAM_TYPE_A_FLAC   ||
+            sdemuxers[i]->stype == OGM_STREAM_TYPE_A_MP2    ||
+            sdemuxers[i]->stype == OGM_STREAM_TYPE_A_MP3    ||
+            sdemuxers[i]->stype == OGM_STREAM_TYPE_A_PCM    ||
+            sdemuxers[i]->stype == OGM_STREAM_TYPE_A_VORBIS) ? "audio"     :
+           (sdemuxers[i]->stype == OGM_STREAM_TYPE_V_MSCOMP ||
+            sdemuxers[i]->stype == OGM_STREAM_TYPE_V_THEORA) ? "video"     :
+           (sdemuxers[i]->stype == OGM_STREAM_TYPE_S_TEXT)   ? "subtitles" :
            "unknown",
-           sdemuxers[i]->stype == OGM_STREAM_TYPE_VORBIS ? "Vorbis" :
-           sdemuxers[i]->stype == OGM_STREAM_TYPE_PCM ? "PCM" :
-           sdemuxers[i]->stype == OGM_STREAM_TYPE_MP3 ? "MP3" :
-           sdemuxers[i]->stype == OGM_STREAM_TYPE_AC3 ? "AC3" :
-           sdemuxers[i]->stype == OGM_STREAM_TYPE_AAC ? "AAC" :
-           sdemuxers[i]->stype == OGM_STREAM_TYPE_VIDEO ? fourcc :
-           sdemuxers[i]->stype == OGM_STREAM_TYPE_THEORA ? "Theora" :
-           sdemuxers[i]->stype == OGM_STREAM_TYPE_TEXT ? "text" :
-           sdemuxers[i]->stype == OGM_STREAM_TYPE_FLAC ? "FLAC" :
+           sdemuxers[i]->stype == OGM_STREAM_TYPE_A_AAC    ? "AAC"    :
+           sdemuxers[i]->stype == OGM_STREAM_TYPE_A_AC3    ? "AC3"    :
+           sdemuxers[i]->stype == OGM_STREAM_TYPE_A_FLAC   ? "FLAC"   :
+           sdemuxers[i]->stype == OGM_STREAM_TYPE_A_MP2    ? "MP2"    :
+           sdemuxers[i]->stype == OGM_STREAM_TYPE_A_MP3    ? "MP3"    :
+           sdemuxers[i]->stype == OGM_STREAM_TYPE_A_PCM    ? "PCM"    :
+           sdemuxers[i]->stype == OGM_STREAM_TYPE_A_VORBIS ? "Vorbis" :
+           sdemuxers[i]->stype == OGM_STREAM_TYPE_S_TEXT   ? "text"   :
+           sdemuxers[i]->stype == OGM_STREAM_TYPE_V_MSCOMP ? fourcc   :
+           sdemuxers[i]->stype == OGM_STREAM_TYPE_V_THEORA ? "Theora" :
            "unknown",
            info.c_str());
   }
@@ -1176,15 +1242,12 @@ ogm_reader_c::handle_stream_comments() {
   string title;
   bool charset_warning_printed;
 
-  if (identifying)
-    return;
-
   charset_warning_printed = false;
   cch = utf8_init(ti.chapter_charset);
 
   for (i = 0; i < sdemuxers.size(); i++) {
     dmx = sdemuxers[i];
-    if ((dmx->stype == OGM_STREAM_TYPE_FLAC) ||
+    if ((dmx->stype == OGM_STREAM_TYPE_A_FLAC) ||
         (dmx->packet_data.size() < 2))
       continue;
     comments = extract_vorbis_comments(dmx->packet_data[1]);
@@ -1252,7 +1315,7 @@ ogm_reader_c::handle_stream_comments() {
     if (title != "") {
       title = to_utf8(cch, title);
       if (!segment_title_set && (segment_title.length() == 0) &&
-          (dmx->stype == OGM_STREAM_TYPE_VIDEO)) {
+          (dmx->stype == OGM_STREAM_TYPE_V_MSCOMP)) {
         segment_title = title;
         segment_title_set = true;
       }
@@ -1286,3 +1349,42 @@ ogm_reader_c::add_available_track_ids() {
     available_track_ids.push_back(i);
 }
 
+memory_cptr
+ogm_reader_c::extract_avcc(ogm_demuxer_t *dmx,
+                           int64_t tid) {
+  avc_es_parser_c parser;
+
+  parser.ignore_nalu_size_length_errors();
+  if (map_has_key(ti.nalu_size_lengths, tid))
+    parser.set_nalu_size_length(ti.nalu_size_lengths[tid]);
+  else if (map_has_key(ti.nalu_size_lengths, -1))
+    parser.set_nalu_size_length(ti.nalu_size_lengths[-1]);
+
+  unsigned char *private_data = dmx->packet_data[0]->get() + 1 + sizeof(stream_header);
+  int private_size            = dmx->packet_data[0]->get_size() - 1 - sizeof(stream_header);
+
+  while (4 < private_size) {
+    if (get_uint32_be(private_data) == 0x00000001) {
+      parser.add_bytes(private_data, private_size);
+      break;
+    }
+
+    ++private_data;
+    --private_size;
+  }
+
+  vector<memory_cptr>::iterator packet(dmx->nh_packet_data.begin());
+
+  while (packet != dmx->nh_packet_data.end()) {
+    if ((*packet)->get_size()) {
+      parser.add_bytes((*packet)->get(), (*packet)->get_size());
+      if (parser.headers_parsed())
+        return parser.get_avcc();
+    }
+
+    ++packet;
+  }
+
+  throw false;
+}
+
