@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_qtmp4.cpp 3416 2007-01-03 14:24:06Z mosu $
+   $Id: r_qtmp4.cpp 3567 2007-07-08 14:38:43Z mosu $
 
    Quicktime and MP4 reader
 
@@ -68,25 +68,30 @@ qtmp4_reader_c::probe_file(mm_io_c *in,
   uint32_t atom;
   uint64_t atom_size;
 
-  if (size < 20)
-    return 0;
   try {
     in->setFilePointer(0, seek_beginning);
 
-    atom_size = in->read_uint32_be();
-    atom = in->read_uint32_be();
-    if (atom_size == 1)
-      atom_size = in->read_uint64_be();
+    while (1) {
+      atom_size = in->read_uint32_be();
+      atom = in->read_uint32_be();
+      if (atom_size == 1)
+        atom_size = in->read_uint64_be();
 
-    mxverb(3, PFX "Atom: '%c%c%c%c'; size: " LLU "\n", BE2STR(atom),
-           atom_size);
+      mxverb(3, PFX "Atom: '%c%c%c%c'; size: " LLU "\n", BE2STR(atom),
+             atom_size);
 
-    if ((atom == FOURCC('m', 'o', 'o', 'v')) ||
-        (atom == FOURCC('f', 't', 'y', 'p')) ||
-        (atom == FOURCC('m', 'd', 'a', 't')) ||
-        (atom == FOURCC('p', 'n', 'o', 't')))
+      if ((atom == FOURCC('m', 'o', 'o', 'v')) ||
+          (atom == FOURCC('f', 't', 'y', 'p')) ||
+          (atom == FOURCC('m', 'd', 'a', 't')) ||
+          (atom == FOURCC('p', 'n', 'o', 't')))
         return 1;
 
+      if (atom == FOURCC('w', 'i', 'd', 'e'))
+        continue;
+
+      return 0;
+    }
+
   } catch (...) {
   }
 
@@ -427,18 +432,17 @@ qtmp4_reader_c::calculate_timecodes() {
   }
 
   if (0 > min_timecode) {
-    mxforeach(idmx, demuxers)
-      (*idmx)->adjust_timecodes(-1 * min_timecode);
-  } else
-    min_timecode = 0;
-
-  if (0 != min_timecode) {
     min_timecode *= -1;
+    mxforeach(idmx, demuxers)
+      (*idmx)->adjust_timecodes(min_timecode);
+
     mxwarn("This file contains at least one frame with a negative "
            "timecode. mkvmerge will adjust all timecodes by "
            FMT_TIMECODEN " so that none is negative anymore.\n",
            ARG_TIMECODEN(min_timecode));
-  }
+
+  } else
+    min_timecode = 0;
 }
 
 // Also taken from mplayer's demux_mov.c file.
@@ -1732,10 +1736,17 @@ qtmp4_reader_c::identify() {
   mxinfo("File '%s': container: Quicktime/MP4\n", ti.fname.c_str());
   for (i = 0; i < demuxers.size(); i++) {
     qtmp4_demuxer_ptr &dmx = demuxers[i];
-    mxinfo("Track ID %u: %s (%.4s)\n", dmx->id,
+    string extended_info;
+
+    if (dmx->v_is_avc)
+      extended_info = "packetizer:mpeg4_p10_video ";
+    if (!extended_info.empty())
+      extended_info = " [" + extended_info + "]";
+
+    mxinfo("Track ID %u: %s (%.4s)%s\n", dmx->id,
            dmx->type == 'v' ? "video" :
            dmx->type == 'a' ? "audio" : "unknown",
-           dmx->fourcc);
+           dmx->fourcc, extended_info.c_str());
   }
 }
 
