@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_qtmp4.h 2951 2005-06-10 07:12:50Z mosu $
+   $Id: r_qtmp4.h 3369 2006-11-13 22:52:45Z mosu $
 
    class definitions for the Quicktime & MP4 reader
 
@@ -56,13 +56,13 @@ struct qt_chunkmap_t {
 };
 
 struct qt_editlist_t {
-  uint32_t duration;
-  uint32_t pos;
+  uint64_t duration;
+  uint64_t pos;
   uint32_t speed;
   uint32_t frames;
-  uint32_t start_sample;
-  uint32_t start_frame;
-  uint32_t pts_offset;
+  uint64_t start_sample;
+  uint64_t start_frame;
+  int64_t pts_offset;
 
   qt_editlist_t():
     duration(0), pos(0), speed(0), frames(0),
@@ -70,7 +70,7 @@ struct qt_editlist_t {
 };
 
 struct qt_sample_t {
-  uint32_t pts;
+  uint64_t pts;
   uint32_t size;
   uint64_t pos;
 
@@ -94,12 +94,11 @@ struct qtmp4_demuxer_t {
   char fourcc[4];
   uint32_t pos;
 
-  uint32_t timescale;
-  uint32_t global_duration;
-  uint32_t avg_duration;
+  uint32_t time_scale;
+  uint64_t global_duration;
   uint32_t sample_size;
 
-  uint32_t duration;
+  uint64_t duration;
 
   vector<qt_sample_t> sample_table;
   vector<qt_chunk_t> chunk_table;
@@ -110,19 +109,23 @@ struct qtmp4_demuxer_t {
   vector<qt_frame_offset_t> raw_frame_offset_table;
   vector<int32_t> frame_offset_table;
 
+  vector<int64_t> timecodes, durations, frame_indices;
+  int64_t min_timecode, max_timecode;
+  double fps;
+
   esds_t esds;
   bool esds_parsed;
 
   unsigned char *v_stsd;
   uint32_t v_stsd_size;
   uint32_t v_width, v_height, v_bitdepth;
-  int64_t v_dts_offset;
   deque<int64_t> references;
-  bool avc_use_bframes;
-  int64_t max_timecode;
+  bool v_is_avc, avc_use_bframes;
   uint32_t a_channels, a_bitdepth;
   float a_samplerate;
   sound_v1_stsd_atom_t a_stsd;
+  int a_aac_profile, a_aac_output_sample_rate;
+  bool a_aac_is_sbr, a_aac_config_parsed;
 
   unsigned char *priv;
   uint32_t priv_size;
@@ -133,14 +136,16 @@ struct qtmp4_demuxer_t {
 
   qtmp4_demuxer_t():
     ok(false), type('?'), id(0), pos(0),
-    timescale(1), global_duration(0), avg_duration(0), sample_size(0),
-    duration(0),
+    time_scale(1), global_duration(0), //avg_duration(0),
+    sample_size(0),
+    duration(0), min_timecode(0), max_timecode(0), fps(0.0),
     esds_parsed(false),
     v_stsd(NULL), v_stsd_size(0),
-    v_width(0), v_height(0), v_bitdepth(0), v_dts_offset(0),
-    avc_use_bframes(false),
-    max_timecode(0),
+    v_width(0), v_height(0), v_bitdepth(0),
+    v_is_avc(false), avc_use_bframes(false),
     a_channels(0), a_bitdepth(0), a_samplerate(0.0),
+    a_aac_profile(0), a_aac_output_sample_rate(0),
+    a_aac_is_sbr(false), a_aac_config_parsed(false),
     priv(NULL), priv_size(0),
     warning_printed(false),
     ptzr(-1) {
@@ -157,7 +162,10 @@ struct qtmp4_demuxer_t {
     safefree(esds.sl_config);
   }
 
-  double calculate_fps();
+  void calculate_fps();
+  int64_t to_nsecs(int64_t value);
+  void calculate_timecodes();
+  void adjust_timecodes(int64_t delta);
 };
 typedef counted_ptr<qtmp4_demuxer_t> qtmp4_demuxer_ptr;
 
@@ -185,7 +193,7 @@ private:
   mm_io_c *io;
   vector<qtmp4_demuxer_ptr> demuxers;
   int64_t file_size, mdat_pos, mdat_size;
-  uint32_t compression_algorithm;
+  uint32_t time_scale, compression_algorithm;
   int main_dmx;
 
 public:
@@ -203,19 +211,19 @@ public:
 
 protected:
   virtual void parse_headers();
-  virtual qt_atom_t read_atom();
-  virtual void parse_header_priv_atoms(qtmp4_demuxer_ptr &dmx,
-                                       unsigned char *mem, int size,
-                                       int level);
+  virtual void calculate_timecodes();
+  virtual qt_atom_t read_atom(mm_io_c *read_from = NULL,
+                              bool exit_on_error = true);
+  virtual void parse_video_header_priv_atoms(qtmp4_demuxer_ptr &dmx,
+                                             unsigned char *mem, int size,
+                                             int level);
+  virtual void parse_audio_header_priv_atoms(qtmp4_demuxer_ptr &dmx,
+                                             unsigned char *mem, int size,
+                                             int level);
   virtual bool parse_esds_atom(mm_mem_io_c &memio, qtmp4_demuxer_ptr &dmx,
                                int level);
   virtual uint32_t read_esds_descr_len(mm_mem_io_c &memio);
 
-  virtual void handle_video_with_bframes(qtmp4_demuxer_ptr &dmx,
-                                         int64_t timecode, int64_t duration,
-                                         bool is_keyframe,
-                                         memory_cptr mem);
-
   virtual void handle_cmov_atom(qt_atom_t parent, int level);
   virtual void handle_cmvd_atom(qt_atom_t parent, int level);
   virtual void handle_ctts_atom(qtmp4_demuxer_ptr &new_dmx, qt_atom_t parent,
@@ -235,6 +243,8 @@ protected:
                                 int level);
   virtual void handle_stco_atom(qtmp4_demuxer_ptr &new_dmx, qt_atom_t parent,
                                 int level);
+  virtual void handle_co64_atom(qtmp4_demuxer_ptr &new_dmx, qt_atom_t parent,
+                                int level);
   virtual void handle_stsc_atom(qtmp4_demuxer_ptr &new_dmx, qt_atom_t parent,
                                 int level);
   virtual void handle_stsd_atom(qtmp4_demuxer_ptr &new_dmx, qt_atom_t parent,
@@ -251,6 +261,13 @@ protected:
                                 int level);
   virtual void handle_trak_atom(qtmp4_demuxer_ptr &new_dmx, qt_atom_t parent,
                                 int level);
+  virtual void handle_edts_atom(qtmp4_demuxer_ptr &new_dmx, qt_atom_t parent,
+                                int level);
+  virtual void handle_elst_atom(qtmp4_demuxer_ptr &new_dmx, qt_atom_t parent,
+                                int level);
+
+  virtual void update_tables(qtmp4_demuxer_ptr &dmx);
+  virtual void update_editlist_table(qtmp4_demuxer_ptr &dmx);
 };
 
 #endif  // __R_QTMP4_H
