@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_real.cpp 2951 2005-06-10 07:12:50Z mosu $
+   $Id: r_real.cpp 3530 2007-06-19 20:15:26Z mosu $
 
    RealMedia demultiplexer module
 
@@ -170,19 +170,13 @@ real_reader_c::real_reader_c(track_info_c &_ti)
 }
 
 real_reader_c::~real_reader_c() {
-  real_demuxer_t *demuxer;
-  int i, j;
+  int i;
 
   for (i = 0; i < demuxers.size(); i++) {
-    demuxer = demuxers[i];
-    if (demuxer->segments != NULL) {
-      for (j = 0; j < demuxer->segments->size(); j++)
-        safefree((*demuxer->segments)[j].data);
-      delete demuxer->segments;
-    }
+    real_demuxer_cptr &demuxer = demuxers[i];
+
     safefree(demuxer->private_data);
     safefree(demuxer->extra_data);
-    safefree(demuxer);
   }
   demuxers.clear();
   ti.private_data = NULL;
@@ -193,7 +187,6 @@ void
 real_reader_c::parse_headers() {
   uint32_t ts_size, ndx, i;
   unsigned char *ts_data;
-  real_demuxer_t *dmx;
   rmff_track_t *track;
 
   if (rmff_read_headers(file) == RMFF_ERR_OK) {
@@ -216,10 +209,7 @@ real_reader_c::parse_headers() {
       ts_data = track->mdpr_header.type_specific_data;
       ts_size = get_uint32_be(&track->mdpr_header.type_specific_size);
 
-      dmx = (real_demuxer_t *)safemalloc(sizeof(real_demuxer_t));
-      memset(dmx, 0, sizeof(real_demuxer_t));
-      dmx->track = track;
-      dmx->ptzr = -1;
+      real_demuxer_cptr dmx(new real_demuxer_t(track));
 
       if (track->type == RMFF_TRACK_TYPE_VIDEO) {
         dmx->rvp = (real_video_props_t *)track->mdpr_header.type_specific_data;
@@ -238,6 +228,7 @@ real_reader_c::parse_headers() {
 
       } else if (track->type == RMFF_TRACK_TYPE_AUDIO) {
         bool ok;
+        int version;
 
         ok = true;
 
@@ -246,7 +237,16 @@ real_reader_c::parse_headers() {
         dmx->ra5p = (real_audio_v5_props_t *)
           track->mdpr_header.type_specific_data;
 
-        if (get_uint16_be(&dmx->ra4p->version1) == 4) {
+        version = get_uint16_be(&dmx->ra4p->version1);
+
+        if (3 == version) {
+          dmx->samples_per_second = 8000;
+          dmx->channels           = 1;
+          dmx->bits_per_sample    = 16;
+          dmx->extra_data_size    = 0;
+          strcpy(dmx->fourcc, "14_4");
+
+        } else if (4 == version) {
           int slen;
           unsigned char *p;
 
@@ -275,7 +275,7 @@ real_reader_c::parse_headers() {
             }
           }
 
-        } else {
+        } else if (5 == version) {
 
           dmx->samples_per_second = get_uint16_be(&dmx->ra5p->sample_rate);
           dmx->channels = get_uint16_be(&dmx->ra5p->channels);
@@ -290,7 +290,12 @@ real_reader_c::parse_headers() {
                                           sizeof(real_audio_v5_props_t),
                                           dmx->extra_data_size);
           }
+
+        } else {
+          mxwarn(PFX "Only audio header versions 3, 4 and 5 are supported. Track ID %u uses version %d and will be skipped.\n", track->id, version);
+          ok = false;
         }
+
         mxverb(2, PFX "extra_data_size: %d\n", dmx->extra_data_size);
 
         if (ok) {
@@ -298,8 +303,7 @@ real_reader_c::parse_headers() {
           dmx->private_size = ts_size;
 
           demuxers.push_back(dmx);
-        } else
-          free(dmx);
+        }
       }
     }
   }
@@ -307,12 +311,11 @@ real_reader_c::parse_headers() {
 
 void
 real_reader_c::create_packetizer(int64_t tid) {
-  int i;
-  real_demuxer_t *dmx;
+  real_demuxer_cptr dmx;
   rmff_track_t *track;
 
   dmx = find_demuxer(tid);
-  if (dmx == NULL)
+  if (dmx.get() == NULL)
     return;
 
   if (dmx->ptzr == -1) {
@@ -349,6 +352,7 @@ real_reader_c::create_packetizer(int64_t tid) {
       } else if (!strcasecmp(dmx->fourcc, "raac") ||
                  !strcasecmp(dmx->fourcc, "racp")) {
         int profile, channels, sample_rate, output_sample_rate;
+        int detected_profile;
         uint32_t extra_len;
         bool sbr, extra_data_parsed;
 
@@ -386,13 +390,21 @@ real_reader_c::create_packetizer(int64_t tid) {
           dmx->channels = channels;
           dmx->samples_per_second = sample_rate;
         }
+        detected_profile = profile;
         if (sbr)
           profile = AAC_PROFILE_SBR;
-        for (i = 0; i < (int)ti.aac_is_sbr.size(); i++)
-          if ((ti.aac_is_sbr[i] == -1) || (ti.aac_is_sbr[i] == track->id)) {
-            profile = AAC_PROFILE_SBR;
-            break;
-          }
+
+        if ((map_has_key(ti.all_aac_is_sbr, track->id) &&
+             ti.all_aac_is_sbr[track->id]) ||
+            (map_has_key(ti.all_aac_is_sbr, -1) && ti.all_aac_is_sbr[-1]))
+          profile = AAC_PROFILE_SBR;
+
+        if ((-1 != detected_profile) &&
+            ((map_has_key(ti.all_aac_is_sbr, track->id) &&
+              !ti.all_aac_is_sbr[track->id]) ||
+             (map_has_key(ti.all_aac_is_sbr, -1) && !ti.all_aac_is_sbr[-1])))
+          profile = detected_profile;
+
         mxverb(2, PFX "2. profile: %d, channels: %d, sample_rate: "
                "%d, output_sample_rate: %d, sbr: %d\n", profile, channels,
                sample_rate, output_sample_rate, (int)sbr);
@@ -420,6 +432,9 @@ real_reader_c::create_packetizer(int64_t tid) {
         dmx->ref_timecode = -1;
 
       } else {
+        if (!strcasecmp(dmx->fourcc, "COOK"))
+          dmx->cook_audio_fix = true;
+
         ptzr = new ra_packetizer_c(this, dmx->samples_per_second,
                                    dmx->channels, dmx->bits_per_sample,
                                    (dmx->fourcc[0] << 24) |
@@ -428,7 +443,6 @@ real_reader_c::create_packetizer(int64_t tid) {
                                    dmx->private_data, dmx->private_size,
                                    ti);
         dmx->ptzr = add_packetizer(ptzr);
-        dmx->segments = new vector<rv_segment_t>;
 
         mxinfo(FMT_TID "Using the RealAudio output module (FourCC: %s).\n",
                ti.fname.c_str(), (int64_t)track->id, dmx->fourcc);
@@ -445,7 +459,7 @@ real_reader_c::create_packetizers() {
     create_packetizer(demuxers[i]->track->id);
 }
 
-real_demuxer_t *
+real_demuxer_cptr
 real_reader_c::find_demuxer(int id) {
   int i;
 
@@ -453,19 +467,20 @@ real_reader_c::find_demuxer(int id) {
     if (demuxers[i]->track->id == id)
       return demuxers[i];
 
-  return NULL;
+  return real_demuxer_cptr(NULL);
 }
 
 file_status_e
 real_reader_c::finish() {
   int i;
   int64_t dur;
-  real_demuxer_t *dmx;
+  real_demuxer_cptr dmx;
 
   for (i = 0; i < demuxers.size(); i++) {
     dmx = demuxers[i];
-    if ((dmx->track->type == RMFF_TRACK_TYPE_AUDIO) &&
-        (dmx->segments != NULL)) {
+    if ((NULL != dmx.get()) && (NULL != dmx->track) &&
+        (dmx->track->type == RMFF_TRACK_TYPE_AUDIO) &&
+        !dmx->segments.empty()) {
       dur = dmx->last_timecode / dmx->num_packets;
       deliver_audio_frames(dmx, dur);
     }
@@ -483,7 +498,7 @@ real_reader_c::read(generic_packetizer_c *,
                     bool) {
   int size;
   unsigned char *chunk;
-  real_demuxer_t *dmx;
+  real_demuxer_cptr dmx;
   int64_t timecode;
   rmff_frame_t *frame;
 
@@ -512,18 +527,31 @@ real_reader_c::read(generic_packetizer_c *,
 
   timecode = (int64_t)frame->timecode * 1000000ll;
   dmx = find_demuxer(frame->id);
-  if (dmx == NULL) {
+  if ((dmx.get() == NULL) || (-1 == dmx->ptzr)) {
     rmff_release_frame(frame);
     return FILE_STATUS_MOREDATA;
   }
 
+  if (dmx->cook_audio_fix && dmx->first_frame &&
+      ((frame->flags & RMFF_FRAME_FLAG_KEYFRAME) !=
+       RMFF_FRAME_FLAG_KEYFRAME))
+    dmx->force_keyframe_flag = true;
+
+  if (dmx->force_keyframe_flag &&
+      ((frame->flags & RMFF_FRAME_FLAG_KEYFRAME) ==
+       RMFF_FRAME_FLAG_KEYFRAME))
+    dmx->force_keyframe_flag = false;
+
+  if (dmx->force_keyframe_flag)
+    frame->flags |= RMFF_FRAME_FLAG_KEYFRAME;
+
   if (dmx->track->type == RMFF_TRACK_TYPE_VIDEO)
     assemble_video_packet(dmx, frame);
 
   else if (dmx->is_aac) {
     // If the first AAC packet does not start at 0 then let the AAC
     // packetizer adjust its data accordingly.
-    if (dmx->ref_timecode == -1) {
+    if (dmx->first_frame) {
       dmx->ref_timecode = timecode;
       PTZR(dmx->ptzr)->set_displacement_maybe(timecode);
     }
@@ -535,83 +563,83 @@ real_reader_c::read(generic_packetizer_c *,
 
   rmff_release_frame(frame);
 
+  dmx->first_frame = false;
+
   return FILE_STATUS_MOREDATA;
 }
 
 void
-real_reader_c::queue_one_audio_frame(real_demuxer_t *dmx,
+real_reader_c::queue_one_audio_frame(real_demuxer_cptr dmx,
                                      memory_c &mem,
                                      uint64_t timecode,
                                      uint32_t flags) {
-  rv_segment_t segment;
+  rv_segment_cptr segment(new rv_segment_t);
 
-  segment.data = mem.data;
-  segment.size = mem.size;
-  mem.lock();
-  segment.flags = flags;
-  dmx->segments->push_back(segment);
+  segment->data = memory_cptr(new memory_c(mem));
+  segment->flags = flags;
+  dmx->segments.push_back(segment);
   dmx->last_timecode = timecode;
-  mxverb(2, "enqueueing one for %u/'%s' length %u timecode %llu flags "
-         "0x%08x\n", dmx->track->id, ti.fname.c_str(), mem.size, timecode,
-         flags);
+  mxverb(2, "enqueueing one for %u/'%s' length %u timecode " LLU " flags "
+         "0x%08x\n", dmx->track->id, ti.fname.c_str(), mem.get_size(),
+         timecode, flags);
 }
 
 void
-real_reader_c::queue_audio_frames(real_demuxer_t *dmx,
+real_reader_c::queue_audio_frames(real_demuxer_cptr dmx,
                                   memory_c &mem,
                                   uint64_t timecode,
                                   uint32_t flags) {
   // Enqueue the packets if no packets are in the queue or if the current
   // packet's timecode is the same as the timecode of those before.
-  if ((dmx->segments->size() == 0) ||
-      (dmx->last_timecode == timecode)) {
+  if (dmx->segments.empty() || (dmx->last_timecode == timecode)) {
     queue_one_audio_frame(dmx, mem, timecode, flags);
     return;
   }
 
   // This timecode is different. So let's push the packets out.
   deliver_audio_frames(dmx, (timecode - dmx->last_timecode) /
-                       dmx->segments->size());
+                       dmx->segments.size());
   // Enqueue this packet.
   queue_one_audio_frame(dmx, mem, timecode, flags);
 }
 
 void
-real_reader_c::deliver_audio_frames(real_demuxer_t *dmx,
+real_reader_c::deliver_audio_frames(real_demuxer_cptr dmx,
                                     uint64_t duration) {
   uint32_t i;
-  rv_segment_t segment;
+  rv_segment_cptr segment;
 
-  if (dmx->segments->size() == 0)
+  if (dmx->segments.empty() || (-1 == dmx->ptzr))
     return;
 
-  for (i = 0; i < dmx->segments->size(); i++) {
-    segment = (*dmx->segments)[i];
-    mxverb(2, "delivering audio for %u/'%s' length %llu timecode %llu flags "
-           "0x%08x duration %llu\n", dmx->track->id, ti.fname.c_str(),
-           segment.size, dmx->last_timecode, (uint32_t)segment.flags,
-           duration);
-    PTZR(dmx->ptzr)->process(new packet_t(new memory_c(segment.data,
-                                                       segment.size, true),
+  for (i = 0; i < dmx->segments.size(); i++) {
+    segment = dmx->segments[i];
+    mxverb(2, "delivering audio for %u/'%s' length %d timecode " LLU " flags "
+           "0x%08x duration " LLU "\n", dmx->track->id, ti.fname.c_str(),
+           segment->data->get_size(), dmx->last_timecode,
+           (uint32_t)segment->flags, duration);
+    PTZR(dmx->ptzr)->process(new packet_t(segment->data,
                                           dmx->last_timecode, duration,
-                                          (segment.flags & 2) == 2 ? -1 :
+                                          (segment->flags &
+                                           RMFF_FRAME_FLAG_KEYFRAME) ==
+                                          RMFF_FRAME_FLAG_KEYFRAME ? -1 :
                                           dmx->ref_timecode));
-    if ((segment.flags & 2) == 2)
+    if ((segment->flags & 2) == 2)
       dmx->ref_timecode = dmx->last_timecode;
   }
-  dmx->num_packets += dmx->segments->size();
-  dmx->segments->clear();
+  dmx->num_packets += dmx->segments.size();
+  dmx->segments.clear();
 }
 
 void
-real_reader_c::deliver_aac_frames(real_demuxer_t *dmx,
+real_reader_c::deliver_aac_frames(real_demuxer_cptr dmx,
                                   memory_c &mem) {
   uint32_t num_sub_packets, data_idx, i, sub_length, len_check;
   uint32_t length;
   unsigned char *chunk;
 
-  chunk = mem.data;
-  length = mem.size;
+  chunk = mem.get();
+  length = mem.get_size();
   if (length < 2) {
     mxwarn(PFX "Short AAC audio packet for track ID %u of "
            "'%s' (length: %u < 2)\n", dmx->track->id, ti.fname.c_str(),
@@ -655,7 +683,7 @@ real_reader_c::get_progress() {
 void
 real_reader_c::identify() {
   int i;
-  real_demuxer_t *demuxer;
+  real_demuxer_cptr demuxer;
 
   mxinfo("File '%s': container: RealMedia\n", ti.fname.c_str());
   for (i = 0; i < demuxers.size(); i++) {
@@ -671,7 +699,7 @@ real_reader_c::identify() {
 }
 
 void
-real_reader_c::assemble_video_packet(real_demuxer_t *dmx,
+real_reader_c::assemble_video_packet(real_demuxer_cptr dmx,
                                      rmff_frame_t *frame) {
   int result;
   rmff_frame_t *assembled;
@@ -688,7 +716,7 @@ real_reader_c::assemble_video_packet(real_demuxer_t *dmx,
       set_dimensions(dmx, assembled->data, assembled->size);
     packet_t *packet =
       new packet_t(new memory_c(assembled->data, assembled->size, true),
-                   (int64_t)assembled->timecode * 1000000, -1,
+                   (int64_t)assembled->timecode * 1000000, 0,
                    (assembled->flags & RMFF_FRAME_FLAG_KEYFRAME) ==
                    RMFF_FRAME_FLAG_KEYFRAME ? VFT_IFRAME :
                    VFT_PFRAMEAUTOMATIC, VFT_NOBFRAME);
@@ -710,45 +738,45 @@ real_reader_c::get_rv_dimensions(unsigned char *buf,
   uint32_t w, h, c, v;
   bit_cursor_c bc(buf, size);
 
-  bc.skip_bits(13);
-  bc.skip_bits(13);
-  if (!bc.get_bits(3, v))
-    return false;
+  try {
+    bc.skip_bits(13);
+    bc.skip_bits(13);
+    v = bc.get_bits(3);
 
-  w = cw[v];
-  if (w == 0) {
-    do {
-      if (!bc.get_bits(8, c))
-        return false;
-      w += (c << 2);
-    } while (c == 255);
-  }
-
-  if (!bc.get_bits(3, c))
-    return false;
-  h = ch1[c];
-  if (h == 0) {
-    if (!bc.get_bits(1, v))
-      return false;
-    c = ((c << 1) | v) & 3;
-    h = ch2[c];
-    if (h == 0) {
+    w = cw[v];
+    if (w == 0) {
       do {
-        if (!bc.get_bits(8, c))
-          return false;
-        h += (c << 2);
+        c = bc.get_bits(8);
+        w += (c << 2);
       } while (c == 255);
     }
+
+    c = bc.get_bits(3);
+    h = ch1[c];
+    if (h == 0) {
+      v = bc.get_bits(1);
+      c = ((c << 1) | v) & 3;
+      h = ch2[c];
+      if (h == 0) {
+        do {
+          c = bc.get_bits(8);
+          h += (c << 2);
+        } while (c == 255);
+      }
+    }
+
+    width = w;
+    height = h;
+
+    return true;
+
+  } catch (...) {
+    return false;
   }
-
-  width = w;
-  height = h;
-
-  return true;
 }
 
 void
-real_reader_c::set_dimensions(real_demuxer_t *dmx,
+real_reader_c::set_dimensions(real_demuxer_cptr dmx,
                               unsigned char *buffer,
                               int size) {
   uint32_t width, height, disp_width, disp_height;
@@ -815,7 +843,7 @@ real_reader_c::set_dimensions(real_demuxer_t *dmx,
 void
 real_reader_c::get_information_from_data() {
   int i;
-  real_demuxer_t *dmx;
+  real_demuxer_cptr dmx;
   bool information_found;
   rmff_frame_t *frame;
   int64_t old_pos;
@@ -834,7 +862,7 @@ real_reader_c::get_information_from_data() {
     frame = rmff_read_next_frame(file, NULL);
     dmx = find_demuxer(frame->id);
 
-    if (dmx == NULL) {
+    if (dmx.get() == NULL) {
       rmff_release_frame(frame);
       continue;
     }
