@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_vobsub.cpp 3024 2005-08-17 11:46:09Z mosu $
+   $Id: r_vobsub.cpp 3524 2007-04-19 07:50:15Z mosu $
 
    VobSub stream reader
 
@@ -35,32 +35,8 @@ using namespace std;
                      tolower(c) == 'c' ? 12 : \
                      tolower(c) == 'd' ? 13 : \
                      tolower(c) == 'e' ? 14 : 15)
-#define istimecodestr(s)       (!strncmp(s, "timecode: ", 10))
-#define istimestampstr(s)      (!strncmp(s, "timestamp: ", 11))
-#define iscommafileposstr(s)   (!strncmp(s, ", filepos: ", 11))
-#define iscolon(s)             (*(s) == ':')
-#define istwodigits(s)         (isdigit(*(s)) && isdigit(*(s + 1)))
-#define isthreedigits(s)       (isdigit(*(s)) && isdigit(*(s + 1)) && \
-                                isdigit(*(s + 2)))
-#define istwodigitscolon(s)    (istwodigits(s) && iscolon(s + 2))
-#define istimecode(s)         (istwodigitscolon(s) && \
-                                istwodigitscolon(s + 3) && \
-                                istwodigitscolon(s + 6) && \
-                                isthreedigits(s + 9))
 #define ishexdigit(s)          (isdigit(s) || \
                                 (strchr("abcdefABCDEF", s) != NULL))
-#define isfilepos(s)           (ishexdigit(*(s)) && ishexdigit(*(s + 1)) && \
-                                ishexdigit(*(s + 2)) && \
-                                ishexdigit(*(s + 3)) && \
-                                ishexdigit(*(s + 4)) && \
-                                ishexdigit(*(s + 5)) && \
-                                ishexdigit(*(s + 6)) && \
-                                ishexdigit(*(s + 7)) && \
-                                ishexdigit(*(s + 8)))
-#define isvobsubline_v7(s)     ((strlen(s) >= 42) && \
-                                istimestampstr(s) && istimecode(s + 11) && \
-                                iscommafileposstr(s + 23) && \
-                                isfilepos(s + 34))
 
 #define PFX "vobsub_reader: "
 
@@ -90,7 +66,9 @@ vobsub_reader_c::probe_file(mm_io_c *io,
 
 vobsub_reader_c::vobsub_reader_c(track_info_c &_ti)
   throw (error_c):
-  generic_reader_c(_ti) {
+  generic_reader_c(_ti),
+  delay(0) {
+
   string sub_name, line;
   int len;
 
@@ -145,8 +123,8 @@ vobsub_reader_c::~vobsub_reader_c() {
   uint32_t i;
 
   for (i = 0; i < tracks.size(); i++) {
-    mxverb(2, "r_vobsub track %u SPU size: %lld, overall size: %lld, "
-           "overhead: %lld (%.3f%%)\n", i, tracks[i]->spu_size,
+    mxverb(2, "r_vobsub track %u SPU size: " LLD ", overall size: " LLD ", "
+           "overhead: " LLD " (%.3f%%)\n", i, tracks[i]->spu_size,
            tracks[i]->spu_size + tracks[i]->overhead,
            tracks[i]->overhead, (float)(100.0 * tracks[i]->overhead /
                                         (tracks[i]->overhead +
@@ -161,19 +139,13 @@ void
 vobsub_reader_c::create_packetizer(int64_t tid) {
   uint32_t k;
   int64_t avg_duration, duration;
-  char language[4];
-  const char *c;
   vobsub_track_c *track;
 
   if ((tid < tracks.size()) && demuxing_requested('s', tid) &&
       (tracks[tid]->ptzr == -1)) {
     track = tracks[tid];
     ti.id = tid;
-    if ((c = map_iso639_1_to_iso639_2(tracks[tid]->language)) != NULL) {
-      strcpy(language, c);
-      ti.language = language;
-    } else
-      ti.language = "";
+    ti.language = tracks[tid]->language;
     track->ptzr =
       add_packetizer(new vobsub_packetizer_c(this, idx_data.c_str(),
                                              idx_data.length(), ti));
@@ -195,7 +167,7 @@ vobsub_reader_c::create_packetizer(int64_t tid) {
     num_indices += track->entries.size();
 
     mxinfo(FMT_TID "Using the VobSub subtitle output module (language: %s).\n",
-           ti.fname.c_str(), (int64_t)tid, track->language);
+           ti.fname.c_str(), (int64_t)tid, track->language.c_str());
     ti.language = "";
   }
 }
@@ -212,10 +184,11 @@ void
 vobsub_reader_c::parse_headers() {
   string line;
   const char *sline;
-  char language[3];
+  string language;
+  vector<string> parts;
   vobsub_track_c *track;
   int64_t filepos, timestamp, line_no, last_timestamp;
-  int hour, minute, second, msecond, idx;
+  int factor, idx, lang_index;
   uint32_t i, k, tsize;
   vobsub_entry_c entry;
   bool sort_required;
@@ -240,11 +213,16 @@ vobsub_reader_c::parse_headers() {
 
     if (!strncasecmp(sline, "id:", 3)) {
       if (line.length() >= 6) {
-        language[0] = sline[4];
-        language[1] = sline[5];
-        language[2] = 0;
+        language = sline[4];
+        language += sline[5];
+        lang_index = map_to_iso639_2_code(language.c_str());
+        if (-1 != lang_index)
+          language = iso639_languages[lang_index].iso639_2_code;
+        else
+          language = "";
       } else
-        language[0] = 0;
+        language = "";
+
       if (track != NULL) {
         if (track->entries.size() == 0)
           delete track;
@@ -266,40 +244,86 @@ vobsub_reader_c::parse_headers() {
         !strncasecmp(sline, "langidx:", 8))
       continue;
 
-    if ((version == 7) && isvobsubline_v7(sline)) {
+    if (starts_with_case(line, "delay:")) {
+      line.erase(0, 6);
+      strip(line);
+
+      if (!parse_timecode(line, timestamp, true))
+        mxerror(PFX "'%s', line " LLD ": The 'delay' timestamp could not be "
+                "parsed.\n", ti.fname.c_str(), line_no);
+      delay = timestamp;
+    }
+
+    if ((version == 7) && starts_with_case(line, "timestamp:")) {
       if (track == NULL)
         mxerror(PFX ".idx file does not contain an 'id: ...' line to indicate "
                 "the language.\n");
 
-      idx = 34;
+      strip(line);
+      shrink_whitespace(line);
+      parts = split(line.c_str(), " ");
+//       for (int asd = 0; asd < parts.size(); asd++) {
+//         mxinfo("ASD: %s ", parts[asd].c_str());
+//       }
+//       mxinfo("\n");
+//       mxexit(0);
+      if ((4 != parts.size()) ||
+          (13 > parts[1].length()) ||
+          (downcase(parts[2]) != "filepos:")) {
+        mxwarn(PFX "'%s', line " LLD ": The line seems to be a subtitle entry "
+               "but the format couldn't be recognized. This entry will be "
+               "skipped.\n", ti.fname.c_str(), line_no);
+        continue;
+      }
+
+      idx = 0;
+      sline = parts[3].c_str();
       filepos = hexvalue(sline[idx]);
       idx++;
-      while ((idx < line.length()) && ishexdigit(sline[idx])) {
-        filepos = filepos * 16 + hexvalue(sline[idx]);
+      while ((0 != sline[idx]) && ishexdigit(sline[idx])) {
+        filepos = (filepos << 4) + hexvalue(sline[idx]);
         idx++;
       }
       entry.position = filepos;
 
-      sscanf(&sline[11], "%02d:%02d:%02d:%03d", &hour, &minute, &second,
-             &msecond);
-      timestamp = (int64_t)hour * 60 * 60 * 1000 +
-        (int64_t)minute * 60 * 1000 + (int64_t)second * 1000 +
-        (int64_t)msecond;
-      entry.timestamp = timestamp * 1000000;
+      parts[1].erase(parts[1].length() - 1);
+      factor = 1;
+      if ('-' == parts[1][0]) {
+        factor = -1;
+        parts[1].erase(0, 1);
+      }
+
+      if (!parse_timecode(parts[1], timestamp)) {
+        mxwarn(PFX "'%s', line " LLD ": The line seems to be a subtitle entry "
+               "but the format couldn't be recognized. This entry will be "
+               "skipped.\n", ti.fname.c_str(), line_no);
+        continue;
+      }
+
+      entry.timestamp = timestamp * factor + delay;
+
+      if (0 > entry.timestamp) {
+        mxwarn(PFX "'%s', line " LLD ": The line seems to be a subtitle entry "
+               "but the timecode was negative even after adding the track "
+               "delay. Negative timecodes are not supported in Matroska. "
+               "This entry will be skipped.\n", ti.fname.c_str(), line_no);
+        continue;
+      }
+
       track->entries.push_back(entry);
 
-      if ((timestamp < last_timestamp) &&
+      if ((entry.timestamp < last_timestamp) &&
           demuxing_requested('s', tracks.size())) {
-        mxwarn(PFX "'%s', line %lld: The current timestamp (" FMT_TIMECODE
+        mxwarn(PFX "'%s', line " LLD ": The current timestamp (" FMT_TIMECODE
                ") is smaller than the last one (" FMT_TIMECODE"). mkvmerge "
                "will sort the entries according to their timestamps. This "
                "might result in the wrong order for some subtitle entries. If "
                "this is the case then you have to fix the .idx file "
                "manually.\n", ti.fname.c_str(), line_no,
-               ARG_TIMECODE(timestamp), ARG_TIMECODE(last_timestamp));
+               ARG_TIMECODE(entry.timestamp), ARG_TIMECODE(last_timestamp));
         sort_required = true;
       }
-      last_timestamp = timestamp;
+      last_timestamp = entry.timestamp;
 
       continue;
     }
@@ -324,8 +348,8 @@ vobsub_reader_c::parse_headers() {
     for (i = 0; i < tsize; i++) {
       mxinfo("vobsub_reader: Track number %u\n", i);
       for (k = 0; k < tracks[i]->entries.size(); k++)
-        mxinfo("vobsub_reader:  %04u position: %12lld (0x%04x%08x), "
-               "timecode: %12lld (" FMT_TIMECODE ")\n", k,
+        mxinfo("vobsub_reader:  %04u position: %12" PRId64 " (0x%04x%08x), "
+               "timecode: %12" PRId64 " (" FMT_TIMECODE ")\n", k,
                tracks[i]->entries[k].position,
                (uint32_t)(tracks[i]->entries[k].position >> 32),
                (uint32_t)(tracks[i]->entries[k].position & 0xffffffff),
@@ -353,7 +377,7 @@ vobsub_reader_c::deliver_packet(unsigned char *buf,
   duration = spu_extract_duration(buf, size, timecode);
   if (duration == -1) {
     mxverb(2, PFX "Could not extract the duration for a SPU packet in track "
-           "%lld of '%s' (timecode: " FMT_TIMECODE ").\n", ti.id,
+           "" LLD " of '%s' (timecode: " FMT_TIMECODE ").\n", ti.id,
            ti.fname.c_str(), ARG_TIMECODE(timecode));
     duration = default_duration;
   }
@@ -375,7 +399,7 @@ vobsub_reader_c::extract_one_spu_packet(int64_t timecode,
   uint32_t len, idx, version, packet_size, dst_size;
   int64_t extraction_start_pos;
   int c, packet_aid, spu_len;
-  float pts;
+  int64_t pts;
   /* Goto start of a packet, it starts with 0x000001?? */
   const unsigned char wanted[] = { 0, 0, 1 };
   unsigned char buf[5];
@@ -384,7 +408,7 @@ vobsub_reader_c::extract_one_spu_packet(int64_t timecode,
   track = tracks[track_id];
   extraction_start_pos = sub_file->getFilePointer();
 
-  pts = 0.0;
+  pts = 0;
   track->packet_num++;
 
   dst_buf = NULL;
@@ -418,8 +442,8 @@ vobsub_reader_c::extract_one_spu_packet(int64_t timecode,
           version = 2;
         else {
           if (!track->mpeg_version_warning_printed) {
-            mxwarn(PFX "Unsupported MPEG version: 0x%02x in packet %lld for "
-                   "track %lld for timecode " FMT_TIMECODE ", assuming "
+            mxwarn(PFX "Unsupported MPEG version: 0x%02x in packet " LLD
+                   "for track " LLD " for timecode " FMT_TIMECODE ", assuming "
                    "MPEG2. No further warnings will be printed for this "
                    "track.\n", c, track->packet_num, track_id,
                    ARG_TIMECODE_NS(timecode));
@@ -482,8 +506,10 @@ vobsub_reader_c::extract_one_spu_packet(int64_t timecode,
                      buf[0], buf[1], buf[2], buf[3], buf[4]);
               pts = 0;
             } else
-              pts = ((buf[0] & 0x0e) << 29 | buf[1] << 22 |
-                     (buf[2] & 0xfe) << 14 | buf[3] << 7 | (buf[4] >> 1));
+              pts =
+                ((int64_t)((buf[0] & 0x0e) << 29 | buf[1] << 22 |
+                           (buf[2] & 0xfe) << 14 | buf[3] << 7 |
+                           (buf[4] >> 1))) * 100000 / 9;
           }
           sub_file->setFilePointer2(dataidx + extraction_start_pos,
                                     seek_beginning);
@@ -499,7 +525,7 @@ vobsub_reader_c::extract_one_spu_packet(int64_t timecode,
           else if (track->aid != packet_aid) {
             // The packet does not belong to the current subtitle stream.
             mxverb(3, PFX "skipping sub packet with aid %d (wanted aid: %d) "
-                   "with size %d at %lld\n", packet_aid, track->aid,
+                   "with size %d at " LLD "\n", packet_aid, track->aid,
                    packet_size, sub_file->getFilePointer() -
                    extraction_start_pos);
             sub_file->skip(packet_size);
@@ -508,8 +534,9 @@ vobsub_reader_c::extract_one_spu_packet(int64_t timecode,
           }
           dst_buf = (unsigned char *)saferealloc(dst_buf, dst_size +
                                                  packet_size);
-          mxverb(3, PFX "sub packet data: aid: %d, pts: %.3fs, packet_size: "
-                 "%u\n", track->aid, pts, packet_size);
+          mxverb(3, PFX "sub packet data: aid: %d, pts: " FMT_TIMECODE
+                 ", packet_size: %u\n", track->aid, ARG_TIMECODE_NS(pts),
+                 packet_size);
           if (sub_file->read(&dst_buf[dst_size], packet_size) != packet_size) {
             mxwarn(PFX "sub_file->read failure");
             return deliver();
@@ -561,7 +588,7 @@ vobsub_reader_c::read(generic_packetizer_c *ptzr,
 
   track = NULL;
   for (i = 0; i < tracks.size(); i++)
-    if (PTZR(tracks[i]->ptzr) == ptzr) {
+    if ((-1 != tracks[i]->ptzr) && (PTZR(tracks[i]->ptzr) == ptzr)) {
       track = tracks[i];
       break;
     }
@@ -592,18 +619,12 @@ void
 vobsub_reader_c::identify() {
   uint32_t i;
   string info;
-  const char *language;
 
   mxinfo("File '%s': container: VobSub\n", ti.fname.c_str());
   for (i = 0; i < tracks.size(); i++) {
-    if (identify_verbose) {
-      language = map_iso639_1_to_iso639_2(tracks[i]->language);
-      if (language != NULL)
-        info = " [language:" + string(language) + "]";
-      else
-        info = "";
-    } else
-      info = "";
+    info = "";
+    if (identify_verbose && (tracks[i]->language != ""))
+      info = string(" [language:") + tracks[i]->language + "]";
     mxinfo("Track ID %u: subtitles (VobSub)%s\n", i, info.c_str());
   }
 }
