@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_vobsub.h 2809 2005-02-24 21:05:35Z mosu $
+   $Id: r_vobsub.h 3173 2006-01-17 17:16:32Z mosu $
 
    class definitions for the VobSub stream reader
 
@@ -36,7 +36,7 @@ public:
 
 class vobsub_track_c {
 public:
-  char *language;
+  string language;
   int ptzr;
   vector<vobsub_entry_c> entries;
   int idx, aid;
@@ -44,18 +44,11 @@ public:
   int64_t packet_num, spu_size, overhead;
 
 public:
-  vobsub_track_c(const char *new_language) {
-    language = safestrdup(new_language);
-    ptzr = -1;
-    idx = 0;
-    mpeg_version_warning_printed = false;
-    packet_num = 0;
-    aid = -1;
-    spu_size = 0;
-    overhead = 0;
-  };
-  ~vobsub_track_c() {
-    safefree(language);
+  vobsub_track_c(const string &new_language):
+    language(new_language),
+    ptzr(-1), idx(0), aid(-1),
+    mpeg_version_warning_printed(false),
+    packet_num(0), spu_size(0), overhead(0) {
   }
 };
 
@@ -64,7 +57,7 @@ private:
   mm_io_c *sub_file;
   mm_text_io_c *idx_file;
   int version, ifo_data_size;
-  int64_t num_indices, indices_processed;
+  int64_t num_indices, indices_processed, delay;
   string idx_data;
 
   vector<vobsub_track_c *> tracks;
