@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: cluster_helper.h 2945 2005-06-05 14:39:57Z mosu $
+   $Id: cluster_helper.h 3521 2007-03-22 09:06:14Z mosu $
 
    class definition for the cluster helper
 
@@ -23,6 +23,7 @@
 #include <matroska/KaxBlock.h>
 #include <matroska/KaxCluster.h>
 
+#include "libmatroska_extensions.h"
 #include "mm_io.h"
 #include "pr_generic.h"
 
@@ -32,24 +33,8 @@ using namespace std;
                                     (double)((int64_t)timecode_scale)) * \
                                (int64_t)timecode_scale)
 
-struct ch_contents_t {
-  KaxCluster *cluster;
-  vector<packet_cptr> packets;
-  bool is_referenced, rendered;
-
-  ch_contents_t():
-    cluster(NULL),
-    is_referenced(false),
-    rendered(false) {
-  }
-
-  ~ch_contents_t() {
-    delete cluster;
-  }
-};
-
 struct render_groups_t {
-  vector<KaxBlockGroup *> groups;
+  vector<kax_block_blob_c *> groups;
   vector<int64_t> durations;
   generic_packetizer_c *source;
   bool more_data, duration_mandatory;
@@ -73,12 +58,14 @@ struct split_point_t {
 
 class cluster_helper_c {
 private:
-  vector<ch_contents_t *> clusters;
+  kax_cluster_c *cluster;
+  vector<packet_cptr> packets;
   int cluster_content_size;
-  int64_t max_timecode_and_duration;
+  int64_t max_timecode_and_duration, max_video_timecode_rendered;
   int64_t last_cluster_tc, num_cue_elements, header_overhead;
   int64_t packet_num, timecode_offset, *last_packets;
   int64_t bytes_in_file, first_timecode_in_file;
+  int64_t min_timecode_in_cluster, max_timecode_in_cluster;
   mm_io_c *out;
 
   vector<split_point_t> split_points;
@@ -89,15 +76,13 @@ public:
   virtual ~cluster_helper_c();
 
   void set_output(mm_io_c *nout);
-  void add_cluster(KaxCluster *cluster);
-  KaxCluster *get_cluster();
+  void prepare_new_cluster();
+  KaxCluster *get_cluster() {
+    return cluster;
+  }
   void add_packet(packet_cptr packet);
   int64_t get_timecode();
-  packet_cptr get_packet(int num);
-  int get_packet_count();
-  int render(bool flush = false);
-  int free_ref(int64_t ref_timecode, generic_packetizer_c *source);
-  int free_clusters();
+  int render();
   int get_cluster_content_size();
   int64_t get_duration();
   int64_t get_first_timecode_in_file() {
@@ -109,17 +94,13 @@ public:
     return !split_points.empty();
   }
 
+  int get_packet_count() {
+    return packets.size();
+  }
+
 private:
-  int find_cluster(KaxCluster *cluster);
-  ch_contents_t *find_packet_cluster(int64_t ref_timecode,
-                                     generic_packetizer_c *source);
-  packet_cptr find_packet(int64_t ref_timecode,
-                          generic_packetizer_c *source);
-  void free_contents(ch_contents_t *clstr);
-  void check_clusters(int num);
-  bool all_references_resolved(ch_contents_t *cluster);
   void set_duration(render_groups_t *rg);
-  int render_cluster(ch_contents_t *clstr);
+  bool must_duration_be_set(render_groups_t *rg, packet_cptr &new_packet);
 };
 
 extern cluster_helper_c *cluster_helper;
