@@ -8,7 +8,7 @@
    or visit http://www.gnu.org/copyleft/gpl.html
 
    \file
-   \version $Id: mkvmerge.cpp 3421 2007-01-04 08:07:17Z mosu $
+   \version $Id: mkvmerge.cpp 3579 2007-08-19 08:53:32Z mosu $
 
    \author Written by Moritz Bunkus <moritz@bunkus.org>.
    \author Modified by Steve Lhomme <steve.lhomme@free.fr>.
@@ -209,7 +209,9 @@ set_usage() {
     "  --delay <TID:Xs|ms|us|ns>\n"
     "                           Delay to apply to the packets of the track\n"
     "                           by simply adjusting the timecodes.\n"
-    "  --default-track <TID>    Sets the 'default' flag for this track.\n"
+    "  --default-track <TID[:bool]>\n"
+    "                           Sets the 'default' flag for this track or\n"
+    "                           forces it not to be present if bool is 0.\n"
     "  --blockadd <TID:x>       Sets the max number of block additional\n"
     "                           levels for this track.\n"
     "  --track-name <TID:name>  Sets the name for a track.\n"
@@ -227,7 +229,7 @@ set_usage() {
     "\n"
     "  --nalu-size-length <TID:n>\n"
     "                           Force the NALU size length to n bytes with\n"
-    "                           2 <= n <= 4.\n"
+    "                           2 <= n <= 4 with 4 being the default.\n"
     "\n Options that only apply to video tracks:\n"
     "  -f, --fourcc <FOURCC>    Forces the FourCC to the specified value.\n"
     "                           Works only for video tracks.\n"
@@ -361,6 +363,11 @@ identify(const string &filename) {
   get_file_type(file);
   ti.fname = file.name;
 
+  if (FILE_TYPE_MPEG_TS == file.type)
+    mxerror(_("The file '%s' has been detected as a MPEG transport stream. "
+              "This file format is not supported by mkvmerge.\n"),
+            file.name.c_str());
+
   if (file.type == FILE_TYPE_IS_UNKNOWN)
     mxerror(_("File %s has unknown type. Please have a look "
               "at the supported file types ('mkvmerge --list-types') and "
@@ -432,7 +439,7 @@ parse_number_with_unit(const string &s,
     else if (29.97 == d_value)
       return (int64_t)(100100000.0 / 3.0);
     else if (23.976 == d_value)
-      return 40040000;
+      return (int64_t)(1001000000.0 / 24.0);
     else
       return (int64_t)(1000000000.0 / d_value);
   } else if (unit != "s")
@@ -446,6 +453,23 @@ parse_number_with_unit(const string &s,
     return (int64_t)(multiplier * d_value);
 }
 
+/** \brief Parse a string for a boolean value
+
+   Interpretes the string \c orig as a boolean value. Accepted
+   is "true", "yes", "1" as boolean true and "false", "no", "0"
+   as boolean false.
+*/
+bool
+parse_bool(const string &orig) {
+  string s(downcase(orig));
+
+  if ((s == "yes") || (s == "true") || (s == "1"))
+    return true;
+  if ((s == "no") || (s == "false") || (s == "0"))
+    return false;
+  throw false;
+}
+
 /** \brief Parse tags and add them to the list of all tags
 
    Also tests the tags for missing mandatory elements.
@@ -910,6 +934,36 @@ parse_delay(const string &s,
                                                 s);
 }
 
+/** \brief Parse the \c --default-track argument
+
+   The argument must have the form \c TID or \c TID:boolean. The former
+   is equivalent to \c TID:1.
+*/
+static void
+parse_default_track(const string &s,
+                    track_info_c &ti) {
+  vector<string> parts;
+  int64_t id;
+  bool is_default = true;
+
+  // Extract the track number.
+  parts = split(s, ":", 2);
+  strip(parts);
+  id = 0;
+  if (!parse_int(parts[0], id))
+    mxerror(_("Invalid track ID specified in '--default-track %s'.\n"),
+            s.c_str());
+
+  try {
+    if (2 == parts.size())
+      is_default = parse_bool(parts[1]);
+  } catch (...) {
+    mxerror(_("Invalid boolean option specified in '--default-track %s'.\n"),
+            s.c_str());
+  }
+  ti.default_track_flags[id] = is_default;
+}
+
 /** \brief Parse the \c --cues argument
 
    The argument must have the form \c TID:cuestyle, e.g. \c 0:none.
@@ -1232,6 +1286,7 @@ parse_default_duration(const string &s,
 static void
 parse_nalu_size_length(const string &s,
                        track_info_c &ti) {
+  static bool nalu_size_length_3_warning_printed = false;
   vector<string> parts;
   int64_t id, nalu_size_length;
 
@@ -1249,6 +1304,13 @@ parse_nalu_size_length(const string &s,
       (2 > nalu_size_length) || (4 < nalu_size_length))
     mxerror(_("The NALU size length must be a number between 2 and 4 "
               "inclusively in '--nalu-size-length %s'.\n"), s.c_str());
+
+  if ((3 == nalu_size_length) && !nalu_size_length_3_warning_printed) {
+    nalu_size_length_3_warning_printed = true;
+    mxwarn("Using a NALU size length of 3 bytes might result in tracks "
+           "that won't be decodable with certain AVC/h.264 codecs.\n");
+  }
+
   ti.nalu_size_lengths[id] = nalu_size_length;
 }
 
@@ -1390,15 +1452,15 @@ handle_segmentinfo(KaxInfo *kax_info_chap) {
 
   uid = FINDFIRST(kax_info_chap, KaxSegmentUID);
   if (NULL != uid)
-    seguid_forced = new bitvalue_c(*uid);
+    forced_seguids.push_back(bitvalue_cptr(new bitvalue_c(*uid)));
 
   uid = FINDFIRST(kax_info_chap, KaxNextUID);
   if (NULL != uid)
-    seguid_link_next = new bitvalue_c(*uid);
+    seguid_link_next = bitvalue_cptr(new bitvalue_c(*uid));
 
   uid = FINDFIRST(kax_info_chap, KaxPrevUID);
   if (NULL != uid)
-    seguid_link_previous = new bitvalue_c(*uid);
+    seguid_link_previous = bitvalue_cptr(new bitvalue_c(*uid));
 }
 
 /** \brief Parses and handles command line arguments
@@ -1423,7 +1485,6 @@ parse_args(vector<string> args) {
   vector<string>::const_iterator sit;
   string this_arg, next_arg;
   bool no_next_arg, inputs_found;
-  int64_t id;
   attachment_t attachment;
   mm_io_c *io;
 
@@ -1584,12 +1645,12 @@ parse_args(vector<string> args) {
       if ((no_next_arg) || (next_arg[0] == 0))
         mxerror(_("'--link-to-previous' lacks the next UID.\n"));
 
-      if (seguid_link_previous != NULL)
+      if (NULL != seguid_link_previous.get())
         mxerror(_("The previous UID was already given in '%s %s'.\n"),
                 this_arg.c_str(), next_arg.c_str());
 
       try {
-        seguid_link_previous = new bitvalue_c(next_arg, 128);
+        seguid_link_previous = bitvalue_cptr(new bitvalue_c(next_arg, 128));
       } catch (...) {
         mxerror(_("Unknown format for the previous UID in '%s %s'.\n"),
                 this_arg.c_str(), next_arg.c_str());
@@ -1601,12 +1662,12 @@ parse_args(vector<string> args) {
       if ((no_next_arg) || (next_arg[0] == 0))
         mxerror(_("'--link-to-next' lacks the previous UID.\n"));
 
-      if (seguid_link_next != NULL)
+      if (NULL != seguid_link_next.get())
         mxerror(_("The next UID was already given in '%s %s'.\n"),
                 this_arg.c_str(), next_arg.c_str());
 
       try {
-        seguid_link_next = new bitvalue_c(next_arg, 128);
+        seguid_link_next = bitvalue_cptr(new bitvalue_c(next_arg, 128));
       } catch (...) {
         mxerror(_("Unknown format for the next UID in '%s %s'.\n"),
                 this_arg.c_str(), next_arg.c_str());
@@ -1969,13 +2030,9 @@ parse_args(vector<string> args) {
 
     } else if (this_arg == "--default-track") {
       if (no_next_arg)
-        mxerror(_("'--default-track' lacks the track ID.\n"));
+        mxerror(_("'--default-track' lacks its argument.\n"));
 
-      if (!parse_int(next_arg, id))
-        mxerror(_("Invalid track ID specified in '%s %s'.\n"),
-                this_arg.c_str(), next_arg.c_str());
-
-      ti->default_track_flags.push_back(id);
+      parse_default_track(next_arg, *ti);
       sit++;
 
     } else if (this_arg == "--language") {
@@ -2124,6 +2181,11 @@ parse_args(vector<string> args) {
       get_file_type(file);
       ti->fname = this_arg;
 
+      if (FILE_TYPE_MPEG_TS == file.type)
+        mxerror(_("The file '%s' has been detected as a MPEG transport stream. "
+                  "This file format is not supported by mkvmerge.\n"),
+                file.name.c_str());
+
       if (file.type == FILE_TYPE_IS_UNKNOWN)
         mxerror(_("The file '%s' has unknown type. Please have a look "
                   "at the supported file types ('mkvmerge --list-types') and "
