@@ -8,7 +8,7 @@
    or visit http://www.gnu.org/copyleft/gpl.html
 
    \file
-   \version $Id: output_control.cpp 3022 2005-08-15 20:47:56Z mosu $
+   \version $Id: output_control.cpp 3579 2007-08-19 08:53:32Z mosu $
 
    \author Written by Moritz Bunkus <moritz@bunkus.org>.
    \author Modified by Steve Lhomme <steve.lhomme@free.fr>.
@@ -73,6 +73,7 @@
 #include "output_control.h"
 #include "r_aac.h"
 #include "r_ac3.h"
+#include "r_avc.h"
 #include "r_avi.h"
 #include "r_dts.h"
 #include "r_flac.h"
@@ -90,6 +91,7 @@
 #include "r_vobsub.h"
 #include "r_wav.h"
 #include "r_wavpack.h"
+#include "r_corepicture.h"
 #include "tag_common.h"
 #include "xml_element_mapping.h"
 
@@ -178,9 +180,8 @@ string default_language = "und";
 static mm_io_c *out = NULL;
 
 static bitvalue_c seguid_prev(128), seguid_current(128), seguid_next(128);
-bitvalue_c *seguid_link_previous = NULL;
-bitvalue_c *seguid_link_next = NULL;
-bitvalue_c *seguid_forced = NULL;
+counted_ptr<bitvalue_c> seguid_link_previous, seguid_link_next;
+vector<counted_ptr<bitvalue_c> > forced_seguids;
 
 /** \brief Add a segment family UID to the list if it doesn't exist already.
 
@@ -317,6 +318,8 @@ get_file_type(filelist_t &file) {
     type = FILE_TYPE_TTA;
   else if (wavpack_reader_c::probe_file(io, size))
     type = FILE_TYPE_WAVPACK4;
+  else if (mpeg_ts_reader_c::probe_file(io, size))
+    type = FILE_TYPE_MPEG_TS;
   else if (mpeg_ps_reader_c::probe_file(io, size))
     type = FILE_TYPE_MPEG_PS;
   else {
@@ -338,6 +341,8 @@ get_file_type(filelist_t &file) {
     type = FILE_TYPE_VOBBTN;
   else if (mpeg_es_reader_c::probe_file(io, size))
     type = FILE_TYPE_MPEG_ES;
+  else if (avc_es_reader_c::probe_file(io, size))
+    type = FILE_TYPE_AVC_ES;
   else {
     try {
       text_io = new mm_text_io_c(new mm_file_io_c(file.name));
@@ -356,6 +361,8 @@ get_file_type(filelist_t &file) {
       type = FILE_TYPE_SSA;
     else if (vobsub_reader_c::probe_file(text_io, size))
       type = FILE_TYPE_VOBSUB;
+    else if (corepicture_reader_c::probe_file(text_io, size))
+      type = FILE_TYPE_COREPICTURE;
     else if (usf_reader_c::probe_file(text_io, size))
       type = FILE_TYPE_USF;
     else
@@ -430,12 +437,12 @@ add_attachment(attachment_t attachment) {
   // check if we already have another attachment stored. This can happen
   // if we're concatenating files.
   if (0 != attachment.id) {
-    foreach(i, attachments)
+    mxforeach(i, attachments)
       if (((i->id == attachment.id) && !hack_engaged(ENGAGE_NO_VARIABLE_DATA))
           ||
           ((i->name == attachment.name) &&
            (i->description == attachment.description) &&
-           (i->data->size == attachment.data->size)))
+           (i->data->m_size == attachment.data->m_size)))
         return attachment.id;
     add_unique_uint32(attachment.id, UNIQUE_ATTACHMENT_IDS);
   } else
@@ -458,7 +465,7 @@ add_packetizer_globally(generic_packetizer_c *packetizer) {
   pack.status = FILE_STATUS_MOREDATA;
   pack.old_status = pack.status;
   pack.file = -1;
-  foreach(file, files)
+  mxforeach(file, files)
     if (file->reader == packetizer->reader) {
       pack.file = distance(files.begin(), file);
       pack.orig_file = pack.file;
@@ -479,7 +486,7 @@ set_timecode_scale() {
   audio_present = false;
   highest_sample_rate = 0;
 
-  foreach(ptzr, packetizers)
+  mxforeach(ptzr, packetizers)
     if ((*ptzr).packetizer->get_track_type() == track_video)
       video_present = true;
     else if ((*ptzr).packetizer->get_track_type() == track_audio) {
@@ -531,6 +538,11 @@ render_headers(mm_io_c *rout) {
       GetChild<EDocTypeReadVersion>(head);
     *(static_cast<EbmlUInteger *>(&doc_type_read_ver)) = 1;
 
+    if (hack_engaged(ENGAGE_USE_SIMPLE_BLOCK)) {
+      *(static_cast<EbmlUInteger *>(&doc_type_ver)) = 2;
+      *(static_cast<EbmlUInteger *>(&doc_type_read_ver)) = 2;
+    }
+
     head.Render(*rout);
 
     kax_infos = &GetChild<KaxInfo>(*kax_segment);
@@ -566,19 +578,25 @@ render_headers(mm_io_c *rout) {
     // Generate the segment UIDs.
     if (!hack_engaged(ENGAGE_NO_VARIABLE_DATA)) {
       if (file_num == 1) {
-        if (NULL == seguid_forced)
+        if (forced_seguids.empty())
           seguid_current.generate_random();
-        else
-          seguid_current = *seguid_forced;
+        else {
+          seguid_current = *forced_seguids.back();
+          forced_seguids.pop_back();
+        }
         seguid_next.generate_random();
+
       } else {
         seguid_prev = seguid_current;
-        if (NULL == seguid_forced)
+        if (forced_seguids.empty())
           seguid_current = seguid_next;
-        else
-          seguid_current = *seguid_forced;
+        else {
+          seguid_current = *forced_seguids.back();
+          forced_seguids.pop_back();
+        }
         seguid_next.generate_random();
       }
+
     } else {
       memset(seguid_current.data(), 0, 128 / 8);
       memset(seguid_prev.data(), 0, 128 / 8);
@@ -608,7 +626,7 @@ render_headers(mm_io_c *rout) {
       }
     }
 
-    if (first_file && (NULL != seguid_link_previous)) {
+    if (first_file && (NULL != seguid_link_previous.get())) {
       KaxPrevUID &kax_prevuid = GetChild<KaxPrevUID>(*kax_infos);
       kax_prevuid.CopyBuffer(seguid_link_previous->data(), 128 / 8);
     }
@@ -616,7 +634,7 @@ render_headers(mm_io_c *rout) {
     // redundant! It is set here as well in order to reserve enough space
     // for the KaxInfo structure in the file. If it is removed later then
     // an EbmlVoid element will be used for the freed space.
-    if (NULL != seguid_link_next) {
+    if (NULL != seguid_link_next.get()) {
       KaxNextUID &kax_nextuid = GetChild<KaxNextUID>(*kax_infos);
       kax_nextuid.CopyBuffer(seguid_link_next->data(), 128 / 8);
     }
@@ -711,13 +729,13 @@ render_attachments(IOCallback *rout) {
   KaxAttached *kax_a;
   KaxFileData *fdata;
   vector<attachment_t>::iterator attch;
-  int name;
+  string name;
 
   if (kax_as != NULL)
     delete kax_as;
   kax_as = new KaxAttachments();
   kax_a = NULL;
-  foreach(attch, attachments) {
+  mxforeach(attch, attachments) {
     if ((file_num == 1) || attch->to_all_files) {
       if (kax_a == NULL)
         kax_a = &GetChild<KaxAttached>(*kax_as);
@@ -733,21 +751,26 @@ render_attachments(IOCallback *rout) {
         *static_cast<EbmlString *>(&GetChild<KaxMimeType>(*kax_a)) =
           attch->mime_type;
 
-      name = attch->name.length() - 1;
-      while ((name > 0) && (attch->name[name] != PATHSEP))
-        name--;
-      if (attch->name[name] == PATHSEP)
-        name++;
+      if (attch->stored_name == "") {
+        int name_idx;
+
+        name_idx = attch->name.length() - 1;
+        while ((name_idx > 0) && (attch->name[name_idx] != PATHSEP))
+          --name_idx;
+        if (attch->name[name_idx] == PATHSEP)
+          ++name_idx;
+        name = attch->name.substr(name_idx);
+      } else
+        name = attch->stored_name;
 
       *static_cast<EbmlUnicodeString *>
-        (&GetChild<KaxFileName>(*kax_a)) =
-        cstr_to_UTFstring(attch->name.substr(name));
+        (&GetChild<KaxFileName>(*kax_a)) = cstr_to_UTFstring(name);
 
       *static_cast<EbmlUInteger *>
         (&GetChild<KaxFileUID>(*kax_a)) = attch->id;
 
       fdata = &GetChild<KaxFileData>(*kax_a);
-      fdata->CopyBuffer(attch->data->buffer, attch->data->size);
+      fdata->CopyBuffer(attch->data->m_buffer, attch->data->m_size);
     }
   }
 
@@ -773,25 +796,25 @@ check_append_mapping() {
   vector<filelist_t>::iterator src_file, dst_file;
   int count, file_id;
 
-  foreach(amap, append_mapping) {
+  mxforeach(amap, append_mapping) {
     // Check each mapping entry for validity.
 
     // 1. Is there a file with the src_file_id?
     if ((amap->src_file_id < 0) || (amap->src_file_id >= files.size()))
-      mxerror("There is no file with the ID '%lld'. The argument for "
+      mxerror("There is no file with the ID '" LLD "'. The argument for "
               "'--append-to' was invalid.\n", amap->src_file_id);
 
     // 2. Is the "source" file in "append mode", meaning does its file name
     // start with a '+'?
     src_file = files.begin() + amap->src_file_id;
     if (!src_file->appending)
-      mxerror("The file no. %lld ('%s') is not being appended. The argument "
-              "for '--append-to' was invalid.\n", amap->src_file_id,
+      mxerror("The file no. " LLD " ('%s') is not being appended. The argument"
+              " for '--append-to' was invalid.\n", amap->src_file_id,
               src_file->name.c_str());
 
     // 3. Is there a file with the dst_file_id?
     if ((amap->dst_file_id < 0) || (amap->dst_file_id >= files.size()))
-      mxerror("There is no file with the ID '%lld'. The argument for "
+      mxerror("There is no file with the ID '" LLD "'. The argument for "
               "'--append-to' was invalid.\n", amap->dst_file_id);
 
     // 4. Files cannot be appended to itself.
@@ -803,13 +826,13 @@ check_append_mapping() {
   // Now let's check each appended file if there are NO append to mappings
   // available (in which case we fill in default ones) or if there are fewer
   // mappings than tracks that are to be copied (which is an error).
-  foreach(src_file, files) {
+  mxforeach(src_file, files) {
     file_id = distance(files.begin(), src_file);
     if (!src_file->appending)
       continue;
 
     count = 0;
-    foreach(amap, append_mapping)
+    mxforeach(amap, append_mapping)
       if (amap->src_file_id == file_id)
         count++;
 
@@ -823,7 +846,7 @@ check_append_mapping() {
       string missing_mappings;
 
       // Default mapping.
-      foreach(id, src_file->reader->used_track_ids) {
+      mxforeach(id, src_file->reader->used_track_ids) {
         append_spec_t new_amap;
 
         new_amap.src_file_id = file_id;
@@ -834,7 +857,7 @@ check_append_mapping() {
         if (missing_mappings.size() > 0)
           missing_mappings += ",";
         missing_mappings +=
-          mxsprintf("%lld:%lld:%lld:%lld",
+          mxsprintf("" LLD ":" LLD ":" LLD ":" LLD,
                     new_amap.src_file_id, new_amap.src_track_id,
                     new_amap.dst_file_id, new_amap.dst_track_id);
       }
@@ -847,47 +870,47 @@ check_append_mapping() {
   }
 
   // Some more checks.
-  foreach(amap, append_mapping) {
+  mxforeach(amap, append_mapping) {
     src_file = files.begin() + amap->src_file_id;
     dst_file = files.begin() + amap->dst_file_id;
 
     // 5. Does the "source" file have a track with the src_track_id, and is
     // that track selected for copying?
     if (!mxfind2(id, amap->src_track_id, src_file->reader->used_track_ids))
-      mxerror("The file no. %lld ('%s') does not contain a track with the "
-              "ID %lld, or that track is not to be copied. The argument "
+      mxerror("The file no. " LLD " ('%s') does not contain a track with the "
+              "ID " LLD ", or that track is not to be copied. The argument "
               "for '--append-to' was invalid.\n", amap->src_file_id,
               src_file->name.c_str(), amap->src_track_id);
 
     // 6. Does the "destination" file have a track with the dst_track_id, and
     // that track selected for copying?
     if (!mxfind2(id, amap->dst_track_id, dst_file->reader->used_track_ids))
-      mxerror("The file no. %lld ('%s') does not contain a track with the "
-              "ID %lld, or that track is not to be copied. Therefore no track "
-              "can be appended to it. The argument for '--append-to' was "
-              "invalid.\n", amap->dst_file_id, dst_file->name.c_str(),
+      mxerror("The file no. " LLD " ('%s') does not contain a track with the "
+              "ID " LLD ", or that track is not to be copied. Therefore no "
+              "track can be appended to it. The argument for '--append-to' was"
+              " invalid.\n", amap->dst_file_id, dst_file->name.c_str(),
               amap->dst_track_id);
 
     // 7. Is this track already mapped to somewhere else?
-    foreach(cmp_amap, append_mapping) {
+    mxforeach(cmp_amap, append_mapping) {
       if (cmp_amap == amap)
         continue;
       if (((*cmp_amap).src_file_id == amap->src_file_id) &&
           ((*cmp_amap).src_track_id == amap->src_track_id))
-        mxerror("The track %lld from file no. %lld ('%s') is to be appended "
-                "more than once. The argument for '--append-to' was "
+        mxerror("The track " LLD " from file no. " LLD " ('%s') is to be "
+                "appended more than once. The argument for '--append-to' was "
                 "invalid.\n", amap->src_track_id, amap->src_file_id,
                 src_file->name.c_str());
     }
 
     // 8. Is there another track that is being appended to the dst_track_id?
-    foreach(cmp_amap, append_mapping) {
+    mxforeach(cmp_amap, append_mapping) {
       if (cmp_amap == amap)
         continue;
       if (((*cmp_amap).dst_file_id == amap->dst_file_id) &&
           ((*cmp_amap).dst_track_id == amap->dst_track_id))
-        mxerror("More than one track is to be appended to the track %lld "
-                "from file no. %lld ('%s'). The argument for '--append-to' "
+        mxerror("More than one track is to be appended to the track " LLD " "
+                "from file no. " LLD " ('%s'). The argument for '--append-to' "
                 "was invalid.\n", amap->dst_track_id, amap->dst_file_id,
                 dst_file->name.c_str());
     }
@@ -895,7 +918,7 @@ check_append_mapping() {
 
   // Finally see if the packetizers can be connected and connect them if they
   // can.
-  foreach(amap, append_mapping) {
+  mxforeach(amap, append_mapping) {
     vector<generic_packetizer_c *>::const_iterator gptzr;
     generic_packetizer_c *src_ptzr, *dst_ptzr;
     string error_message;
@@ -903,13 +926,13 @@ check_append_mapping() {
 
     src_file = files.begin() + amap->src_file_id;
     src_ptzr = NULL;
-    foreach(gptzr, src_file->reader->reader_packetizers)
+    mxforeach(gptzr, src_file->reader->reader_packetizers)
       if ((*gptzr)->ti.id == amap->src_track_id)
         src_ptzr = (*gptzr);
 
     dst_file = files.begin() + amap->dst_file_id;
     dst_ptzr = NULL;
-    foreach(gptzr, dst_file->reader->reader_packetizers)
+    mxforeach(gptzr, dst_file->reader->reader_packetizers)
       if ((*gptzr)->ti.id == amap->dst_track_id)
         dst_ptzr = (*gptzr);
 
@@ -918,33 +941,42 @@ check_append_mapping() {
 
     // And now try to connect the packetizers.
     result = src_ptzr->can_connect_to(dst_ptzr, error_message);
-    if (CAN_CONNECT_YES != result) {
+    if (CAN_CONNECT_MAYBE_CODECPRIVATE == result)
+      mxwarn("The track number " LLD " from the file '%s' can probably "
+             "not be appended correctly to the track number " LLD
+             " from the file '%s': %s Please make sure that "
+             "the resulting file plays correctly the whole time. "
+             "The author of this program will probably not give support "
+             "for playback issues with the resulting file.\n",
+             amap->src_track_id, files[amap->src_file_id].name.c_str(),
+             amap->dst_track_id, files[amap->dst_file_id].name.c_str(),
+             error_message.c_str());
+
+    else if (CAN_CONNECT_YES != result) {
       string reason(result == CAN_CONNECT_NO_FORMAT ?
                     "the formats do not match" :
                     result == CAN_CONNECT_NO_PARAMETERS ?
                     "the track parameters do not match" :
                     "of unknown reasons");
-
-      if (CAN_CONNECT_NO_PARAMETERS == result)
-        reason += mxsprintf(" (%s)", error_message.c_str());
-
-      mxerror("The track number %lld from the file '%s' cannot be appended "
-              "to the track number %lld from the file '%s' because %s.\n",
+      mxerror("The track number " LLD " from the file '%s' cannot be "
+              "appended to the track number " LLD " from the file '%s' "
+              "because %s.\n",
               amap->src_track_id, files[amap->src_file_id].name.c_str(),
               amap->dst_track_id, files[amap->dst_file_id].name.c_str(),
               reason.c_str());
     }
+
     src_ptzr->connect(dst_ptzr);
     dst_file->appended_to = true;
   }
 
   // Calculate the "longest path" -- meaning the maximum number of
   // concatenated files. This is needed for displaying the progress.
-  foreach(amap, append_mapping) {
+  mxforeach(amap, append_mapping) {
     int path_length;
 
     // Is this the first in a chain?
-    foreach(cmp_amap, append_mapping) {
+    mxforeach(cmp_amap, append_mapping) {
       if (amap == cmp_amap)
         continue;
       if ((amap->dst_file_id == cmp_amap->src_file_id) &&
@@ -958,7 +990,7 @@ check_append_mapping() {
     trav_amap = amap;
     path_length = 2;
     do {
-      foreach(cmp_amap, append_mapping)
+      mxforeach(cmp_amap, append_mapping)
         if ((trav_amap->src_file_id == cmp_amap->dst_file_id) &&
             (trav_amap->src_track_id == cmp_amap->dst_track_id)) {
           trav_amap = cmp_amap;
@@ -989,7 +1021,7 @@ calc_max_chapter_size() {
   KaxChapters *chapters;
 
   // Step 1: Add all chapters from files that are not being appended.
-  foreach(file, files) {
+  mxforeach(file, files) {
     if (file->appending)
       continue;
     chapters = file->reader->chapters;
@@ -1012,7 +1044,7 @@ calc_max_chapter_size() {
     max_chapter_size += kax_chapters->ElementSize();
   }
 
-  foreach(file, files) {
+  mxforeach(file, files) {
     chapters = file->reader->chapters;
     if (chapters == NULL)
       continue;
@@ -1034,7 +1066,7 @@ void
 create_readers() {
   vector<filelist_t>::iterator file;
 
-  foreach(file, files) {
+  mxforeach(file, files) {
     try {
       switch (file->type) {
         case FILE_TYPE_AAC:
@@ -1043,6 +1075,9 @@ create_readers() {
         case FILE_TYPE_AC3:
           file->reader = new ac3_reader_c(*file->ti);
           break;
+        case FILE_TYPE_AVC_ES:
+          file->reader = new avc_es_reader_c(*file->ti);
+          break;
         case FILE_TYPE_AVI:
           file->reader = new avi_reader_c(*file->ti);
           break;
@@ -1087,6 +1122,9 @@ create_readers() {
         case FILE_TYPE_USF:
           file->reader = new usf_reader_c(*file->ti);
           break;
+        case FILE_TYPE_COREPICTURE:
+          file->reader = new corepicture_reader_c(*file->ti);
+          break;
         case FILE_TYPE_WAV:
           file->reader = new wav_reader_c(*file->ti);
           break;
@@ -1114,13 +1152,13 @@ create_readers() {
     vector<attachment_t>::const_iterator att;
 
     // Create the packetizers.
-    foreach(file, files) {
+    mxforeach(file, files) {
       file->reader->appending = file->appending;
       file->reader->create_packetizers();
     }
     // Check if all track IDs given on the command line are actually
     // present.
-    foreach(file, files) {
+    mxforeach(file, files) {
       file->reader->check_track_ids_and_packetizers();
       file->num_unfinished_packetizers =
         file->reader->reader_packetizers.size();
@@ -1131,10 +1169,10 @@ create_readers() {
     check_append_mapping();
 
     // Calculate the size of all attachments for split control.
-    foreach(att, attachments) {
-      attachment_sizes_first += att->data->size;
+    mxforeach(att, attachments) {
+      attachment_sizes_first += att->data->m_size;
       if (att->to_all_files)
-        attachment_sizes_others += att->data->size;
+        attachment_sizes_others += att->data->m_size;
     }
 
     calc_max_chapter_size();
@@ -1347,7 +1385,7 @@ finish_file(bool last_file) {
   // Now re-render the kax_duration and fill in the biggest timecode
   // as the file's duration.
   out->save_pos(kax_duration->GetElementPosition());
-  mxverb(3, "mkvmerge: kax_duration: gdur %lld tcs %f du %lld\n",
+  mxverb(3, "mkvmerge: kax_duration: gdur " LLD " tcs %f du " LLD "\n",
          cluster_helper->get_duration(), timecode_scale,
          irnd((double)cluster_helper->get_duration() /
               (double)((int64_t)timecode_scale)));
@@ -1363,7 +1401,7 @@ finish_file(bool last_file) {
   changed = 0;
   kax_infos->UpdateSize(true);
   info_size = kax_infos->ElementSize();
-  if (last_file && (NULL != seguid_link_next)) {
+  if (last_file && (NULL != seguid_link_next.get())) {
     KaxNextUID &kax_nextuid = GetChild<KaxNextUID>(*kax_infos);
     kax_nextuid.CopyBuffer(seguid_link_next->data(), 128 / 8);
     changed = 1;
@@ -1484,8 +1522,8 @@ finish_file(bool last_file) {
     kax_sh_main->UpdateSize();
     if (kax_sh_void->ReplaceWith(*kax_sh_main, *out, true) == 0)
       mxwarn(_("This should REALLY not have happened. The space reserved for "
-               "the first meta seek element was too small. Size needed: %lld. "
-               "%s\n"), kax_sh_main->ElementSize(), BUGMSG);
+               "the first meta seek element was too small. Size needed: " LLD
+               ". %s\n"), kax_sh_main->ElementSize(), BUGMSG);
   }
 
   // Set the correct size for the segment.
@@ -1535,11 +1573,11 @@ append_track(packetizer_t &ptzr,
 
   if (NULL != deferred_file)
     src_file.deferred_max_timecode_seen =
-      deferred_file->reader->max_timecode_seen;      
+      deferred_file->reader->max_timecode_seen;
 
   // Find the generic_packetizer_c that we will be appending to the one
   // stored in ptzr.
-  foreach(gptzr, src_file.reader->reader_packetizers)
+  mxforeach(gptzr, src_file.reader->reader_packetizers)
     if (amap.src_track_id == (*gptzr)->ti.id)
       break;
   if (gptzr == src_file.reader->reader_packetizers.end())
@@ -1565,13 +1603,13 @@ append_track(packetizer_t &ptzr,
       (-1 == src_file.deferred_max_timecode_seen)) {
     vector<filelist_t>::iterator file;
 
-    foreach(file, files) {
+    mxforeach(file, files) {
       vector<generic_packetizer_c *>::const_iterator vptzr;
 
       if (file->done)
         continue;
 
-      foreach(vptzr, file->reader->reader_packetizers) {
+      mxforeach(vptzr, file->reader->reader_packetizers) {
         if ((*vptzr)->get_track_type() == track_video)
           break;
       }
@@ -1588,8 +1626,8 @@ append_track(packetizer_t &ptzr,
     }
   }
 
-  mxinfo("Appending track %lld from file no. %lld ('%s') to track %lld from "
-         "file no. %lld ('%s').\n",
+  mxinfo("Appending track " LLD " from file no. " LLD " ('%s') to track " LLD
+         " from file no. " LLD " ('%s').\n",
          (*gptzr)->ti.id, amap.src_file_id, (*gptzr)->ti.fname.c_str(),
          ptzr.packetizer->ti.id, amap.dst_file_id,
          ptzr.packetizer->ti.fname.c_str());
@@ -1636,14 +1674,14 @@ append_track(packetizer_t &ptzr,
     if (src_file.reader->ptzr_first_packet == NULL)
       ptzr.status = ptzr.packetizer->read();
     if (src_file.reader->ptzr_first_packet != NULL) {
-      foreach(cmp_amap, append_mapping)
+      mxforeach(cmp_amap, append_mapping)
         if ((cmp_amap->src_file_id == amap.src_file_id) &&
             (cmp_amap->src_track_id ==
              src_file.reader->ptzr_first_packet->ti.id) &&
             (cmp_amap->dst_file_id == amap.dst_file_id))
           break;
       if (cmp_amap != append_mapping.end()) {
-        foreach(gptzr, dst_file.reader->reader_packetizers)
+        mxforeach(gptzr, dst_file.reader->reader_packetizers)
           if ((*gptzr)->ti.id == cmp_amap->dst_track_id) {
             timecode_adjustment = (*gptzr)->max_timecode_seen;
             break;
@@ -1654,12 +1692,12 @@ append_track(packetizer_t &ptzr,
 
   if (ptzr.packetizer->get_track_type() == track_subtitle) {
     mxverb(2, "append_track: new timecode_adjustment for subtitle track: "
-           "%lld for %lld\n", timecode_adjustment, ptzr.packetizer->ti.id);
+           LLD " for " LLD "\n", timecode_adjustment, ptzr.packetizer->ti.id);
     // The actual connection.
     ptzr.packetizer->connect(old_packetizer, timecode_adjustment);
   } else {
     mxverb(2, "append_track: new timecode_adjustment for NON subtitle track: "
-           "%lld for %lld\n", timecode_adjustment, ptzr.packetizer->ti.id);
+           LLD " for " LLD "\n", timecode_adjustment, ptzr.packetizer->ti.id);
     // The actual connection.
     ptzr.packetizer->connect(old_packetizer);
   }
@@ -1694,14 +1732,14 @@ append_tracks_maybe() {
   bool appended_a_track;
 
   appended_a_track = false;
-  foreach(ptzr, packetizers) {
+  mxforeach(ptzr, packetizers) {
     if (ptzr->deferred)
       continue;
     if (!files[ptzr->orig_file].appended_to)
       continue;
     if (FILE_STATUS_DONE_AND_DRY != ptzr->status)
       continue;
-    foreach(amap, append_mapping)
+    mxforeach(amap, append_mapping)
       if ((amap->dst_file_id == ptzr->file) &&
           (amap->dst_track_id == ptzr->packetizer->ti.id))
         break;
@@ -1737,7 +1775,7 @@ establish_deferred_connections(filelist_t &file) {
   def_cons = file.deferred_connections;
   file.deferred_connections.clear();
 
-  foreach(def_con, def_cons)
+  mxforeach(def_con, def_cons)
     append_track(*def_con->ptzr, def_con->amap, &file);
 
   // \todo Select a new file that the subs will defer to.
@@ -1759,7 +1797,7 @@ main_loop() {
 
     // Step 1: Make sure a packet is available for each output
     // as long we haven't already processed the last one.
-    foreach(ptzr, packetizers) {
+    mxforeach(ptzr, packetizers) {
       if (ptzr->status == FILE_STATUS_HOLDING)
         ptzr->status = FILE_STATUS_MOREDATA;
       ptzr->old_status = ptzr->status;
@@ -1786,8 +1824,10 @@ main_loop() {
         // If all packetizers for a file have finished then establish the
         // deferred connections.
         if ((file.num_unfinished_packetizers <= 0) &&
-            (file.old_num_unfinished_packetizers > 0))
+            (file.old_num_unfinished_packetizers > 0)) {
           establish_deferred_connections(file);
+          file.done = true;
+        }
         file.old_num_unfinished_packetizers = file.num_unfinished_packetizers;
       }
     }
@@ -1795,7 +1835,7 @@ main_loop() {
     // Step 2: Pick the packet with the lowest timecode and
     // stuff it into the Matroska file.
     winner = packetizers.end();
-    foreach(ptzr, packetizers) {
+    mxforeach(ptzr, packetizers) {
       if (ptzr->pack.get() != NULL) {
         if ((winner == packetizers.end()) || (winner->pack.get() == NULL))
           winner = ptzr;
@@ -1868,7 +1908,7 @@ static void
 destroy_readers() {
   vector<filelist_t>::const_iterator file;
 
-  foreach(file, files)
+  mxforeach(file, files)
     if ((*file).reader != NULL)
       delete (*file).reader;
 
@@ -1887,7 +1927,7 @@ cleanup() {
 
   destroy_readers();
 
-  foreach(file, files)
+  mxforeach(file, files)
     delete (*file).ti;
   files.clear();
 
@@ -1904,12 +1944,7 @@ cleanup() {
   delete kax_info_chap;
   kax_info_chap = NULL;
 
-  delete seguid_link_previous;
-  seguid_link_previous = NULL;
-  delete seguid_link_next;
-  seguid_link_next = NULL;
-  delete seguid_forced;
-  seguid_forced = NULL;
+  forced_seguids.clear();
 
   utf8_done();
 }
