@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: output_control.h 3022 2005-08-15 20:47:56Z mosu $
+   $Id: output_control.h 3579 2007-08-19 08:53:32Z mosu $
 
    output handling
 
@@ -95,7 +95,7 @@ struct filelist_t {
 };
 
 struct attachment_t {
-  string name, mime_type, description;
+  string name, stored_name, mime_type, description;
   int64_t id;
   bool to_all_files;
   counted_ptr<buffer_t> data;
@@ -105,6 +105,7 @@ struct attachment_t {
   }
   void clear() {
     name = "";
+    stored_name = "";
     mime_type = "";
     description = "";
     id = 0;
@@ -126,7 +127,7 @@ enum timecode_scale_mode_e {
 
 class family_uids_c: public vector<bitvalue_c> {
 public:
-	bool add_family_uid(const KaxSegmentFamily &family);
+  bool add_family_uid(const KaxSegmentFamily &family);
 };
 
 extern vector<packetizer_t> packetizers;
@@ -140,7 +141,10 @@ extern string outfile;
 extern double timecode_scale;
 extern timecode_scale_mode_e timecode_scale_mode;
 
-extern bitvalue_c *seguid_link_previous, *seguid_link_next, *seguid_forced;
+typedef counted_ptr<bitvalue_c> bitvalue_cptr;
+
+extern bitvalue_cptr seguid_link_previous, seguid_link_next;
+extern vector<bitvalue_cptr> forced_seguids;
 extern family_uids_c segfamily_uids;
 
 extern KaxInfo *kax_info_chap;
