@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: pr_generic.cpp 3014 2005-08-06 11:52:14Z mosu $
+   $Id: pr_generic.cpp 3533 2007-06-23 14:56:28Z mosu $
 
    functions common for all readers/packetizers
 
@@ -28,6 +28,7 @@
 #include "common.h"
 #include "commonebml.h"
 #include "compression.h"
+#include "hacks.h"
 #include "mkvmerge.h"
 #include "output_control.h"
 #include "pr_generic.h"
@@ -39,11 +40,6 @@ using namespace std;
 int64_t packet_t::packet_number_counter = 0;
 
 packet_t::~packet_t() {
-  vector<unsigned char *>::iterator i;
-
-  safefree(data);
-  foreach(i, data_adds)
-    safefree(*i);
 }
 
 // ---------------------------------------------------------------------
@@ -54,7 +50,6 @@ generic_packetizer_c::generic_packetizer_c(generic_reader_c *nreader,
                                            track_info_c &_ti)
   throw(error_c): ti(_ti) {
   int i;
-  bool found;
 
 #ifdef DEBUG
   debug_facility.add_packetizer(this);
@@ -78,176 +73,151 @@ generic_packetizer_c::generic_packetizer_c(generic_reader_c *nreader,
   timecode_factory_application_mode = TFA_AUTOMATIC;
 
   // Let's see if the user specified audio sync for this track.
-  found = false;
-  for (i = 0; i < ti.audio_syncs.size(); i++) {
-    audio_sync_t &as = ti.audio_syncs[i];
-    if ((as.id == ti.id) || (as.id == -1)) { // -1 == all tracks
-      found = true;
-      ti.async = as;
-      break;
-    }
-  }
-  if (!found && (ti.async.linear == 0.0)) {
+  if (map_has_key(ti.audio_syncs, ti.id))
+    ti.async = ti.audio_syncs[ti.id];
+  else if (map_has_key(ti.audio_syncs, -1))
+    ti.async = ti.audio_syncs[-1];
+  if (0.0 == ti.async.linear)
     ti.async.linear = 1.0;
-    ti.async.displacement = 0;
-  } else
-    ti.async.displacement *= (int64_t)1000000; // ms to ns
+  ti.async.displacement *= (int64_t)1000000; // ms to ns
   initial_displacement = ti.async.displacement;
   ti.async.displacement = 0;
 
   // Let's see if the user has specified a delay for this track.
-  ti.packet_delay = 0;
-  for (i = 0; i < ti.packet_delays.size(); i++) {
-    packet_delay_t &pd = ti.packet_delays[i];
-    if ((pd.id == ti.id) || (pd.id == -1)) { // -1 == all tracks
-      ti.packet_delay = pd.delay;
-      break;
-    }
-  }
+  if (map_has_key(ti.packet_delays, ti.id))
+    ti.packet_delay = ti.packet_delays[ti.id];
+  else if (map_has_key(ti.packet_delays, -1))
+    ti.packet_delay = ti.packet_delays[-1];
 
   // Let's see if the user has specified which cues he wants for this track.
-  ti.cues = CUE_STRATEGY_UNSPECIFIED;
-  for (i = 0; i < ti.cue_creations.size(); i++) {
-    cue_creation_t &cc = ti.cue_creations[i];
-    if ((cc.id == ti.id) || (cc.id == -1)) { // -1 == all tracks
-      ti.cues = cc.cues;
-      break;
-    }
-  }
+  if (map_has_key(ti.cue_creations, ti.id))
+    ti.cues = ti.cue_creations[ti.id];
+  else if (map_has_key(ti.cue_creations, -1))
+    ti.cues = ti.cue_creations[-1];
 
   // Let's see if the user has given a default track flag for this track.
-  for (i = 0; i < ti.default_track_flags.size(); i++) {
-    int64_t id = ti.default_track_flags[i];
-    if ((id == ti.id) || (id == -1)) { // -1 == all tracks
-      ti.default_track = true;
-      break;
-    }
+  if (map_has_key(ti.default_track_flags, ti.id)) {
+    ti.default_track = ti.default_track_flags[ti.id];
+    ti.default_track_flag_present = true;
+  } else if (map_has_key(ti.default_track_flags, -1)) {
+    ti.default_track = ti.default_track_flags[-1];
+    ti.default_track_flag_present = true;
   }
 
   // Let's see if the user has specified a language for this track.
-  for (i = 0; i < ti.languages.size(); i++) {
-    language_t &lang = ti.languages[i];
-    if ((lang.id == ti.id) || (lang.id == -1)) { // -1 == all tracks
-      ti.language = lang.language;
-      break;
-    }
-  }
+  if (map_has_key(ti.languages, ti.id))
+    ti.language = ti.languages[ti.id];
+  else if (map_has_key(ti.languages, -1))
+    ti.language = ti.languages[-1];
 
   // Let's see if the user has specified a sub charset for this track.
-  for (i = 0; i < ti.sub_charsets.size(); i++) {
-    subtitle_charset_t &sc = ti.sub_charsets[i];
-    if ((sc.id == ti.id) || (sc.id == -1)) { // -1 == all tracks
-      ti.sub_charset = sc.charset;
-      break;
-    }
-  }
+  if (map_has_key(ti.sub_charsets, ti.id))
+    ti.sub_charset = ti.sub_charsets[ti.id];
+  else if (map_has_key(ti.sub_charsets, -1))
+    ti.sub_charset = ti.sub_charsets[-1];
 
   // Let's see if the user has specified a sub charset for this track.
-  for (i = 0; i < ti.all_tags.size(); i++) {
-    tags_t &tags = ti.all_tags[i];
-    if ((tags.id == ti.id) || (tags.id == -1)) { // -1 == all tracks
-      ti.tags_ptr = i;
-      if (ti.tags != NULL)
-        delete ti.tags;
-      ti.tags = new KaxTags;
-      parse_xml_tags(ti.all_tags[ti.tags_ptr].file_name, ti.tags);
-      break;
-    }
+  if (map_has_key(ti.all_tags, ti.id))
+    ti.tags_file_name = ti.all_tags[ti.id];
+  else if (map_has_key(ti.all_tags, -1))
+    ti.tags_file_name = ti.all_tags[-1];
+  if (ti.tags_file_name != "") {
+    ti.tags = new KaxTags;
+    parse_xml_tags(ti.tags_file_name, ti.tags);
   }
 
   // Let's see if the user has specified how this track should be compressed.
-  ti.compression = COMPRESSION_UNSPECIFIED;
-  for (i = 0; i < ti.compression_list.size(); i++) {
-    compression_method_t &cm = ti.compression_list[i];
-    if ((cm.id == ti.id) || (cm.id == -1)) { // -1 == all tracks
-      ti.compression = cm.method;
-      break;
-    }
-  }
+  if (map_has_key(ti.compression_list, ti.id))
+    ti.compression = ti.compression_list[ti.id];
+  else if (map_has_key(ti.compression_list, -1))
+    ti.compression = ti.compression_list[-1];
 
   // Let's see if the user has specified a name for this track.
-  for (i = 0; i < ti.track_names.size(); i++) {
-    track_name_t &tn = ti.track_names[i];
-    if ((tn.id == ti.id) || (tn.id == -1)) { // -1 == all tracks
-      ti.track_name = tn.name;
-      break;
-    }
-  }
+  if (map_has_key(ti.track_names, ti.id))
+    ti.track_name = ti.track_names[ti.id];
+  else if (map_has_key(ti.track_names, -1))
+    ti.track_name = ti.track_names[-1];
 
   // Let's see if the user has specified external timecodes for this track.
-  for (i = 0; i < ti.all_ext_timecodes.size(); i++) {
-    ext_timecodes_t &et = ti.all_ext_timecodes[i];
-    if ((et.id == ti.id) || (et.id == -1)) { // -1 == all tracks
-      ti.ext_timecodes = et.ext_timecodes;
-      break;
-    }
-  }
+  if (map_has_key(ti.all_ext_timecodes, ti.id))
+    ti.ext_timecodes = ti.all_ext_timecodes[ti.id];
+  else if (map_has_key(ti.all_ext_timecodes, -1))
+    ti.ext_timecodes = ti.all_ext_timecodes[-1];
 
   // Let's see if the user has specified an aspect ratio or display dimensions
   // for this track.
-  for (i = 0; i < ti.display_properties.size(); i++) {
+  i = -2;
+  if (map_has_key(ti.display_properties, ti.id))
+    i = ti.id;
+  else if (map_has_key(ti.display_properties, -1))
+    i = -1;
+  if (-2 != i) {
     display_properties_t &dprop = ti.display_properties[i];
-    if ((dprop.id == ti.id) || (dprop.id == -1)) { // -1 == all tracks
-      if (dprop.aspect_ratio < 0) {
-        ti.display_width = dprop.width;
-        ti.display_height = dprop.height;
-        ti.display_dimensions_given = true;
-      } else {
-        ti.aspect_ratio = dprop.aspect_ratio;
-        ti.aspect_ratio_given = true;
-        ti.aspect_ratio_is_factor = dprop.ar_factor;
-      }
+    if (dprop.aspect_ratio < 0) {
+      ti.display_width = dprop.width;
+      ti.display_height = dprop.height;
+      ti.display_dimensions_given = true;
+    } else {
+      ti.aspect_ratio = dprop.aspect_ratio;
+      ti.aspect_ratio_given = true;
+      ti.aspect_ratio_is_factor = dprop.ar_factor;
     }
   }
   if (ti.aspect_ratio_given && ti.display_dimensions_given)
     mxerror(_("Both '%s' and '--display-dimensions' were given "
-              "for track %lld of '%s'.\n"), ti.aspect_ratio_is_factor ?
+              "for track " LLD " of '%s'.\n"), ti.aspect_ratio_is_factor ?
             _("Aspect ratio factor") : _("Aspect ratio"), ti.id,
             ti.fname.c_str());
 
-  memset(ti.fourcc, 0, 5);
   // Let's see if the user has specified a FourCC for this track.
-  for (i = 0; i < ti.all_fourccs.size(); i++) {
-    fourcc_t &fourcc = ti.all_fourccs[i];
-    if ((fourcc.id == ti.id) || (fourcc.id == -1)) { // -1 == all tracks
-      memcpy(ti.fourcc, fourcc.fourcc, 4);
-      ti.fourcc[4] = 0;
-      break;
-    }
-  }
+  if (map_has_key(ti.all_fourccs, ti.id))
+    ti.fourcc = ti.all_fourccs[ti.id];
+  else if (map_has_key(ti.all_fourccs, -1))
+    ti.fourcc = ti.all_fourccs[-1];
 
-  memset(&ti.pixel_cropping, 0, sizeof(pixel_crop_t));
-  ti.pixel_cropping.id = -2;
   // Let's see if the user has specified a FourCC for this track.
-  for (i = 0; i < ti.pixel_crop_list.size(); i++) {
-    pixel_crop_t &cropping = ti.pixel_crop_list[i];
-    if ((cropping.id == ti.id) || (cropping.id == -1)) { // -1 == all tracks
-      ti.pixel_cropping = cropping;
-      break;
-    }
-  }
+  ti.pixel_cropping_specified = true;
+  if (map_has_key(ti.pixel_crop_list, ti.id))
+    ti.pixel_cropping = ti.pixel_crop_list[ti.id];
+  else if (map_has_key(ti.pixel_crop_list, -1))
+    ti.pixel_cropping = ti.pixel_crop_list[-1];
+  else
+    ti.pixel_cropping_specified = false;
+
+  // Let's see if the user has specified a stereo mode for this track.
+  if (map_has_key(ti.stereo_mode_list, ti.id))
+    ti.stereo_mode = ti.stereo_mode_list[ti.id];
+  else if (map_has_key(ti.stereo_mode_list, -1))
+    ti.stereo_mode = ti.stereo_mode_list[-1];
 
   // Let's see if the user has specified a default duration for this track.
   htrack_default_duration = -1;
-  default_duration_forced = false;
-  for (i = 0; i < ti.default_durations.size(); i++) {
-    default_duration_t &dd = ti.default_durations[i];
-    if ((dd.id == ti.id) || (dd.id == -1)) { // -1 == all tracks
-      htrack_default_duration = dd.default_duration;
-      default_duration_forced = true;
-      break;
-    }
-  }
+  default_duration_forced = true;
+  if (map_has_key(ti.default_durations, ti.id))
+    htrack_default_duration = ti.default_durations[ti.id];
+  else if (map_has_key(ti.default_durations, -1))
+    htrack_default_duration = ti.default_durations[-1];
+  else
+    default_duration_forced = false;
 
   // Let's see if the user has set a max_block_add_id
   htrack_max_add_block_ids = -1;
-  for (i = 0; i < ti.max_blockadd_ids.size(); i++) {
-    max_blockadd_id_t &mbi = ti.max_blockadd_ids[i];
-    if ((mbi.id == ti.id) || (mbi.id == -1)) { // -1 == all tracks
-      htrack_max_add_block_ids = mbi.max_blockadd_id;
-      break;
-    }
-  }
+  if (map_has_key(ti.max_blockadd_ids, ti.id))
+    htrack_max_add_block_ids = ti.max_blockadd_ids[ti.id];
+  else if (map_has_key(ti.max_blockadd_ids, -1))
+    htrack_max_add_block_ids = ti.max_blockadd_ids[-1];
+
+  // Let's see if the user has specified "AAC is SBR" for this track.
+  if (map_has_key(ti.all_aac_is_sbr, ti.id))
+    ti.aac_is_sbr = ti.all_aac_is_sbr[ti.id] ? 1 : 0;
+  else if (map_has_key(ti.all_aac_is_sbr, -1))
+    ti.aac_is_sbr = ti.all_aac_is_sbr[-1] ? 1 : 0;
+
+  // Let's see if the user has specified a NALU size length for this track.
+  if (map_has_key(ti.nalu_size_lengths, ti.id))
+    ti.nalu_size_length = ti.nalu_size_lengths[ti.id];
+  else if (map_has_key(ti.nalu_size_lengths, -1))
+    ti.nalu_size_length = ti.nalu_size_lengths[-1];
 
   // Set default header values to 'unset'.
   if (!reader->appending)
@@ -280,7 +250,7 @@ generic_packetizer_c::generic_packetizer_c(generic_reader_c *nreader,
 generic_packetizer_c::~generic_packetizer_c() {
   safefree(hcodec_private);
   if (!packet_queue.empty())
-    mxerror("Packet queue not empty for new track ID %lld (flushed: %d). "
+    mxerror("Packet queue not empty for new track ID " LLD " (flushed: %d). "
             "Frames have been lost during remux. %s\n", ti.id,
             has_been_flushed, BUGMSG);
 }
@@ -323,8 +293,8 @@ generic_packetizer_c::set_tag_track_uid() {
     if (!tag->CheckMandatory())
       mxerror(_("The tags in '%s' could not be parsed: some mandatory "
                 "elements are missing.\n"),
-              ti.tags_ptr >= 0 ? ti.all_tags[ti.tags_ptr].file_name.c_str()
-              : ti.fname.c_str());
+              ti.tags_file_name != "" ? ti.tags_file_name.c_str():
+              ti.fname.c_str());
   }
 }
 
@@ -416,7 +386,7 @@ generic_packetizer_c::set_track_max_cache(int max_cache) {
   htrack_max_cache = max_cache;
   if (track_entry != NULL)
     *(static_cast<EbmlUInteger *>
-      (&GetChild<KaxTrackMinCache>(*track_entry))) = max_cache;
+      (&GetChild<KaxTrackMaxCache>(*track_entry))) = max_cache;
 }
 
 void
@@ -533,8 +503,8 @@ generic_packetizer_c::set_as_default_track(int type,
              (default_tracks[type] != hserialno) &&
              !default_track_warning_printed) {
     mxwarn(_("Another default track for %s tracks has already "
-             "been set. The 'default' flag for track %lld of '%s' will not be "
-             "set.\n"),
+             "been set. The 'default' flag for track " LLD " of '%s' will not "
+             "be set.\n"),
            type == 0 ? "audio" : type == 'v' ? "video" : "subtitle",
            ti.id, ti.fname.c_str());
     default_track_warning_printed = true;
@@ -554,7 +524,6 @@ generic_packetizer_c::set_video_pixel_cropping(int left,
                                                int top,
                                                int right,
                                                int bottom) {
-  ti.pixel_cropping.id = 0;
   ti.pixel_cropping.left = left;
   ti.pixel_cropping.top = top;
   ti.pixel_cropping.right = right;
@@ -578,6 +547,19 @@ generic_packetizer_c::set_video_pixel_cropping(int left,
   }
 }
 
+void
+generic_packetizer_c::set_stereo_mode(stereo_mode_e stereo_mode) {
+  ti.stereo_mode = stereo_mode;
+
+  if ((NULL != track_entry) &&
+      (STEREO_MODE_UNSPECIFIED != stereo_mode)) {
+    KaxTrackVideo &video = GetChild<KaxTrackVideo>(*track_entry);
+
+    *(static_cast<EbmlUInteger *>
+      (&GetChild<KaxVideoStereoMode>(video))) = ti.stereo_mode;
+  }
+}
+
 void
 generic_packetizer_c::set_headers() {
   int idx, disp_width, disp_height;
@@ -641,7 +623,7 @@ generic_packetizer_c::set_headers() {
       (&GetChild<KaxMaxBlockAdditionID>(*track_entry))) =
       htrack_max_add_block_ids;
 
-  if (NULL != timecode_factory)
+  if (NULL != timecode_factory.get())
     htrack_default_duration =
       (int64_t)timecode_factory->get_default_duration(htrack_default_duration);
   if (htrack_default_duration != -1.0)
@@ -656,10 +638,12 @@ generic_packetizer_c::set_headers() {
   else
     idx = DEFTRACK_TYPE_SUBS;
 
-  if (ti.default_track)
-    set_as_default_track(idx, DEFAULT_TRACK_PRIORITY_CMDLINE);
-  else
+  if (!ti.default_track_flag_present)
     set_as_default_track(idx, DEFAULT_TRACK_PRIORITY_FROM_TYPE);
+  else if (ti.default_track)
+    set_as_default_track(idx, DEFAULT_TRACK_PRIORITY_CMDLINE);
+  else if (default_tracks[idx] == hserialno)
+    default_tracks[idx] = 0;
 
   if (ti.language != "")
     *(static_cast<EbmlString *>
@@ -726,7 +710,7 @@ generic_packetizer_c::set_headers() {
       dheight.SetDefaultSize(4);
       ti.display_height = disp_height;
 
-      if (ti.pixel_cropping.id != -2) {
+      if (ti.pixel_cropping_specified) {
         *(static_cast<EbmlUInteger *>
           (&GetChild<KaxVideoPixelCropLeft>(video))) =
           ti.pixel_cropping.left;
@@ -740,6 +724,10 @@ generic_packetizer_c::set_headers() {
           (&GetChild<KaxVideoPixelCropBottom>(video))) =
           ti.pixel_cropping.bottom;
       }
+
+      if (STEREO_MODE_UNSPECIFIED != ti.stereo_mode)
+        *(static_cast<EbmlUInteger *>
+          (&GetChild<KaxVideoStereoMode>(video))) = ti.stereo_mode;
     }
 
   } else if (htrack_type == track_audio) {
@@ -781,7 +769,6 @@ generic_packetizer_c::set_headers() {
       (hcompression != COMPRESSION_NONE)) {
     KaxContentEncodings *c_encodings;
     KaxContentEncoding *c_encoding;
-    KaxContentCompression *c_comp;
 
     c_encodings = &GetChild<KaxContentEncodings>(*track_entry);
     c_encoding = &GetChild<KaxContentEncoding>(*c_encodings);
@@ -795,12 +782,8 @@ generic_packetizer_c::set_headers() {
     *static_cast<EbmlUInteger *>
       (&GetChild<KaxContentEncodingScope>(*c_encoding)) = 1;
 
-    c_comp = &GetChild<KaxContentCompression>(*c_encoding);
-    // Set compression method.
-    *static_cast<EbmlUInteger *>(&GetChild<KaxContentCompAlgo>(*c_comp)) =
-      hcompression - 1;
-
     compressor = compressor_c::create(hcompression);
+    compressor->set_track_headers(*c_encoding);
   }
 
   if (no_lacing)
@@ -839,41 +822,40 @@ generic_packetizer_c::fix_headers() {
 
 void
 generic_packetizer_c::add_packet(packet_cptr pack) {
-  int length, add_length, i;
+  int length, i;
 
   if (reader->ptzr_first_packet == NULL)
     reader->ptzr_first_packet = this;
 
   // strip elements to be removed
   if ((htrack_max_add_block_ids != -1) &&
-      (htrack_max_add_block_ids < pack->memory_adds.size()))
-    pack->memory_adds.resize(htrack_max_add_block_ids);
+      (htrack_max_add_block_ids < pack->data_adds.size()))
+    pack->data_adds.resize(htrack_max_add_block_ids);
 
-  pack->data_adds.resize(pack->memory_adds.size());
-  pack->data_adds_lengths.resize(pack->memory_adds.size());
-
-  length = pack->memory->size;
+  length = pack->data->get_size();
   if (NULL != compressor.get()) {
-    pack->data = compressor->compress(pack->memory->data, length);
-    pack->memory->release();
-    for (i = 0; i < pack->memory_adds.size(); i++) {
-      add_length = pack->memory_adds[i]->size;
-      pack->data_adds[i] = compressor->compress(pack->memory_adds[i]->data,
-                                                add_length);
-      pack->data_adds_lengths[i] = add_length;
-      pack->memory_adds[i]->release();
+    try {
+      compressor->compress(pack->data);
+      for (i = 0; i < pack->data_adds.size(); i++)
+        compressor->compress(pack->data_adds[i]);
+    } catch (compression_error_c &e) {
+      mxerror(FMT_TID "Compression failed: %s\n", ti.fname.c_str(),
+              (int64_t)ti.id, e.get_error().c_str());
     }
   } else {
-    pack->data = pack->memory->grab();
-    for (i = 0; i < pack->memory_adds.size(); i++) {
-      pack->data_adds[i] = pack->memory_adds[i]->grab();
-      pack->data_adds_lengths[i] = pack->memory_adds[i]->size;
-    }
+    pack->data->grab();
+    for (i = 0; i < pack->data_adds.size(); i++)
+      pack->data_adds[i]->grab();
   }
-  pack->length = length;
   pack->source = this;
 
-  enqueued_bytes += pack->length;
+  enqueued_bytes += pack->data->get_size();
+
+  if ((0 > pack->bref) && (0 <= pack->fref)) {
+    int64_t tmp = pack->bref;
+    pack->bref = pack->fref;
+    pack->fref = tmp;
+  }
 
   if (connected_to != 1)
     add_packet2(pack);
@@ -883,11 +865,14 @@ generic_packetizer_c::add_packet(packet_cptr pack) {
 
 void
 generic_packetizer_c::add_packet2(packet_cptr pack) {
-  pack->timecode += correction_timecode_offset + append_timecode_offset;
+  pack->timecode += correction_timecode_offset + append_timecode_offset +
+    ti.packet_delay;
   if (pack->bref >= 0)
-    pack->bref += correction_timecode_offset + append_timecode_offset;
+    pack->bref += correction_timecode_offset + append_timecode_offset +
+      ti.packet_delay;
   if (pack->fref >= 0)
-    pack->fref += correction_timecode_offset + append_timecode_offset;
+    pack->fref += correction_timecode_offset + append_timecode_offset +
+      ti.packet_delay;
 
   if ((htrack_min_cache < 2) && (pack->fref >= 0)) {
     set_track_min_cache(2);
@@ -921,7 +906,7 @@ generic_packetizer_c::add_packet2(packet_cptr pack) {
              "timecode is smaller than that of the previous packet. This "
              "usually means that the source file is a Matroska file that "
              "has not been created 100%% correctly. The timecodes of all "
-             "packets will be adjusted by %lldms in order not to lose any "
+             "packets will be adjusted by " LLD "ms in order not to lose any "
              "data. This may throw A/V sync off, but that can be corrected "
              "with mkvmerge's \"--sync\" option. If you already use "
              "\"--sync\" and you still get this warning then do NOT worry "
@@ -932,10 +917,10 @@ generic_packetizer_c::add_packet2(packet_cptr pack) {
              "contains a bug. In this case you should contact the author "
              "Moritz Bunkus <moritz@bunkus.org>.\n", ti.fname.c_str(), ti.id,
              (needed_timecode_offset + 500000) / 1000000);
-    } else
+    } else if (hack_engaged(ENGAGE_ENABLE_TIMECODE_WARNING))
       mxwarn("pr_generic.cpp/generic_packetizer_c::add_packet(): timecode < "
-             "last_timecode (" FMT_TIMECODE " < " FMT_TIMECODE ") for %lld of "
-             "'%s'. %s\n", ARG_TIMECODE_NS(pack->timecode),
+             "last_timecode (" FMT_TIMECODE " < " FMT_TIMECODE ") for " LLD
+             " of '%s'. %s\n", ARG_TIMECODE_NS(pack->timecode),
              ARG_TIMECODE_NS(safety_last_timecode), ti.id, ti.fname.c_str(),
              BUGMSG);
   }
@@ -944,7 +929,7 @@ generic_packetizer_c::add_packet2(packet_cptr pack) {
   pack->timecode_before_factory = pack->timecode;
 
   packet_queue.push_back(pack);
-  if ((NULL == timecode_factory) ||
+  if ((NULL == timecode_factory.get()) ||
       (TFA_IMMEDIATE == timecode_factory_application_mode))
     apply_factory_once(pack);
   else
@@ -955,26 +940,11 @@ void
 generic_packetizer_c::process_deferred_packets() {
   deque<packet_cptr>::iterator packet;
 
-  foreach(packet, deferred_packets)
+  mxforeach(packet, deferred_packets)
     add_packet2(*packet);
   deferred_packets.clear();
 }
 
-bool 
-generic_packetizer_c::has_enough_packets() const {
-  if (packet_queue.size() < 2)
-    return false;
-
-  const packet_cptr &pack = packet_queue.front();
-  deque<packet_cptr>::const_iterator packet = packet_queue.begin();
-
-  ++packet;
-  while ((packet_queue.end() != packet) &&
-         ((*packet)->timecode_before_factory <= pack->timecode_before_factory))
-    ++packet;
-  return (packet_queue.end() != packet);
-}
-
 packet_cptr
 generic_packetizer_c::get_packet() {
   if ((packet_queue.size() == 0) || !packet_queue.front()->factory_applied)
@@ -983,22 +953,22 @@ generic_packetizer_c::get_packet() {
   packet_cptr pack = packet_queue.front();
   packet_queue.pop_front();
 
-  enqueued_bytes -= pack->length;
+  enqueued_bytes -= pack->data->get_size();
 
   return pack;
 }
 
 void
 generic_packetizer_c::apply_factory_once(packet_cptr &packet) {
-  if (NULL == timecode_factory) {
+  if (NULL == timecode_factory.get()) {
     packet->assigned_timecode = packet->timecode;
     packet->gap_following = false;
   } else
     packet->gap_following = timecode_factory->get_next(packet);
   packet->factory_applied = true;
 
-  mxverb(4, "apply_factory_once(): source %lld t %lld tbf %lld at %lld\n",
-         packet->source->get_source_track_num(), packet->timecode,
+  mxverb(4, "apply_factory_once(): source " LLD " t " LLD " tbf " LLD " at "
+         LLD "\n", packet->source->get_source_track_num(), packet->timecode,
          packet->timecode_before_factory, packet->assigned_timecode);
 
   if (max_timecode_seen < (packet->assigned_timecode + packet->duration))
@@ -1046,7 +1016,7 @@ generic_packetizer_c::apply_factory_short_queueing(packet_cptr_di &p_start) {
     // packetizer has been flushed already.
     if (!has_been_flushed && (packet_queue.end() == p_end))
       return;
-  
+
     // Now assign timecodes to the ones between p_start and p_end...
     for (p_current = p_start + 1; p_current != p_end; ++p_current)
       apply_factory_once(*p_current);
@@ -1123,13 +1093,13 @@ generic_packetizer_c::displace(float by_ns) {
   ti.async.displacement += (int64_t)by_ns;
   if (initial_displacement < 0) {
     if (ti.async.displacement < initial_displacement) {
-      mxverb(3, "EODIS 1 by %f dis is now %lld with idis %lld\n",
+      mxverb(3, "EODIS 1 by %f dis is now " LLD " with idis " LLD "\n",
              by_ns, ti.async.displacement, initial_displacement);
       initial_displacement = 0;
     }
   } else if (iabs(initial_displacement - ti.async.displacement) <
              (by_ns / 2)) {
-    mxverb(3, "EODIS 2 by %f dis is now %lld with idis %lld\n",
+    mxverb(3, "EODIS 2 by %f dis is now " LLD " with idis " LLD "\n",
            by_ns, ti.async.displacement, initial_displacement);
     initial_displacement = 0;
   }
@@ -1139,13 +1109,13 @@ void
 generic_packetizer_c::force_duration_on_last_packet() {
   if (packet_queue.empty()) {
     mxverb(2, "force_duration_on_last_packet: packet queue is empty for "
-           "'%s'/%lld\n", ti.fname.c_str(), ti.id);
+           "'%s'/" LLD "\n", ti.fname.c_str(), ti.id);
     return;
   }
   packet_cptr &packet = packet_queue.back();
   packet->duration_mandatory = true;
   mxverb(2, "force_duration_on_last_packet: forcing at " FMT_TIMECODE " with "
-         "%.3fms for '%s'/%lld\n", ARG_TIMECODE_NS(packet->timecode),
+         "%.3fms for '%s'/" LLD "\n", ARG_TIMECODE_NS(packet->timecode),
          packet->duration / 1000.0, ti.fname.c_str(), ti.id);
 }
 
@@ -1195,6 +1165,7 @@ generic_packetizer_c::connect(generic_packetizer_c *src,
   hcompression = src->hcompression;
   compressor = compressor_c::create(hcompression);
   last_cue_timecode = src->last_cue_timecode;
+  timecode_factory = src->timecode_factory;
   correction_timecode_offset = 0;
   if (_append_timecode_offset == -1)
     append_timecode_offset = src->max_timecode_seen;
@@ -1215,7 +1186,7 @@ generic_packetizer_c::set_displacement_maybe(int64_t displacement) {
 
 bool
 generic_packetizer_c::contains_gap() {
-  if (timecode_factory != NULL)
+  if (NULL != timecode_factory.get())
     return timecode_factory->contains_gap();
   else
     return false;
@@ -1229,43 +1200,41 @@ generic_packetizer_c::flush() {
 
 //--------------------------------------------------------------------
 
-#define add_all_requested_track_ids(container) \
-  for (i = 0; i < ti.container.size(); i++) \
-    add_requested_track_id(ti.container[i].id);
+#define add_all_requested_track_ids(type, container) \
+  for (map<int64_t, type>::const_iterator i = ti.container.begin(); \
+       ti.container.end() != i; ++i) \
+    add_requested_track_id(i->first);
 #define add_all_requested_track_ids2(container) \
-  for (i = 0; i < ti.container.size(); i++) \
+  for (int i = 0; i < ti.container.size(); i++) \
     add_requested_track_id(ti.container[i]);
 
 generic_reader_c::generic_reader_c(track_info_c &_ti):
-  ti(_ti) {
-  int i;
-
-  max_timecode_seen = 0;
-  appending = false;
-  chapters = NULL;
-  ptzr_first_packet = NULL;
-  num_video_tracks = 0;
-  num_audio_tracks = 0;
-  num_subtitle_tracks = 0;
+  ti(_ti),
+  ptzr_first_packet(NULL),
+  max_timecode_seen(0),
+  chapters(NULL),
+  appending(false),
+  num_video_tracks(0), num_audio_tracks(0), num_subtitle_tracks(0),
+  reference_timecode_tolerance(0) {
 
   add_all_requested_track_ids2(atracks);
   add_all_requested_track_ids2(vtracks);
   add_all_requested_track_ids2(stracks);
   add_all_requested_track_ids2(btracks);
-  add_all_requested_track_ids(all_fourccs);
-  add_all_requested_track_ids(display_properties);
-  add_all_requested_track_ids(audio_syncs);
-  add_all_requested_track_ids(cue_creations);
-  add_all_requested_track_ids2(default_track_flags);
-  add_all_requested_track_ids(languages);
-  add_all_requested_track_ids(sub_charsets);
-  add_all_requested_track_ids(all_tags);
-  add_all_requested_track_ids2(aac_is_sbr);
-  add_all_requested_track_ids(packet_delays);
-  add_all_requested_track_ids(compression_list);
-  add_all_requested_track_ids(track_names);
-  add_all_requested_track_ids(all_ext_timecodes);
-  add_all_requested_track_ids(pixel_crop_list);
+  add_all_requested_track_ids(string, all_fourccs);
+  add_all_requested_track_ids(display_properties_t, display_properties);
+  add_all_requested_track_ids(audio_sync_t, audio_syncs);
+  add_all_requested_track_ids(cue_strategy_e, cue_creations);
+  add_all_requested_track_ids(bool, default_track_flags);
+  add_all_requested_track_ids(string, languages);
+  add_all_requested_track_ids(string, sub_charsets);
+  add_all_requested_track_ids(string, all_tags);
+  add_all_requested_track_ids(bool, all_aac_is_sbr);
+  add_all_requested_track_ids(int64_t, packet_delays);
+  add_all_requested_track_ids(compression_method_e, compression_list);
+  add_all_requested_track_ids(string, track_names);
+  add_all_requested_track_ids(string, all_ext_timecodes);
+  add_all_requested_track_ids(pixel_crop_t, pixel_crop_list);
 }
 
 generic_reader_c::~generic_reader_c() {
@@ -1338,7 +1307,7 @@ generic_reader_c::set_timecode_offset(int64_t offset) {
   vector<generic_packetizer_c *>::const_iterator it;
 
   max_timecode_seen = offset;
-  foreach(it, reader_packetizers)
+  mxforeach(it, reader_packetizers)
     (*it)->correction_timecode_offset = offset;
 }
 
@@ -1346,7 +1315,7 @@ void
 generic_reader_c::set_headers() {
   vector<generic_packetizer_c *>::const_iterator it;
 
-  foreach(it, reader_packetizers)
+  mxforeach(it, reader_packetizers)
     (*it)->set_headers();
 }
 
@@ -1354,7 +1323,7 @@ void
 generic_reader_c::set_headers_for_track(int64_t tid) {
   vector<generic_packetizer_c *>::const_iterator it;
 
-  foreach(it, reader_packetizers)
+  mxforeach(it, reader_packetizers)
     if ((*it)->ti.id == tid) {
       (*it)->set_headers();
       break;
@@ -1380,7 +1349,7 @@ generic_reader_c::check_track_ids_and_packetizers() {
       }
 
     if (!found)
-      mxwarn(FMT_FN "A track with the ID %lld was requested but not found "
+      mxwarn(FMT_FN "A track with the ID " LLD " was requested but not found "
              "in the file. The corresponding option will be ignored.\n",
              ti.fname.c_str(), requested_track_ids[r]);
   }
@@ -1410,7 +1379,7 @@ generic_reader_c::get_queued_bytes() {
   int64_t bytes;
 
   bytes = 0;
-  foreach(it, reader_packetizers)
+  mxforeach(it, reader_packetizers)
     bytes += (*it)->get_queued_bytes();
 
   return bytes;
@@ -1420,7 +1389,7 @@ void
 generic_reader_c::flush_packetizers() {
   vector<generic_packetizer_c *>::const_iterator it;
 
-  foreach(it, reader_packetizers)
+  mxforeach(it, reader_packetizers)
     (*it)->flush();
 }
 
@@ -1442,22 +1411,24 @@ track_info_c::track_info_c():
   aspect_ratio_given(false),
   aspect_ratio_is_factor(false),
   display_dimensions_given(false),
-  cues(CUE_STRATEGY_NONE),
+  cues(CUE_STRATEGY_UNSPECIFIED),
   default_track(false),
-  tags_ptr(-1),
+  default_track_flag_present(false),
   tags(NULL),
+  aac_is_sbr(-1),
   packet_delay(0),
-  compression(COMPRESSION_NONE),
+  compression(COMPRESSION_UNSPECIFIED),
+  pixel_cropping_specified(false),
+  stereo_mode(STEREO_MODE_UNSPECIFIED),
+  nalu_size_length(0),
   no_chapters(false),
   no_attachments(false),
   no_tags(false),
   avi_audio_sync_enabled(false) {
 
-  memset(fourcc, 0, 5);
   async.linear = 1.0;
   async.displacement = 0;
   memset(&pixel_cropping, 0, sizeof(pixel_crop_t));
-  pixel_cropping.id = -2;
 }
 
 void
@@ -1492,7 +1463,6 @@ track_info_c::operator =(const track_info_c &src) {
   private_data = (unsigned char *)safememdup(src.private_data, private_size);
 
   all_fourccs = src.all_fourccs;
-  memcpy(fourcc, src.fourcc, 5);
 
   display_properties = src.display_properties;
   aspect_ratio = src.aspect_ratio;
@@ -1508,6 +1478,7 @@ track_info_c::operator =(const track_info_c &src) {
 
   default_track_flags = src.default_track_flags;
   default_track = src.default_track;
+  default_track_flag_present = src.default_track_flag_present;
 
   languages = src.languages;
   language = src.language;
@@ -1516,12 +1487,13 @@ track_info_c::operator =(const track_info_c &src) {
   sub_charset = src.sub_charset;
 
   all_tags = src.all_tags;
-  tags_ptr = src.tags_ptr;
+  tags_file_name = src.tags_file_name;
   if (src.tags != NULL)
     tags = static_cast<KaxTags *>(src.tags->Clone());
   else
     tags = NULL;
 
+  all_aac_is_sbr = src.all_aac_is_sbr;
   aac_is_sbr = src.aac_is_sbr;
 
   packet_delay = src.packet_delay;
@@ -1538,6 +1510,13 @@ track_info_c::operator =(const track_info_c &src) {
 
   pixel_crop_list = src.pixel_crop_list;
   pixel_cropping = src.pixel_cropping;
+  pixel_cropping_specified = src.pixel_cropping_specified;
+
+  stereo_mode_list = src.stereo_mode_list;
+  stereo_mode = src.stereo_mode;
+
+  nalu_size_lengths = src.nalu_size_lengths;
+  nalu_size_length = src.nalu_size_length;
 
   no_chapters = src.no_chapters;
   no_attachments = src.no_attachments;
