@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: jobs.cpp 2849 2005-03-13 17:47:27Z mosu $
+   $Id: jobs.cpp 3559 2007-07-05 14:56:32Z mosu $
 
    job queue management dialog
 
@@ -76,11 +76,11 @@ job_run_dialog::job_run_dialog(wxWindow *parent,
   g_progress = new wxGauge(this, -1, 100);
   siz_fg->Add(g_progress, 1, wxALIGN_CENTER_VERTICAL | wxGROW, 0);
   siz_sb->Add(siz_fg, 0, wxGROW | wxLEFT | wxRIGHT | wxTOP, 10);
-  siz_sb->Add(0, 10, 0, 0, 0);
+  siz_sb->AddSpacer(10);
 
   siz_sb->Add(new wxStaticText(this, -1, wxT("Log output:")),
               wxALIGN_LEFT, wxLEFT, 10);
-  siz_sb->Add(0, 5, 0, 0, 0);
+  siz_sb->AddSpacer(5);
   tc_log = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition,
                           wxSize(450, 150), wxTE_MULTILINE | wxTE_READONLY);
   siz_sb->Add(tc_log, 1, wxGROW | wxLEFT | wxRIGHT | wxBOTTOM, 10);
@@ -148,7 +148,7 @@ job_run_dialog::start_next_job() {
   ndx = jobs_to_start[current_job];
   st_jobs->SetLabel(wxString::Format(wxT("Processing job %d/%d"),
                                      current_job + 1,
-                                     jobs_to_start.size()));
+                                     (int)jobs_to_start.size()));
   st_current->SetLabel(wxString::Format(wxT("Current job ID %d:"),
                                         jobs[ndx].id));
 
@@ -158,7 +158,7 @@ job_run_dialog::start_next_job() {
   opt_file_name.Printf(wxT("%smmg-mkvmerge-options-%d-%d"),
                        get_temp_dir().c_str(),
                        (int)wxGetProcessId(), (int)time(NULL));
-  
+
   try {
     opt_file = new wxFile(opt_file_name, wxFile::write);
   } catch (...) {
@@ -334,7 +334,7 @@ job_log_dialog::job_log_dialog(wxWindow *parent,
   siz_all = new wxBoxSizer(wxVERTICAL);
   siz_all->Add(new wxStaticText(this, -1, wxT("Output of the selected jobs:")),
                0, wxALIGN_LEFT | wxLEFT | wxTOP, 10);
-  siz_all->Add(0, 5, 0, 0, 0);
+  siz_all->AddSpacer(5);
   siz_all->Add(new wxTextCtrl(this, -1, log, wxDefaultPosition,
                               wxSize(550, 150), wxTE_MULTILINE |
                               wxTE_READONLY),
@@ -363,8 +363,12 @@ job_log_dialog::on_save(wxCommandEvent &evt) {
                       wxT(""), wxT("Text files (*.txt)|*.txt|" ALLFILES),
                       wxSAVE | wxOVERWRITE_PROMPT);
   if(dialog.ShowModal() == wxID_OK) {
+    wxString copy = text;
+#if defined(SYS_WINDOWS)
+    copy.Replace(wxT("\n"), wxT("\r\n"));
+#endif
     wxFile file(dialog.GetPath(), wxFile::write);
-    file.Write(text);
+    file.Write(copy);
     last_open_dir = dialog.GetDirectory();
   }
 }
@@ -383,9 +387,7 @@ job_dialog::job_dialog(wxWindow *parent):
   siz_all = new wxBoxSizer(wxVERTICAL);
   siz_all->Add(new wxStaticText(this, -1, wxT("Current and past jobs:")),
                0, wxALIGN_LEFT | wxALL, 10);
-  lv_jobs =
-    new wxListView(this, ID_JOBS_LV_JOBS, wxDefaultPosition, wxSize(400, -1),
-                   wxLC_REPORT | wxSUNKEN_BORDER);
+  lv_jobs = new jobdlg_list_view(this, ID_JOBS_LV_JOBS);
 
   item.m_mask = wxLIST_MASK_TEXT;
   item.m_text = wxT("ID");
@@ -402,23 +404,21 @@ job_dialog::job_dialog(wxWindow *parent):
   item.m_text = wxT("Finished on");
   lv_jobs->InsertColumn(5, item);
 
-  if (jobs.size() == 0) {
-    dummy = lv_jobs->InsertItem(0, wxT("12345"));
-    lv_jobs->SetItem(dummy, 1, wxT("aborted"));
-    lv_jobs->SetItem(dummy, 2, wxT("2004-05-06 07:08:09"));
-    lv_jobs->SetItem(dummy, 3, wxT("2004-05-06 07:08:09"));
-    lv_jobs->SetItem(dummy, 4, wxT("2004-05-06 07:08:09"));
-    lv_jobs->SetItem(dummy, 5, wxT("2004-05-06 07:08:09"));
+  for (i = 0; i < jobs.size(); i++)
+    create_list_item(i);
 
-  } else
-    for (i = 0; i < jobs.size(); i++)
-      create_list_item(i);
+  dummy = lv_jobs->InsertItem(0, wxT("12345"));
+  lv_jobs->SetItem(dummy, 1, wxT("aborted"));
+  lv_jobs->SetItem(dummy, 2, wxT("2004-05-06 07:08:09"));
+  lv_jobs->SetItem(dummy, 3, wxT("2004-05-06 07:08:09"));
+  lv_jobs->SetItem(dummy, 4, wxT("2004-05-06 07:08:09"));
+  lv_jobs->SetItem(dummy, 5, wxT("2004-05-06 07:08:09"));
 
   for (i = 0; i < 6; i++)
     lv_jobs->SetColumnWidth(i, wxLIST_AUTOSIZE);
 
-  if (jobs.size() == 0)
-    lv_jobs->DeleteItem(0);
+  lv_jobs->DeleteItem(0);
+
   siz_line = new wxBoxSizer(wxHORIZONTAL);
   siz_line->Add(lv_jobs, 1, wxGROW | wxRIGHT, 10);
 
@@ -429,7 +429,7 @@ job_dialog::job_dialog(wxWindow *parent):
   b_down = new wxButton(this, ID_JOBS_B_DOWN, wxT("&Down"));
   b_down->SetToolTip(TIP("Move the selected job(s) down"));
   siz_b_right->Add(b_down, 0, wxLEFT | wxBOTTOM, 10);
-  siz_b_right->Add(0, 15, 0, 0, 0);
+  siz_b_right->AddSpacer(15);
 
   b_reenable = new wxButton(this, ID_JOBS_B_REENABLE, wxT("&Re-enable"));
   b_reenable->SetToolTip(TIP("Re-enable the selected job(s)"));
@@ -438,12 +438,12 @@ job_dialog::job_dialog(wxWindow *parent):
   b_disable->SetToolTip(TIP("Disable the selected job(s) and sets their "
                             "status to 'done'"));
   siz_b_right->Add(b_disable, 0, wxLEFT | wxBOTTOM, 10);
-  siz_b_right->Add(0, 15, 0, 0, 0);
+  siz_b_right->AddSpacer(15);
 
   b_delete = new wxButton(this, ID_JOBS_B_DELETE, wxT("D&elete"));
   b_delete->SetToolTip(TIP("Delete the selected job(s) from the job queue"));
   siz_b_right->Add(b_delete, 0, wxLEFT | wxBOTTOM, 10);
-  siz_b_right->Add(0, 15, 0, 0, 0);
+  siz_b_right->AddSpacer(15);
 
   b_view_log = new wxButton(this, ID_JOBS_B_VIEW_LOG, wxT("&View log"));
   b_view_log->SetToolTip(TIP("View the output that mkvmerge generated during "
@@ -518,9 +518,10 @@ job_dialog::create_list_item(int i) {
 }
 
 void
-job_dialog::enable_buttons(bool enable) {
-  b_up->Enable(enable);
-  b_down->Enable(enable);
+job_dialog::enable_buttons(bool enable,
+                           bool enable_up_down) {
+  b_up->Enable(enable && enable_up_down);
+  b_down->Enable(enable && enable_up_down);
   b_reenable->Enable(enable);
   b_disable->Enable(enable);
   b_delete->Enable(enable);
@@ -559,23 +560,19 @@ job_dialog::on_start_selected(wxCommandEvent &evt) {
 void
 job_dialog::on_delete(wxCommandEvent &evt) {
   int i, j;
-  vector<job_t>::iterator dit;
   vector<bool> selected;
 
   for (i = 0; i < lv_jobs->GetItemCount(); i++)
     selected.push_back(lv_jobs->IsSelected(i));
-  dit = jobs.begin();
   i = 0;
-  j =0;
+  j = 0;
   while (i < jobs.size()) {
     if (selected[j]) {
-      wxRemoveFile(wxString::Format(wxT("jobs/%d.mmg"), dit->id));
-      jobs.erase(dit);
+      wxRemoveFile(wxString::Format(wxT("jobs/%d.mmg"), jobs[i].id));
+      jobs.erase(jobs.begin() + i);
       lv_jobs->DeleteItem(i);
-    } else {
-      dit++;
+    } else
       i++;
-    }
     j++;
   }
 
@@ -732,7 +729,8 @@ job_dialog::on_view_log(wxCommandEvent &evt) {
 
 void
 job_dialog::on_item_selected(wxListEvent &evt) {
-  enable_buttons(lv_jobs->GetSelectedItemCount() > 0);
+  enable_buttons(lv_jobs->GetSelectedItemCount() > 0,
+                 lv_jobs->GetSelectedItemCount() < lv_jobs->GetItemCount());
 }
 
 void
@@ -760,6 +758,33 @@ job_dialog::start_jobs(vector<int> &jobs_to_start) {
   }
 }
 
+void
+job_dialog::on_key_pressed(wxKeyEvent &evt) {
+  if (1 == evt.GetKeyCode()) {  // Ctrl-A
+    int i;
+
+    for (i = 0; lv_jobs->GetItemCount() > i; ++i)
+      lv_jobs->Select(i, true);
+    enable_buttons(lv_jobs->GetSelectedItemCount() > 0, false);
+  } else
+    evt.Skip(true);
+}
+
+jobdlg_list_view::jobdlg_list_view(wxWindow *parent,
+                                   wxWindowID id):
+  wxListView(parent, id, wxDefaultPosition, wxSize(400, -1),
+             wxLC_REPORT | wxSUNKEN_BORDER) {
+}
+
+void
+jobdlg_list_view::on_key_pressed(wxKeyEvent &evt) {
+  if (1 == evt.GetKeyCode())    // Ctrl-A
+    ((job_dialog *)GetParent())->on_key_pressed(evt);
+  else
+    evt.Skip(true);
+}
+
+
 IMPLEMENT_CLASS(job_run_dialog, wxDialog);
 BEGIN_EVENT_TABLE(job_run_dialog, wxDialog)
   EVT_BUTTON(ID_JOBS_B_ABORT, job_run_dialog::on_abort)
@@ -786,3 +811,8 @@ BEGIN_EVENT_TABLE(job_dialog, wxDialog)
   EVT_LIST_ITEM_SELECTED(ID_JOBS_LV_JOBS, job_dialog::on_item_selected)
   EVT_LIST_ITEM_DESELECTED(ID_JOBS_LV_JOBS, job_dialog::on_item_selected)
 END_EVENT_TABLE();
+
+IMPLEMENT_CLASS(jobdlg_list_view, wxListView);
+BEGIN_EVENT_TABLE(jobdlg_list_view, wxListView)
+  EVT_CHAR(jobdlg_list_view::on_key_pressed)
+END_EVENT_TABLE();
