@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mmg.cpp 3463 2007-01-12 16:02:16Z mosu $
+   $Id: mmg.cpp 3568 2007-07-08 17:30:57Z mosu $
 
    main stuff
 
@@ -45,17 +45,16 @@
 #include "mmg.h"
 #include "mmg_dialog.h"
 #include "mux_dialog.h"
+#include "options_dialog.h"
 #include "tab_attachments.h"
 #include "tab_chapters.h"
 #include "tab_input.h"
 #include "tab_global.h"
-#include "tab_settings.h"
 #include "xml_element_mapping.h"
 
 mmg_app *app;
 mmg_dialog *mdlg;
 wxString last_open_dir;
-wxString mkvmerge_path;
 vector<wxString> last_settings;
 vector<wxString> last_chapters;
 vector<mmg_file_t> files;
@@ -133,11 +132,17 @@ wxString cli_options[][2] = {
   { wxT("--engage force_passthrough_packetizer"),
     wxT("Forces the Matroska reader to use the generic passthrough "
         "packetizer even for known and supported track types.") },
+  { wxT("--engage allow_avc_in_vfw_mode"),
+    wxT("Allows storing AVC/h.264 video in Video-for-Windows compatibility "
+        "mode, e.g. when it is read from an AVI") },
   { wxT("--engage use_simpleblock"),
     wxT("Enable use of SimpleBlock instead of BlockGroup when possible.") },
   { wxT("--engage old_aac_codecid"),
     wxT("Use the old AAC codec IDs (e.g. 'A_AAC/MPEG4/SBR') instead of the "
         "new one ('A_AAC').") },
+  { wxT("--engage use_codec_state"),
+    wxT("Allows the use of the CodecState element. This is used for e.g. "
+        "MPEG-1/-2 video tracks for storing the sequence headers.") },
   { wxT("--engage cow"),
     wxT("No help available.") },
   { wxT(""), wxT("") }
@@ -166,7 +171,7 @@ cli_options_dlg::cli_options_dlg(wxWindow *parent):
   int i;
 
   siz_all = new wxBoxSizer(wxVERTICAL);
-  siz_all->Add(0, 10, 0, 0, 0);
+  siz_all->AddSpacer(10);
   siz_all->Add(new wxStaticText(this, -1, wxT("Here you can add more command "
                                               "line options either by\n"
                                               "entering them below or by "
@@ -174,17 +179,17 @@ cli_options_dlg::cli_options_dlg(wxWindow *parent):
                                               "down box and pressing the "
                                               "'add' button.")),
                0, wxLEFT | wxRIGHT, 10);
-  siz_all->Add(0, 10, 0, 0, 0);
+  siz_all->AddSpacer(10);
   siz_all->Add(new wxStaticText(this, -1, wxT("Command line options:")),
                0, wxLEFT, 10);
-  siz_all->Add(0, 5, 0, 0, 0);
+  siz_all->AddSpacer(5);
   tc_options = new wxTextCtrl(this, -1);
   siz_all->Add(tc_options, 0, wxGROW | wxLEFT | wxRIGHT, 10);
 
-  siz_all->Add(0, 10, 0, 0, 0);
+  siz_all->AddSpacer(10);
   siz_all->Add(new wxStaticText(this, -1, wxT("Available options:")),
                0, wxLEFT, 10);
-  siz_all->Add(0, 5, 0, 0, 0);
+  siz_all->AddSpacer(5);
   siz_line = new wxBoxSizer(wxHORIZONTAL);
   cob_option = new wxComboBox(this, ID_CLIOPTIONS_COB);
   i = 0;
@@ -198,27 +203,27 @@ cli_options_dlg::cli_options_dlg(wxWindow *parent):
   siz_line->Add(button, 0, wxALIGN_CENTER_VERTICAL, 0);
   siz_all->Add(siz_line, 0, wxGROW | wxLEFT | wxRIGHT, 10);
 
-  siz_all->Add(0, 10, 0, 0, 0);
+  siz_all->AddSpacer(10);
   siz_all->Add(new wxStaticText(this, -1, wxT("Description:")), 0, wxLEFT, 10);
-  siz_all->Add(0, 5, 0, 0, 0);
+  siz_all->AddSpacer(5);
   tc_description =
     new wxTextCtrl(this, -1, cli_options[0][1], wxDefaultPosition,
                    wxDefaultSize, wxTE_MULTILINE | wxTE_READONLY |
                    wxTE_WORDWRAP);
   siz_all->Add(tc_description, 1, wxGROW | wxLEFT | wxRIGHT, 10);
 
-  siz_all->Add(0, 10, 0, 0, 0);
+  siz_all->AddSpacer(10);
   siz_all->Add(new wxStaticLine(this, -1), 0, wxGROW | wxLEFT | wxRIGHT, 10);
 
-  siz_all->Add(0, 10, 0, 0, 0);
+  siz_all->AddSpacer(10);
+
   siz_line = new wxBoxSizer(wxHORIZONTAL);
-  siz_line->Add(1, 0, 1, wxGROW, 0);
-  siz_line->Add(new wxButton(this, wxID_OK, wxT("Ok")), 0, 0, 0);
-  siz_line->Add(1, 0, 1, wxGROW, 0);
-  siz_line->Add(new wxButton(this, wxID_CANCEL, wxT("Cancel")), 0, 0, 0);
-  siz_line->Add(1, 0, 1, wxGROW, 0);
+  siz_line->AddStretchSpacer();
+  siz_line->Add(CreateStdDialogButtonSizer(wxOK | wxCANCEL), 0, 0, 0);
+  siz_line->AddSpacer(10);
+
   siz_all->Add(siz_line, 0, wxGROW, 0);
-  siz_all->Add(0, 10, 0, 0, 0);
+  siz_all->AddSpacer(10);
 
   SetSizer(siz_all);
 }
@@ -256,6 +261,54 @@ cli_options_dlg::go(wxString &options) {
   return false;
 }
 
+class show_text_dlg: public wxDialog {
+  DECLARE_CLASS(show_text_dlg);
+  DECLARE_EVENT_TABLE();
+public:
+  show_text_dlg(wxWindow *parent, const wxString &title, const wxString &text);
+};
+
+
+show_text_dlg::show_text_dlg(wxWindow *parent,
+                             const wxString &title,
+                             const wxString &text):
+  wxDialog(parent, 0, title, wxDefaultPosition, wxSize(400, 350)) {
+
+  wxBoxSizer *siz_all, *siz_line;
+  wxTextCtrl *tc_message;
+
+  siz_all = new wxBoxSizer(wxVERTICAL);
+
+  tc_message = new wxTextCtrl(this, 0, text, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE | wxTE_READONLY);
+
+  siz_line = new wxBoxSizer(wxHORIZONTAL);
+  siz_line->Add(tc_message, 1, wxALL | wxGROW, 10);
+
+  siz_all->Add(siz_line, 1, wxGROW, 0);
+
+  siz_line = new wxBoxSizer(wxHORIZONTAL);
+  siz_line->AddStretchSpacer();
+  siz_line->Add(CreateStdDialogButtonSizer(wxOK), 0, 0, 0);
+  siz_line->AddSpacer(10);
+
+  siz_all->Add(siz_line, 0, wxGROW, 0);
+  siz_all->AddSpacer(10);
+
+  SetSizer(siz_all);
+}
+
+void
+mmg_options_t::validate() {
+  if (   (ODM_FROM_FIRST_INPUT_FILE != output_directory_mode)
+      && (ODM_PREVIOUS              != output_directory_mode)
+      && (ODM_FIXED                 != output_directory_mode))
+    output_directory_mode = ODM_FROM_FIRST_INPUT_FILE;
+
+  if (   (priority != wxU("highest")) && (priority != wxU("higher")) && (priority != wxU("normal"))
+      && (priority != wxU("lower"))   && (priority != wxU("lowest")))
+    priority = wxU("normal");
+}
+
 #if WXUNICODE
 wxString
 UTFstring_to_wxString(const UTFstring &u) {
@@ -588,7 +641,7 @@ default_track_checked(char type) {
   int i;
 
   for (i = 0; i < tracks.size(); i++)
-    if ((tracks[i]->type == type) && tracks[i]->default_track)
+    if ((tracks[i]->type == type) && (1 == tracks[i]->default_track))
       return i;
   return -1;
 }
@@ -618,8 +671,6 @@ mmg_dialog::mmg_dialog():
   wxFrame(NULL, -1, wxT("mkvmerge GUI v" VERSION " ('" VERSIONNAME "')")) {
   wxBoxSizer *bs_main;
   wxPanel *panel;
-  wxConfigBase *cfg;
-  int window_pos_x, window_pos_y;
 
   mdlg = this;
 
@@ -637,6 +688,8 @@ mmg_dialog::mmg_dialog():
   file_menu->Append(ID_M_FILE_SETOUTPUT, wxT("Set &output file"),
                     wxT("Select the file you want to write to"));
   file_menu->AppendSeparator();
+  file_menu->Append(ID_M_FILE_OPTIONS, wxT("Op&tions\tCtrl-P"), wxT("Change mmg's preferences and options"));
+  file_menu->AppendSeparator();
   file_menu->Append(ID_M_FILE_EXIT, wxT("&Quit\tCtrl-Q"),
                     wxT("Quit the application"));
 
@@ -647,6 +700,10 @@ mmg_dialog::mmg_dialog():
   muxing_menu->Append(ID_M_MUXING_START,
                       wxT("Sta&rt muxing (run mkvmerge)\tCtrl-R"),
                       wxT("Run mkvmerge and start the muxing process"));
+  muxing_menu->AppendSeparator();
+  muxing_menu->Append(ID_M_MUXING_SHOW_CMDLINE,
+                      wxT("S&how the command line"),
+                      wxT("Show the command line mmg creates for mkvmerge"));
   muxing_menu->Append(ID_M_MUXING_COPY_CMDLINE,
                       wxT("&Copy command line to clipboard"),
                       wxT("Copy the command line to the clipboard"));
@@ -663,7 +720,7 @@ mmg_dialog::mmg_dialog():
                       wxT("Adds the current settings as a new job entry to "
                           "the job queue"));
   muxing_menu->Append(ID_M_MUXING_MANAGE_JOBS,
-                      wxT("&Manage jobs\tCtrl-j"),
+                      wxT("&Manage jobs\tCtrl-J"),
                       wxT("Brings up the job queue editor"));
   muxing_menu->AppendSeparator();
   muxing_menu->Append(ID_M_MUXING_ADD_CLI_OPTIONS,
@@ -698,10 +755,8 @@ mmg_dialog::mmg_dialog():
   window_menu->Append(ID_M_WINDOW_INPUT, wxT("&Input\tAlt-1"));
   window_menu->Append(ID_M_WINDOW_ATTACHMENTS, wxT("&Attachments\tAlt-2"));
   window_menu->Append(ID_M_WINDOW_GLOBAL, wxT("&Global options\tAlt-3"));
-  window_menu->Append(ID_M_WINDOW_SETTINGS, wxT("&Settings\tAlt-4"));
   window_menu->AppendSeparator();
-  window_menu->Append(ID_M_WINDOW_CHAPTEREDITOR,
-                      wxT("&Chapter editor\tAlt-5"));
+  window_menu->Append(ID_M_WINDOW_CHAPTEREDITOR, wxT("&Chapter editor\tAlt-4"));
 
   wxMenu *help_menu = new wxMenu();
   help_menu->Append(ID_M_HELP_HELP, wxT("&Help\tF1"),
@@ -730,7 +785,6 @@ mmg_dialog::mmg_dialog():
   notebook =
     new wxNotebook(panel, ID_NOTEBOOK, wxDefaultPosition, wxSize(500, 500),
                    wxNB_TOP);
-  settings_page = new tab_settings(notebook);
   input_page = new tab_input(notebook);
   attachments_page = new tab_attachments(notebook);
   global_page = new tab_global(notebook);
@@ -739,7 +793,6 @@ mmg_dialog::mmg_dialog():
   notebook->AddPage(input_page, wxT("Input"));
   notebook->AddPage(attachments_page, wxT("Attachments"));
   notebook->AddPage(global_page, wxT("Global"));
-  notebook->AddPage(settings_page, wxT("Settings"));
   notebook->AddPage(chapter_editor_page, wxT("Chapter Editor"));
 
   bs_main->Add(notebook, 1, wxALIGN_CENTER_HORIZONTAL | wxLEFT | wxRIGHT |
@@ -751,25 +804,14 @@ mmg_dialog::mmg_dialog():
                wxGROW, 5);
 
   tc_output = new wxTextCtrl(panel, ID_TC_OUTPUT, wxT(""));
-  sbs_low->Add(0, 5, 0, 0, 0);
+  sbs_low->AddSpacer(5);
   sbs_low->Add(tc_output, 1, wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM |
                wxGROW, 2);
-  sbs_low->Add(0, 5, 0, 0, 0);
+  sbs_low->AddSpacer(5);
 
   b_browse_output = new wxButton(panel, ID_B_BROWSEOUTPUT, wxT("Browse"));
   sbs_low->Add(b_browse_output, 0, wxALIGN_CENTER_VERTICAL | wxALL, 3);
 
-  sb_commandline = new wxStaticBox(panel, -1, wxT("Command line"));
-  wxStaticBoxSizer *sbs_low2 =
-    new wxStaticBoxSizer(sb_commandline, wxHORIZONTAL);
-  bs_main->Add(sbs_low2, 0, wxALIGN_CENTER_HORIZONTAL | wxLEFT | wxRIGHT |
-               wxGROW, 5);
-
-  tc_cmdline =
-    new wxTextCtrl(panel, ID_TC_CMDLINE, wxT(""), wxDefaultPosition,
-                   wxSize(490, 50), wxTE_READONLY | wxTE_LINEWRAP |
-                   wxTE_MULTILINE);
-  sbs_low2->Add(tc_cmdline, 1, wxALIGN_CENTER_VERTICAL | wxALL, 3);
 
   wxBoxSizer *bs_buttons = new wxBoxSizer(wxHORIZONTAL);
 
@@ -793,31 +835,22 @@ mmg_dialog::mmg_dialog():
   bs_main->Add(bs_buttons, 0, wxALIGN_CENTER_HORIZONTAL);
 
 #ifdef SYS_WINDOWS
-  SetSizeHints(600, 740);
-  SetSize(600, 740);
+  SetSizeHints(600, 680);
+  SetSize(600, 680);
 #else
-  SetSizeHints(600, 718);
-  SetSize(600, 718);
+  SetSizeHints(600, 660);
+  SetSize(600, 660);
 #endif
 
+  load_preferences();
+
+  log_window->Show(options.gui_debugging);
+  set_on_top(options.on_top);
+  query_mkvmerge_capabilities();
+
   muxing_in_progress = false;
-
-  cfg = wxConfigBase::Get();
-  cfg->SetPath(wxT("/GUI"));
-  if (cfg->Read(wxT("window_position_x"), &window_pos_x) &&
-      cfg->Read(wxT("window_position_y"), &window_pos_y) &&
-      (0 < window_pos_x) && (0xffff > window_pos_x) &&
-      (0 < window_pos_y) && (0xffff > window_pos_y))
-    Move(window_pos_x, window_pos_y);
-  cfg->Read(wxT("warned_chapterditor_not_empty"),
-            &warned_chapter_editor_not_empty, false);
-
-  last_open_dir = wxT("");
-  cmdline = wxT("\"") + mkvmerge_path + wxT("\" -o \"") +
-    tc_output->GetValue() + wxT("\" ");
-  tc_cmdline->SetValue(cmdline);
-  cmdline_timer.SetOwner(this, ID_T_UPDATECMDLINE);
-  cmdline_timer.Start(1000);
+  last_open_dir      = wxT("");
+  cmdline            = wxString::Format(wxU("\"%s\" -o \"%s\""), options.mkvmerge.c_str(), tc_output->GetValue().c_str());
 
   load_job_queue();
 
@@ -844,7 +877,11 @@ mmg_dialog::mmg_dialog():
       if ((file.Length() > 0) && (file.c_str()[0] != wxT('/')))
         file = wxGetCwd() + wxT("/") + file;
 #endif
-      load(file);
+
+      if (wxFileName(file).GetExt() == wxU("mmg"))
+        load(file);
+      else
+        input_page->add_file(file, false);
     }
   }
 }
@@ -868,6 +905,7 @@ mmg_dialog::on_browse_output(wxCommandEvent &evt) {
   if(dlg.ShowModal() == wxID_OK) {
     last_open_dir = dlg.GetDirectory();
     tc_output->SetValue(dlg.GetPath());
+    remember_previous_output_directory();
   }
 }
 
@@ -890,22 +928,18 @@ mmg_dialog::on_quit(wxCommandEvent &evt) {
 
 void
 mmg_dialog::on_file_new(wxCommandEvent &evt) {
-  wxFileConfig *cfg;
   wxString tmp_name;
 
-  tmp_name.Printf(wxT("%stempsettings-%d.mmg"),
-                  get_temp_dir().c_str(), (int)wxGetProcessId());
-  cfg = new wxFileConfig(wxT("mkvmerge GUI"), wxT("Moritz Bunkus"), tmp_name);
+  tmp_name.Printf(wxT("%stempsettings-%d.mmg"), get_temp_dir().c_str(), (int)wxGetProcessId());
+  wxFileConfig cfg(wxT("mkvmerge GUI"), wxT("Moritz Bunkus"), tmp_name);
   tc_output->SetValue(wxT(""));
 
-  input_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
+  input_page->load(&cfg, MMG_CONFIG_FILE_VERSION_MAX);
   input_page->on_file_new(evt);
-  attachments_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
-  global_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
-  settings_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
+  attachments_page->load(&cfg, MMG_CONFIG_FILE_VERSION_MAX);
+  global_page->load(&cfg, MMG_CONFIG_FILE_VERSION_MAX);
   notebook->SetSelection(0);
 
-  delete cfg;
   wxRemoveFile(tmp_name);
 
   set_status_bar(wxT("Configuration cleared."));
@@ -931,13 +965,12 @@ mmg_dialog::on_file_load(wxCommandEvent &evt) {
 void
 mmg_dialog::load(wxString file_name,
                  bool used_for_jobs) {
-  wxFileConfig *cfg;
   wxString s;
   int version;
 
-  cfg = new wxFileConfig(wxT("mkvmerge GUI"), wxT("Moritz Bunkus"), file_name);
-  cfg->SetPath(wxT("/mkvmergeGUI"));
-  if (!cfg->Read(wxT("file_version"), &version) || (1 > version) ||
+  wxFileConfig cfg(wxT("mkvmerge GUI"), wxT("Moritz Bunkus"), file_name);
+  cfg.SetPath(wxT("/mkvmergeGUI"));
+  if (!cfg.Read(wxT("file_version"), &version) || (1 > version) ||
       (MMG_CONFIG_FILE_VERSION_MAX < version)) {
     if (used_for_jobs)
       return;
@@ -946,16 +979,14 @@ mmg_dialog::load(wxString file_name,
                  wxOK | wxCENTER | wxICON_ERROR);
     return;
   }
-  cfg->Read(wxT("output_file_name"), &s);
+
+  cfg.Read(wxT("output_file_name"), &s);
   tc_output->SetValue(s);
-  cfg->Read(wxT("cli_options"), &cli_options, wxT(""));
+  cfg.Read(wxT("cli_options"), &cli_options, wxT(""));
 
-  input_page->load(cfg, version);
-  attachments_page->load(cfg, version);
-  global_page->load(cfg, version);
-  settings_page->load(cfg, version);
-
-  delete cfg;
+  input_page->load(&cfg, version);
+  attachments_page->load(&cfg, version);
+  global_page->load(&cfg, version);
 
   if (!used_for_jobs) {
     set_last_settings_in_menu(file_name);
@@ -991,7 +1022,6 @@ mmg_dialog::save(wxString file_name,
   input_page->save(cfg);
   attachments_page->save(cfg);
   global_page->save(cfg);
-  settings_page->save(cfg);
 
   delete cfg;
 
@@ -1120,14 +1150,12 @@ mmg_dialog::on_run(wxCommandEvent &evt) {
 
   if (!input_page->validate_settings() ||
       !attachments_page->validate_settings() ||
-      !global_page->validate_settings() ||
-      !settings_page->validate_settings())
+      !global_page->validate_settings())
     return;
 
   if (!chapter_editor_page->is_empty() &&
       (global_page->tc_chapters->GetValue() == wxT("")) &&
-      (!warned_chapter_editor_not_empty ||
-       settings_page->cb_warn_usage->GetValue())) {
+      (!warned_chapter_editor_not_empty || options.warn_usage)) {
     warned_chapter_editor_not_empty = true;
     if (wxMessageBox(wxT("The chapter editor has been used and contains "
                          "data. However, no chapter file has been selected "
@@ -1146,10 +1174,11 @@ mmg_dialog::on_run(wxCommandEvent &evt) {
       return;
   }
 
-  if (settings_page->cb_ask_before_overwriting->IsChecked() &&
-      !check_before_overwriting())
+  if (options.ask_before_overwriting && !check_before_overwriting())
     return;
 
+  remember_previous_output_directory();
+
   set_on_top(false);
   muxing_in_progress = true;
   new mux_dialog(this);
@@ -1259,6 +1288,14 @@ mmg_dialog::on_about(wxCommandEvent &evt) {
                wxOK | wxCENTER | wxICON_INFORMATION);
 }
 
+void
+mmg_dialog::on_show_cmdline(wxCommandEvent &evt) {
+  update_command_line();
+
+  show_text_dlg dlg(this, wxT("Current command line"), cmdline);
+  dlg.ShowModal();
+}
+
 void
 mmg_dialog::on_save_cmdline(wxCommandEvent &evt) {
   wxFile *file;
@@ -1343,26 +1380,24 @@ mmg_dialog::update_command_line() {
   mmg_file_t *f;
   mmg_attachment_t *a;
   wxString sid, old_cmdline, arg, aids, sids, dids, track_order;
-  wxString append_mapping, format;
+  wxString append_mapping;
 
   old_cmdline = cmdline;
-  cmdline = wxT("\"") + mkvmerge_path + wxT("\" -o \"") +
-    tc_output->GetValue() + wxT("\" ");
+  cmdline = wxT("\"") + options.mkvmerge + wxT("\" -o \"") + tc_output->GetValue() + wxT("\" ");
 
   clargs.Clear();
-  clargs.Add(mkvmerge_path);
+  clargs.Add(options.mkvmerge);
   clargs.Add(wxT("--output-charset"));
   clargs.Add(wxT("UTF-8"));
   clargs.Add(wxT("-o"));
   clargs.Add(tc_output->GetValue());
   args_start = clargs.Count();
 
-  if (settings_page->cob_priority->GetValue() != wxT("normal")) {
+  if (options.priority != wxT("normal")) {
     clargs.Add(wxT("--priority"));
-    clargs.Add(settings_page->cob_priority->GetValue());
+    clargs.Add(options.priority);
   }
 
-  format = wxT(LLD);
   for (fidx = 0; fidx < files.size(); fidx++) {
     f = &files[fidx];
     tracks_selected_here = false;
@@ -1378,9 +1413,7 @@ mmg_dialog::update_command_line() {
         continue;
 
       tracks_selected_here = true;
-      // Avoid compiler warnings about mismatching format and arguments
-      // because mingw does not know about the %I64d syntax.
-      sid.Printf(format, t->id);
+      sid = wxLongLong(t->id).ToString();
 
       if (t->type == wxT('a')) {
         no_audio = false;
@@ -1449,9 +1482,10 @@ mmg_dialog::update_command_line() {
         clargs.Add(sid + wxT(":") + t->track_name);
       }
 
-      if (!t->appending && t->default_track) {
+      if (!t->appending && (0 != t->default_track)) {
         clargs.Add(wxT("--default-track"));
-        clargs.Add(sid);
+        clargs.Add(sid + wxT(":") + (1 == t->default_track ? wxT("yes") :
+                                     wxT("no")));
       }
 
       if (!t->appending && (t->tags.Length() > 0)) {
@@ -1479,10 +1513,7 @@ mmg_dialog::update_command_line() {
         clargs.Add(sid + wxT(":") + t->fps + wxT("fps"));
       }
 
-      if ((FILE_TYPE_AVC_ES == f->container) &&
-          ('v' == t->type) &&
-          (t->ctype.Find(wxT("MPEG-4 part 10 ES")) >= 0) &&
-          (2 != t->nalu_size_length)) {
+      if (0 != t->nalu_size_length) {
         clargs.Add(wxT("--nalu-size-length"));
         clargs.Add(wxString::Format(wxT("%s:%d"), sid.c_str(),
                                     t->nalu_size_length));
@@ -1653,7 +1684,7 @@ mmg_dialog::update_command_line() {
       clargs.Add(strip(opts[i]));
   }
 
-  if (settings_page->cb_always_use_simpleblock->IsChecked()) {
+  if (options.always_use_simpleblock) {
     clargs.Add(wxT("--engage"));
     clargs.Add(wxT("use_simpleblock"));
   }
@@ -1664,9 +1695,6 @@ mmg_dialog::update_command_line() {
     else
       cmdline += wxT(" ") + shell_escape(clargs[i]);
   }
-
-  if (old_cmdline != cmdline)
-    tc_cmdline->SetValue(cmdline);
 }
 
 void
@@ -1789,28 +1817,42 @@ mmg_dialog::set_title_maybe(const wxString &new_title) {
 
 void
 mmg_dialog::set_output_maybe(const wxString &new_output) {
-  if (settings_page->cb_autoset_output_filename->IsChecked() &&
-      (new_output.length() > 0) &&
-      (tc_output->GetValue().length() == 0)) {
-    wxString output(new_output.BeforeLast('.'));
-    bool has_video = false, has_audio = false;
-    vector<mmg_track_t *>::iterator t;
+  if (!options.autoset_output_filename
+      || new_output.empty()
+      || !tc_output->GetValue().empty())
+    return;
 
-    mxforeach(t, tracks) {
-      if ('v' == (*t)->type) {
-        has_video = true;
-        break;
-      } else if ('a' == (*t)->type)
-        has_audio = true;
-    }
-    output += has_video ? wxT(".mkv") : has_audio ? wxT(".mka") : wxT(".mks");
-    tc_output->SetValue(output);
+  bool has_video = false, has_audio = false;
+  vector<mmg_track_t *>::iterator t;
+
+  mxforeach(t, tracks) {
+    if ('v' == (*t)->type) {
+      has_video = true;
+      break;
+    } else if ('a' == (*t)->type)
+      has_audio = true;
   }
+
+  wxString output;
+  wxFileName filename(new_output);
+
+  if (ODM_PREVIOUS == options.output_directory_mode)
+    output = previous_output_directory;
+  else if (ODM_FIXED == options.output_directory_mode)
+    output = options.output_directory;
+
+  if (output.IsEmpty())
+    output = filename.GetPath();
+
+  output += wxFileName::GetPathSeparator() + filename.GetName()
+    + (has_video ? wxU(".mkv") : has_audio ? wxU(".mka") : wxU(".mks"));
+
+  tc_output->SetValue(output);
 }
 
 void
 mmg_dialog::remove_output_filename() {
-  if (settings_page->cb_autoset_output_filename->IsChecked())
+  if (options.autoset_output_filename)
     tc_output->SetValue(wxT(""));
 }
 
@@ -1829,13 +1871,12 @@ mmg_dialog::on_add_to_jobqueue(wxCommandEvent &evt) {
 
   if (!input_page->validate_settings() ||
       !attachments_page->validate_settings() ||
-      !global_page->validate_settings() ||
-      !settings_page->validate_settings())
+      !global_page->validate_settings())
     return;
 
   line = wxT("The output file '") + tc_output->GetValue() +
     wxT("' already exists. Do you want to overwrite it?");
-  if (settings_page->cb_ask_before_overwriting->IsChecked() &&
+  if (options.ask_before_overwriting &&
       wxFile::Exists(tc_output->GetValue()) &&
       (wxMessageBox(break_line(line, 60), wxT("Overwrite existing file?"),
                     wxYES_NO) != wxYES))
@@ -1851,7 +1892,7 @@ mmg_dialog::on_add_to_jobqueue(wxCommandEvent &evt) {
     if (description.length() == 0)
       return;
 
-    if (!settings_page->cb_ask_before_overwriting->IsChecked())
+    if (!options.ask_before_overwriting)
       break;
     line = wxT("A job with the description '") + description +
       wxT("' already exists. Do you really want to add another one "
@@ -1891,7 +1932,9 @@ mmg_dialog::on_add_to_jobqueue(wxCommandEvent &evt) {
 
   save_job_queue();
 
-  if (settings_page->cb_filenew_after_add_to_jobqueue->IsChecked()) {
+  remember_previous_output_directory();
+
+  if (options.filenew_after_add_to_jobqueue) {
     wxCommandEvent dummy;
     on_file_new(dummy);
   }
@@ -1997,6 +2040,77 @@ mmg_dialog::save_job_queue() {
   cfg->Flush();
 }
 
+void
+mmg_dialog::save_preferences() {
+  wxConfigBase *cfg = (wxConfig *)wxConfigBase::Get();
+  int x, y;
+
+  cfg->SetPath(wxU("/GUI"));
+
+  GetPosition(&x, &y);
+  cfg->Write(wxU("window_position_x"), x);
+  cfg->Write(wxU("window_position_y"), y);
+  cfg->Write(wxU("warned_chapter_editor_not_empty"), warned_chapter_editor_not_empty);
+  cfg->Write(wxU("previous_output_directory"), previous_output_directory);
+
+  cfg->Write(wxU("mkvmerge_executable"), options.mkvmerge);
+  cfg->Write(wxU("process_priority"), options.priority);
+  cfg->Write(wxU("autoset_output_filename"), options.autoset_output_filename);
+  cfg->Write(wxU("output_directory_mode"), (long)options.output_directory_mode);
+  cfg->Write(wxU("output_directory"), options.output_directory);
+  cfg->Write(wxU("ask_before_overwriting"), options.ask_before_overwriting);
+  cfg->Write(wxU("filenew_after_add_to_jobqueue"), options.filenew_after_add_to_jobqueue);
+  cfg->Write(wxU("on_top"), options.on_top);
+  cfg->Write(wxU("warn_usage"), options.warn_usage);
+  cfg->Write(wxU("gui_debugging"), options.gui_debugging);
+  cfg->Write(wxU("always_use_simpleblock"), options.always_use_simpleblock);
+  cfg->Write(wxU("set_delay_from_filename"), options.set_delay_from_filename);
+
+  cfg->Flush();
+}
+
+void
+mmg_dialog::load_preferences() {
+  wxConfigBase *cfg = wxConfigBase::Get();
+  wxString s;
+  int window_pos_x, window_pos_y;
+
+  cfg->SetPath(wxT("/GUI"));
+
+  if (cfg->Read(wxT("window_position_x"), &window_pos_x) &&
+      cfg->Read(wxT("window_position_y"), &window_pos_y) &&
+      (0 < window_pos_x) && (0xffff > window_pos_x) &&
+      (0 < window_pos_y) && (0xffff > window_pos_y))
+    Move(window_pos_x, window_pos_y);
+
+  cfg->Read(wxU("warned_chapterditor_not_empty"), &warned_chapter_editor_not_empty, false);
+  cfg->Read(wxU("previous_output_directory"), &previous_output_directory, wxU(""));
+
+  cfg->Read(wxU("mkvmerge_executable"), &options.mkvmerge, wxU("mkvmerge"));
+  cfg->Read(wxU("process_priority"), &options.priority, wxU("normal"));
+  cfg->Read(wxU("autoset_output_filename"), &options.autoset_output_filename, true);
+  cfg->Read(wxU("output_directory_mode"), (long *)&options.output_directory_mode, ODM_FROM_FIRST_INPUT_FILE);
+  cfg->Read(wxU("output_directory"), &options.output_directory, wxU(""));
+  cfg->Read(wxU("ask_before_overwriting"), &options.ask_before_overwriting, true);
+  cfg->Read(wxU("filenew_after_add_to_jobqueue"), &options.filenew_after_add_to_jobqueue, false);
+  cfg->Read(wxU("on_top"), &options.on_top, false);
+  cfg->Read(wxU("warn_usage"), &options.warn_usage, true);
+  cfg->Read(wxU("gui_debugging"), &options.gui_debugging, false);
+  cfg->Read(wxU("always_use_simpleblock"), &options.always_use_simpleblock, false);
+  cfg->Read(wxU("set_delay_from_filename"), &options.set_delay_from_filename, true);
+
+  options.validate();
+}
+
+void
+mmg_dialog::remember_previous_output_directory() {
+  wxFileName filename(tc_output->GetValue());
+  wxString path = filename.GetPath();
+
+  if (!path.IsEmpty())
+    previous_output_directory = path;
+}
+
 void
 mmg_dialog::on_add_cli_options(wxCommandEvent &evt) {
   cli_options_dlg dlg(this);
@@ -2007,18 +2121,7 @@ mmg_dialog::on_add_cli_options(wxCommandEvent &evt) {
 
 void
 mmg_dialog::on_close(wxCloseEvent &evt) {
-  int x, y;
-  wxConfigBase *cfg;
-
-  GetPosition(&x, &y);
-  cfg = wxConfigBase::Get();
-  cfg->SetPath(wxT("/GUI"));
-  cfg->Write(wxT("window_position_x"), x);
-  cfg->Write(wxT("window_position_y"), y);
-  cfg->Write(wxT("warned_chapter_editor_not_empty"),
-             warned_chapter_editor_not_empty);
-  cfg->Flush();
-
+  save_preferences();
   Destroy();
 }
 
@@ -2036,7 +2139,79 @@ mmg_dialog::set_on_top(bool on_top) {
 
 void
 mmg_dialog::restore_on_top() {
-  set_on_top(settings_page->cb_on_top->IsChecked());
+  set_on_top(options.on_top);
+}
+
+void
+mmg_dialog::on_file_options(wxCommandEvent &evt) {
+  options_dialog dlg(this, options);
+
+  if (dlg.ShowModal() != wxID_OK)
+    return;
+
+  log_window->Show(options.gui_debugging);
+  set_on_top(options.on_top);
+  query_mkvmerge_capabilities();
+}
+
+void
+mmg_dialog::query_mkvmerge_capabilities() {
+  wxString tmp;
+  wxArrayString output;
+  vector<wxString> parts;
+  int result, i;
+
+  wxLogMessage(wxT("Querying mkvmerge's capabilities"));
+  tmp = wxT("\"") + options.mkvmerge + wxT("\" --capabilities");
+#if defined(SYS_WINDOWS)
+  result = wxExecute(tmp, output);
+#else
+  // Workaround for buggy behaviour of some wxWindows/GTK combinations.
+  wxProcess *process;
+  wxInputStream *out;
+  int c;
+  string tmps;
+
+  process = new wxProcess(this, 1);
+  process->Redirect();
+  result = wxExecute(tmp, wxEXEC_ASYNC, process);
+  if (result == 0)
+    return;
+
+  out = process->GetInputStream();
+  tmps = "";
+  c = 0;
+  while (1) {
+    if (!out->Eof()) {
+      c = out->GetC();
+      if (c == '\n') {
+        output.Add(wxU(tmps.c_str()));
+        tmps = "";
+      } else if (c < 0)
+        break;
+      else if (c != '\r')
+        tmps += c;
+    } else
+      break;
+  }
+  if (tmps.length() > 0)
+    output.Add(wxU(tmps.c_str()));
+  result = 0;
+#endif
+
+  if (result == 0) {
+    capabilities.clear();
+    for (i = 0; i < output.Count(); i++) {
+      tmp = output[i];
+      strip(tmp);
+      wxLogMessage(wxT("Capability: %s"), tmp.c_str());
+      parts = split(tmp, wxU("="), 2);
+      if (parts.size() == 1)
+        capabilities[parts[0]] = wxT("true");
+      else
+        capabilities[parts[0]] = parts[1];
+    }
+  }
 }
 
 IMPLEMENT_CLASS(cli_options_dlg, wxDialog);
@@ -2045,6 +2220,10 @@ BEGIN_EVENT_TABLE(cli_options_dlg, wxDialog)
   EVT_BUTTON(ID_CLIOPTIONS_ADD, cli_options_dlg::on_add_clicked)
 END_EVENT_TABLE();
 
+IMPLEMENT_CLASS(show_text_dlg, wxDialog);
+BEGIN_EVENT_TABLE(show_text_dlg, wxDialog)
+END_EVENT_TABLE();
+
 IMPLEMENT_CLASS(mmg_dialog, wxFrame);
 BEGIN_EVENT_TABLE(mmg_dialog, wxFrame)
   EVT_BUTTON(ID_B_BROWSEOUTPUT, mmg_dialog::on_browse_output)
@@ -2057,8 +2236,10 @@ BEGIN_EVENT_TABLE(mmg_dialog, wxFrame)
   EVT_MENU(ID_M_FILE_NEW, mmg_dialog::on_file_new)
   EVT_MENU(ID_M_FILE_LOAD, mmg_dialog::on_file_load)
   EVT_MENU(ID_M_FILE_SAVE, mmg_dialog::on_file_save)
+  EVT_MENU(ID_M_FILE_OPTIONS, mmg_dialog::on_file_options)
   EVT_MENU(ID_M_FILE_SETOUTPUT, mmg_dialog::on_browse_output)
   EVT_MENU(ID_M_MUXING_START, mmg_dialog::on_run)
+  EVT_MENU(ID_M_MUXING_SHOW_CMDLINE, mmg_dialog::on_show_cmdline)
   EVT_MENU(ID_M_MUXING_COPY_CMDLINE, mmg_dialog::on_copy_to_clipboard)
   EVT_MENU(ID_M_MUXING_SAVE_CMDLINE, mmg_dialog::on_save_cmdline)
   EVT_MENU(ID_M_MUXING_CREATE_OPTIONFILE, mmg_dialog::on_create_optionfile)
@@ -2086,7 +2267,6 @@ BEGIN_EVENT_TABLE(mmg_dialog, wxFrame)
   EVT_MENU(ID_M_WINDOW_INPUT, mmg_dialog::on_window_selected)
   EVT_MENU(ID_M_WINDOW_ATTACHMENTS, mmg_dialog::on_window_selected)
   EVT_MENU(ID_M_WINDOW_GLOBAL, mmg_dialog::on_window_selected)
-  EVT_MENU(ID_M_WINDOW_SETTINGS, mmg_dialog::on_window_selected)
   EVT_MENU(ID_M_WINDOW_CHAPTEREDITOR, mmg_dialog::on_window_selected)
   EVT_CLOSE(mmg_dialog::on_close)
 END_EVENT_TABLE();
