@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mmg_dialog.h 2795 2005-02-19 15:42:31Z mosu $
+   $Id: mmg_dialog.h 3568 2007-07-08 17:30:57Z mosu $
 
    main dialog declarations
 
@@ -21,7 +21,6 @@
 #define ID_TC_OUTPUT                      10000
 #define ID_B_BROWSEOUTPUT                 10001
 #define ID_NOTEBOOK                       10002
-#define ID_TC_CMDLINE                     10003
 #define ID_T_UPDATECMDLINE                10004
 #define ID_T_STATUSBAR                    10005
 #define ID_B_STARTMUXING                  10006
@@ -33,6 +32,7 @@
 #define ID_M_FILE_SAVE                    60002
 #define ID_M_FILE_SETOUTPUT               60003
 #define ID_M_FILE_EXIT                    60004
+#define ID_M_FILE_OPTIONS                 60005
 #define ID_M_FILE_LOADSEPARATOR           60090
 #define ID_M_FILE_LOADLAST1               60091
 #define ID_M_FILE_LOADLAST2               60092
@@ -46,6 +46,7 @@
 #define ID_M_MUXING_ADD_TO_JOBQUEUE       60104
 #define ID_M_MUXING_MANAGE_JOBS           60105
 #define ID_M_MUXING_ADD_CLI_OPTIONS       60106
+#define ID_M_MUXING_SHOW_CMDLINE          60107
 
 #define ID_M_CHAPTERS_NEW                 60200
 #define ID_M_CHAPTERS_LOAD                60201
@@ -63,8 +64,7 @@
 #define ID_M_WINDOW_INPUT                 60300
 #define ID_M_WINDOW_ATTACHMENTS           60301
 #define ID_M_WINDOW_GLOBAL                60302
-#define ID_M_WINDOW_SETTINGS              60303
-#define ID_M_WINDOW_CHAPTEREDITOR         60304
+#define ID_M_WINDOW_CHAPTEREDITOR         60303
 
 #define ID_M_HELP_ABOUT                   65000
 #define ID_M_HELP_HELP                    65001
@@ -74,7 +74,6 @@ class tab_attachments;
 class tab_chapters;
 class tab_global;
 class tab_input;
-class tab_settings;
 class job_dialog;
 
 class mmg_dialog: public wxFrame {
@@ -82,13 +81,12 @@ class mmg_dialog: public wxFrame {
   DECLARE_EVENT_TABLE();
 public:
   wxButton *b_browse_output;
-  wxStaticBox *sb_commandline;
-  wxTextCtrl *tc_output, *tc_cmdline;
+  wxTextCtrl *tc_output;
+  wxString previous_output_directory;
 
   wxString cmdline, cli_options;
   wxArrayString clargs;
 
-  wxTimer cmdline_timer;
   wxTimer status_bar_timer;
 
   wxStatusBar *status_bar;
@@ -100,7 +98,6 @@ public:
   tab_input *input_page;
   tab_attachments *attachments_page;
   tab_global *global_page;
-  tab_settings *settings_page;
   tab_chapters *chapter_editor_page;
 
   wxButton *b_start_muxing, *b_copy_to_clipboard, *b_add_to_jobqueue;
@@ -111,9 +108,12 @@ public:
   job_dialog *jdlg;
 
   wxHtmlHelpController *help;
+  wxLogWindow *log_window;
 
   bool warned_chapter_editor_not_empty;
 
+  mmg_options_t options;
+
 public:
   mmg_dialog();
   virtual ~mmg_dialog();
@@ -121,6 +121,7 @@ public:
   void on_browse_output(wxCommandEvent &evt);
   void on_run(wxCommandEvent &evt);
   void on_save_cmdline(wxCommandEvent &evt);
+  void on_show_cmdline(wxCommandEvent &evt);
   void on_copy_to_clipboard(wxCommandEvent &evt);
   void on_create_optionfile(wxCommandEvent &evt);
 
@@ -128,6 +129,7 @@ public:
   void on_file_new(wxCommandEvent &evt);
   void on_file_load(wxCommandEvent &evt);
   void on_file_save(wxCommandEvent &evt);
+  void on_file_options(wxCommandEvent &evt);
   void on_help(wxCommandEvent &evt);
   void on_about(wxCommandEvent &evt);
 
@@ -162,6 +164,7 @@ public:
 
   void set_title_maybe(const wxString &new_title);
   void set_output_maybe(const wxString &new_output);
+  void remove_output_filename();
 
   void on_add_to_jobqueue(wxCommandEvent &evt);
   void on_manage_jobs(wxCommandEvent &evt);
@@ -177,6 +180,15 @@ public:
   void restore_on_top();
 
   void muxing_has_finished();
+
+  bool check_before_overwriting();
+
+  void load_preferences();
+  void save_preferences();
+
+  void remember_previous_output_directory();
+
+  void query_mkvmerge_capabilities();
 };
 
 extern mmg_dialog *mdlg;
