@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mux_dialog.cpp 2849 2005-03-13 17:47:27Z mosu $
+   $Id: mux_dialog.cpp 3559 2007-07-05 14:56:32Z mosu $
 
    muxing dialog
 
@@ -92,17 +92,17 @@ mux_dialog::mux_dialog(wxWindow *parent):
   siz_output->Add(tc_errors, 1, wxGROW | wxALL, 5);
 
   siz_buttons = new wxBoxSizer(wxHORIZONTAL);
-  siz_buttons->Add(0, 0, 1, wxGROW, 0);
+  siz_buttons->AddStretchSpacer();
   b_ok = new wxButton(this, ID_B_MUX_OK, wxT("Ok"));
   b_ok->Enable(false);
   siz_buttons->Add(b_ok);
-  siz_buttons->Add(0, 0, 1, wxGROW, 0);
+  siz_buttons->AddStretchSpacer();
   b_abort = new wxButton(this, ID_B_MUX_ABORT, wxT("Abort"));
   siz_buttons->Add(b_abort);
-  siz_buttons->Add(0, 0, 1, wxGROW, 0);
+  siz_buttons->AddStretchSpacer();
   b_save_log = new wxButton(this, ID_B_MUX_SAVELOG, wxT("Save log"));
   siz_buttons->Add(b_save_log);
-  siz_buttons->Add(0, 0, 1, wxGROW, 0);
+  siz_buttons->AddStretchSpacer();
 
   siz_all = new wxBoxSizer(wxVERTICAL);
   siz_all->Add(siz_status, 0, wxGROW | wxALL, 5);
@@ -162,7 +162,9 @@ mux_dialog::mux_dialog(wxWindow *parent):
 #else
       wx_line = from_utf8(cc_local_utf8, line).c_str();
 #endif
-      log += wx_line + wxT("\n");
+      log += wx_line;
+      if (c != '\r')
+        log += wxT("\n");
       if (wx_line.Find(wxT("Warning:")) == 0)
         tc_warnings->AppendText(wx_line + wxT("\n"));
       else if (wx_line.Find(wxT("Error:")) == 0)
@@ -218,6 +220,9 @@ mux_dialog::on_save_log(wxCommandEvent &evt) {
     last_open_dir = dlg.GetDirectory();
     file = new wxFile(dlg.GetPath(), wxFile::write);
     s = log + wxT("\n");
+#if defined(SYS_WINDOWS)
+    s.Replace(wxT("\n"), wxT("\r\n"));
+#endif
     file->Write(s);
     delete file;
   }
