@@ -1,22 +1,16 @@
 /*
-  mkvmerge GUI -- utility for splicing together matroska files
-      from component media subtypes
+   mkvmerge GUI -- utility for splicing together matroska files
+   from component media subtypes
 
-  mux_dialog.h
+   Distributed under the GPL
+   see the file COPYING for details
+   or visit http://www.gnu.org/copyleft/gpl.html
 
-  Written by Moritz Bunkus <moritz@bunkus.org>
-  Parts of this code were written by Florian Wager <root@sirelvis.de>
+   $Id: mux_dialog.h 2711 2005-01-12 15:03:47Z mosu $
 
-  Distributed under the GPL
-  see the file COPYING for details
-  or visit http://www.gnu.org/copyleft/gpl.html
-*/
+   muxing dialog
 
-/*!
-    \file
-    \version $Id: mux_dialog.h 1438 2004-01-24 09:48:21Z mosu $
-    \brief muxing dialog
-    \author Moritz Bunkus <moritz@bunkus.org>
+   Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
 #ifndef __MUX_DIALOG_H
@@ -29,6 +23,8 @@
 #define ID_B_MUX_SAVELOG                  17001
 #define ID_B_MUX_ABORT                    17002
 
+class mux_process;
+
 class mux_dialog: public wxDialog {
   DECLARE_CLASS(mux_dialog);
   DECLARE_EVENT_TABLE();
@@ -36,7 +32,7 @@ protected:
   long pid;
   wxStaticText *st_label;
   wxGauge *g_progress;
-  wxProcess *process;
+  mux_process *process;
   wxString log, opt_file_name;
   wxButton *b_ok, *b_save_log, *b_abort;
   wxTextCtrl *tc_output, *tc_warnings, *tc_errors;
@@ -51,6 +47,8 @@ public:
   void on_ok(wxCommandEvent &evt);
   void on_save_log(wxCommandEvent &evt);
   void on_abort(wxCommandEvent &evt);
+  void on_close(wxCloseEvent &evt);
+  void done();
 };
 
 class mux_process: public wxProcess {
