@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: tab_chapters.cpp 2985 2005-07-16 14:51:18Z mosu $
+   $Id: tab_chapters.cpp 3559 2007-07-05 14:56:32Z mosu $
 
    "chapter editor" tab
 
@@ -96,9 +96,8 @@ chapter_values_dlg::chapter_values_dlg(wxWindow *parent,
                                        wxString old_def_language,
                                        wxString old_def_country):
   wxDialog(parent, 0, wxT(""), wxDefaultPosition, wxSize(350, 200)) {
-  wxBoxSizer *siz_all, *siz_buttons, *siz_line;
+  wxBoxSizer *siz_all, *siz_buttons;
   wxFlexGridSizer *siz_input;
-  wxButton *b_ok, *b_cancel;
   uint32_t i;
 
   siz_all = new wxBoxSizer(wxVERTICAL);
@@ -115,9 +114,6 @@ chapter_values_dlg::chapter_values_dlg(wxWindow *parent,
 
     siz_input = new wxFlexGridSizer(2);
     siz_input->AddGrowableCol(1);
-    siz_input->Add(0, 1, 1, wxGROW, 0);
-    siz_input->Add(0, 1, 1, wxGROW, 0);
-    siz_input->AddGrowableRow(0);
 
     siz_input->Add(new wxStaticText(this, wxID_STATIC, wxT("Language:")),
                    0, wxALIGN_CENTER_VERTICAL | wxRIGHT, 10);
@@ -126,10 +122,6 @@ chapter_values_dlg::chapter_values_dlg(wxWindow *parent,
                                   wxCB_DROPDOWN | wxCB_READONLY);
     siz_input->Add(cob_language, 0, wxGROW, 0);
 
-    siz_input->Add(0, 1, 1, wxGROW, 0);
-    siz_input->Add(0, 1, 1, wxGROW, 0);
-    siz_input->AddGrowableRow(2);
-
     siz_input->Add(new wxStaticText(this, wxID_STATIC, wxT("Country:")),
                    0, wxALIGN_CENTER_VERTICAL | wxRIGHT, 10);
     cob_country = new wxComboBox(this, wxID_STATIC, wxT(""),
@@ -149,9 +141,6 @@ chapter_values_dlg::chapter_values_dlg(wxWindow *parent,
 
     siz_input = new wxFlexGridSizer(2);
     siz_input->AddGrowableCol(1);
-    siz_input->Add(0, 1, 1, wxGROW, 0);
-    siz_input->Add(0, 1, 1, wxGROW, 0);
-    siz_input->AddGrowableRow(0);
 
     cb_language =
       new wxCheckBox(this, ID_CVD_CB_LANGUAGE, wxT("Set language to:"));
@@ -163,10 +152,6 @@ chapter_values_dlg::chapter_values_dlg(wxWindow *parent,
     cob_language->Enable(false);
     siz_input->Add(cob_language, 0, wxGROW, 0);
 
-    siz_input->Add(0, 1, 1, wxGROW, 0);
-    siz_input->Add(0, 1, 1, wxGROW, 0);
-    siz_input->AddGrowableRow(2);
-
     cb_country =
       new wxCheckBox(this, ID_CVD_CB_COUNTRY, wxT("Set country to:"));
     cb_country->SetValue(false);
@@ -179,7 +164,9 @@ chapter_values_dlg::chapter_values_dlg(wxWindow *parent,
 
   }
 
-  siz_all->Add(siz_input, 3, wxGROW | wxLEFT | wxRIGHT, 25);
+  siz_all->AddSpacer(10);
+  siz_all->Add(siz_input, 0, wxGROW | wxLEFT | wxRIGHT, 10);
+  siz_all->AddSpacer(10);
 
   for (i = 0; i < sorted_iso_codes.Count(); i++) {
     cob_language->Append(sorted_iso_codes[i]);
@@ -192,21 +179,17 @@ chapter_values_dlg::chapter_values_dlg(wxWindow *parent,
     cob_country->Append(wxU(cctlds[i]));
   cob_country->SetValue(old_def_country);
 
-  siz_buttons = new wxBoxSizer(wxVERTICAL);
-  siz_buttons->Add(0, 1, 1, wxGROW, 0);
-  siz_line = new wxBoxSizer(wxHORIZONTAL);
-  b_ok = new wxButton(this, wxID_OK, wxT("Ok"));
-  b_ok->SetDefault();
-  b_cancel = new wxButton(this, wxID_CANCEL, wxT("Cancel"));
-  b_ok->SetSize(b_cancel->GetSize());
-  siz_line->Add(1, 0, 1, wxGROW, 0);
-  siz_line->Add(b_ok, 0, 0, 0);
-  siz_line->Add(1, 0, 1, wxGROW, 0);
-  siz_line->Add(b_cancel, 0, 0, 0);
-  siz_line->Add(1, 0, 1, wxGROW, 0);
-  siz_buttons->Add(siz_line, 0, wxGROW, 0);
-  siz_all->Add(siz_buttons, 2, wxGROW | wxBOTTOM, 10);
+  siz_buttons = new wxBoxSizer(wxHORIZONTAL);
+  siz_buttons->AddStretchSpacer();
+  siz_buttons->Add(CreateStdDialogButtonSizer(wxOK | wxCANCEL), 0, 0, 0);
+  siz_buttons->AddSpacer(10);
+
+  siz_all->Add(siz_buttons, 0, wxGROW, 0);
+  siz_all->AddSpacer(10);
+
   SetSizer(siz_all);
+
+  siz_all->SetSizeHints(this);
 }
 
 void
@@ -252,10 +235,10 @@ tab_chapters::tab_chapters(wxWindow *parent,
 
   siz_all = new wxBoxSizer(wxVERTICAL);
 
-  siz_all->Add(0, 5, 0, 0, 0);
+  siz_all->AddSpacer(5);
   siz_all->Add(new wxStaticText(this, wxID_STATIC, wxT("Chapters:")),
                0, wxLEFT, 10);
-  siz_all->Add(0, 5, 0, 0, 0);
+  siz_all->AddSpacer(5);
 
 #ifdef SYS_WINDOWS
   tc_chapters = new wxTreeCtrl(this, ID_TRC_CHAPTERS);
@@ -324,8 +307,8 @@ tab_chapters::tab_chapters(wxWindow *parent,
   siz_fg->Add(siz_line, 0, wxGROW | wxRIGHT, 5);
   siz_fg->Add(1, 0, 0, 0, 0);
 
-  siz_fg->Add(0, 5, 0, 0, 0);
-  siz_fg->Add(0, 5, 0, 0, 0);
+  siz_fg->AddSpacer(5);
+  siz_fg->AddSpacer(5);
 
   siz_line = new wxBoxSizer(wxHORIZONTAL);
   st_uid = new wxStaticText(this, wxID_STATIC, wxT("UID:"));
@@ -335,18 +318,18 @@ tab_chapters::tab_chapters(wxWindow *parent,
                           "identifier. This identifier is normally assigned "
                           "automatically by the programs, but it can be "
                           "changed manually if it is really needed."));
-  siz_line->Add(tc_uid, 1, wxGROW | wxRIGHT, 10);
+  siz_line->Add(tc_uid, 1, wxGROW | wxRIGHT | wxALIGN_CENTER_VERTICAL, 10);
   cb_flag_hidden = new wxCheckBox(this, ID_CB_CHAPTERHIDDEN, wxT("hidden"));
   cb_flag_hidden->SetToolTip(TIP("If a chapter is marked 'hidden' then the "
                                  "player should not show this chapter entry "
                                  "to the user. Such entries could still be "
                                  "used by the menu system."));
-  siz_line->Add(cb_flag_hidden, 0, wxRIGHT, 10);
+  siz_line->Add(cb_flag_hidden, 0, wxRIGHT | wxALIGN_CENTER_VERTICAL, 10);
   cb_flag_enabled = new wxCheckBox(this, ID_CB_CHAPTERENABLED, wxT("enabled"));
   cb_flag_enabled->SetToolTip(TIP("If a chapter is not marked 'enabled' then "
                                   "the player should skip the part of the "
                                   "file that this chapter occupies."));
-  siz_line->Add(cb_flag_enabled, 0, 0, 0);
+  siz_line->Add(cb_flag_enabled, 0, wxALIGN_CENTER_VERTICAL, 0);
 
   siz_fg->Add(siz_line, 0, wxGROW | wxRIGHT, 5);
   siz_fg->Add(1, 0, 0, 0, 0);
@@ -475,10 +458,10 @@ tab_chapters::enable_buttons(bool enable) {
 void
 tab_chapters::on_new_chapters(wxCommandEvent &evt) {
   file_name = wxT("");
-  if (chapters != NULL)
-    delete chapters;
   tc_chapters->DeleteAllItems();
   tid_root = tc_chapters->AddRoot(wxT("(new chapter file)"));
+  if (chapters != NULL)
+    delete chapters;
   chapters = new KaxChapters;
 
   m_chapters->Enable(ID_M_CHAPTERS_SAVE, true);
@@ -560,7 +543,7 @@ tab_chapters::add_recursively(wxTreeItemId &parent,
     e = master[i];
     if (EbmlId(*e) == KaxEditionEntry::ClassInfos.GlobalId) {
       s.Printf(wxT("Edition %d"),
-               tc_chapters->GetChildrenCount(tid_root, false) + 1);
+               (int)tc_chapters->GetChildrenCount(tid_root, false) + 1);
       eentry = static_cast<KaxEditionEntry *>(e);
       this_item =
         tc_chapters->AppendItem(parent, s, -1, -1,
@@ -628,6 +611,9 @@ tab_chapters::load(wxString name) {
   wxString s;
   int pos;
 
+  clear_list_of_unique_uint32(UNIQUE_CHAPTER_IDS);
+  clear_list_of_unique_uint32(UNIQUE_EDITION_IDS);
+
   try {
     if (kax_analyzer_c::probe(wxMB(name))) {
       if (analyzer != NULL)
@@ -662,6 +648,8 @@ tab_chapters::load(wxString name) {
       source_is_kax_file = false;
     }
   } catch (error_c ex) {
+    if (analyzer)
+      delete analyzer;
     analyzer = NULL;
     s = wxU(ex.get_error().c_str());
     break_line(s);
@@ -790,9 +778,6 @@ tab_chapters::save() {
 
   try {
     out = new mm_file_io_c(wxMB(file_name), MODE_CREATE);
-#if defined(SYS_WINDOWS)
-    out->use_dos_style_newlines(true);
-#endif
   } catch (...) {
     err.Printf(wxT("Could not open the destination file '%s' for writing."
                    " Error code: %d (%s)."), file_name.c_str(), errno,
@@ -802,6 +787,9 @@ tab_chapters::save() {
     return;
   }
 
+#if defined(SYS_WINDOWS)
+  out->use_dos_style_newlines(true);
+#endif
   out->write_bom("UTF-8");
   out->printf("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
               "\n"
@@ -959,7 +947,7 @@ tab_chapters::on_add_chapter(wxCommandEvent &evt) {
       create_unique_uint32(UNIQUE_EDITION_IDS);
     eentry->PushElement(*euid);
     d = new chapter_node_data_c(eentry);
-    s.Printf(wxT("EditionEntry %u"), chapters->ListSize());
+    s.Printf(wxT("EditionEntry %u"), (unsigned int)chapters->ListSize());
     id = tc_chapters->AppendItem(tid_root, s, -1, -1, d);
   }
 
@@ -1039,7 +1027,7 @@ tab_chapters::on_add_subchapter(wxCommandEvent &evt) {
     eentry = new KaxEditionEntry;
     chapters->PushElement(*eentry);
     d = new chapter_node_data_c(eentry);
-    s.Printf(wxT("EditionEntry %u"), chapters->ListSize());
+    s.Printf(wxT("EditionEntry %u"), (unsigned int)chapters->ListSize());
     id = tc_chapters->AppendItem(tid_root, s, -1, -1, d);
   }
   if (d->is_atom)
@@ -1343,6 +1331,7 @@ tab_chapters::on_chapter_name_changed(wxCommandEvent &evt) {
 void
 tab_chapters::on_set_default_values(wxCommandEvent &evt) {
   wxString language;
+  string cctld;
 
   chapter_values_dlg dlg(this, true, wxU(default_chapter_language.c_str()),
                          wxU(default_chapter_country.c_str()));
@@ -1359,7 +1348,15 @@ tab_chapters::on_set_default_values(wxCommandEvent &evt) {
     return;
   }
   default_chapter_language = wxMB(language);
-  default_chapter_country = wxMB(dlg.cob_country->GetValue());
+  cctld = wxMB(dlg.cob_country->GetValue());
+  if (!is_valid_cctld(cctld.c_str())) {
+    wxMessageBox(wxT("The country '") + dlg.cob_country->GetValue() +
+                 wxT("' is not a valid ccTLD and cannot be selected."),
+                 wxT("Invalid country selected"),
+                 wxICON_ERROR | wxOK);
+    return;
+  }
+  default_chapter_country = cctld;
 }
 
 void
@@ -1488,7 +1485,10 @@ tab_chapters::on_adjust_timecodes(wxCommandEvent &evt) {
                           "children by a fixed amount.\nThe amount can be "
                           "positive or negative. The format used can be\n"
                           "either just a number in which case it is inter"
-                          "preted as the number of seconds,\nor it can have "
+                          "preted as the number of seconds,\n"
+                          "it can be followed by the unit like 'ms' or 's' "
+                          "for milliseconds and seconds respectively,\n"
+                          "or it can have "
                           "the usual HH:MM:SS.mmm or HH:MM:SS format.\n"
                           "Example: -00:05:23 would let all the chpaters "
                           "begin\n5minutes and 23seconds earlier than now."),
@@ -1983,7 +1983,7 @@ tab_chapters::set_display_values(KaxChapterDisplay *display) {
   count = cob_language_code->GetCount();
   found = false;
   for (i = 0; i < count; i++)
-    if (cob_language_code->GetString(i) == language) {
+    if (extract_language_code(cob_language_code->GetString(i)) == language) {
       found = true;
       break;
     }
