@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: tab_input.cpp 3463 2007-01-12 16:02:16Z mosu $
+   $Id: tab_input.cpp 3562 2007-07-07 17:21:36Z mosu $
 
    "input" tab
 
@@ -35,7 +35,6 @@
 #include "mmg_dialog.h"
 #include "tab_input.h"
 #include "tab_global.h"
-#include "tab_settings.h"
 
 using namespace std;
 
@@ -108,12 +107,16 @@ tab_input::tab_input(wxWindow *parent):
                                wxDefaultPosition, wxSize(60, -1));
   b_append_file->Enable(false);
 
+  b_remove_all_files = new wxButton(this, ID_B_REMOVE_ALL_FILES, wxT("rem all"), wxDefaultPosition, wxSize(60, -1));
+  b_remove_all_files->Enable(false);
+
   siz_column->Add(b_add_file, 0, wxALL, STDSPACING);
   siz_column->Add(b_remove_file, 0, wxALL, STDSPACING);
   siz_line->Add(siz_column);
 
   siz_column = new wxBoxSizer(wxVERTICAL);
   siz_column->Add(b_append_file, 0, wxALL, STDSPACING);
+  siz_column->Add(b_remove_all_files, 0, wxALL, STDSPACING);
   siz_line->Add(siz_column);
 
   siz_all->Add(siz_line, 0, wxGROW | wxLEFT | wxRIGHT, LEFTRIGHTSPACING);
@@ -349,8 +352,7 @@ tab_input::add_file(const wxString &file_name,
   opt_file->Write(wxT("\n"));
   delete opt_file;
 
-  command = wxT("\"") + mkvmerge_path + wxT("\" \"@") + opt_file_name +
-    wxT("\"");
+  command = wxT("\"") + mdlg->options.mkvmerge + wxT("\" \"@") + opt_file_name + wxT("\"");
 
   wxLogMessage(wxT("identify 1: command: ``%s''"), command.c_str());
 
@@ -386,7 +388,7 @@ tab_input::add_file(const wxString &file_name,
   }
 
   wxString delay_from_file_name;
-  if (mdlg->settings_page->cb_set_delay_from_filename->IsChecked()) {
+  if (mdlg->options.set_delay_from_filename) {
     wxRegEx re_delay(wxT("delay[[:blank:]]+(-?[[:digit:]]+)"), wxRE_ICASE);
     if (re_delay.Matches(file_name))
       delay_from_file_name = re_delay.GetMatch(file_name, 1);
@@ -472,6 +474,9 @@ tab_input::add_file(const wxString &file_name,
             track->aac_is_sbr = track->aac_is_sbr_detected =
               pair[1] == wxT("true");
 
+          else if (pair[0] == wxT("packetizer"))
+            track->packetizer = pair[1];
+
         }
       }
 
@@ -486,7 +491,7 @@ tab_input::add_file(const wxString &file_name,
           ('v' == track->type) &&
           (track->ctype.Find(wxT("MPEG-4 part 10 ES")) >= 0) &&
           (!avc_es_fps_warning_shown ||
-           mdlg->settings_page->cb_warn_usage->GetValue())) {
+           mdlg->options.warn_usage)) {
         wxMessageBox(wxT("You're adding an AVC/h.264 elementary stream to "
                          "the output file. "
                          "mkvmerge cannot determine the number of frames "
@@ -660,6 +665,7 @@ tab_input::add_file(const wxString &file_name,
   st_tracks->Enable(true);
   clb_tracks->Enable(true);
   b_append_file->Enable(true);
+  b_remove_all_files->Enable(true);
 }
 
 void
@@ -720,6 +726,7 @@ tab_input::on_remove_file(wxCommandEvent &evt) {
   cb_no_attachments->Enable(-1 != selected_file);
   cb_no_tags->Enable(-1 != selected_file);
   b_remove_file->Enable(-1 != selected_file);
+  b_remove_all_files->Enable(-1 != selected_file);
   b_append_file->Enable(tracks.size() > 0);
   b_track_up->Enable(-1 != selected_file);
   b_track_down->Enable(-1 != selected_file);
@@ -735,6 +742,37 @@ tab_input::on_remove_file(wxCommandEvent &evt) {
   dont_copy_values_now = false;
 }
 
+void
+tab_input::on_remove_all_files(wxCommandEvent &evt) {
+  dont_copy_values_now = true;
+
+  lb_input_files->Clear();
+  clb_tracks->Clear();
+
+  tracks.clear();
+  files.clear();
+
+  selected_file  = -1;
+  selected_track = -1;
+  st_tracks->Enable(false);
+  clb_tracks->Enable(false);
+  st_file_options->Enable(false);
+  cb_no_chapters->Enable(false);
+  cb_no_attachments->Enable(false);
+  cb_no_tags->Enable(false);
+  b_remove_file->Enable(false);
+  b_remove_all_files->Enable(false);
+  b_append_file->Enable(false);
+  b_track_up->Enable(false);
+  b_track_down->Enable(false);
+
+  set_track_mode(NULL);
+
+  mdlg->remove_output_filename();
+
+  dont_copy_values_now = false;
+}
+
 void
 tab_input::on_move_track_up(wxCommandEvent &evt) {
   wxString s;
@@ -749,21 +787,24 @@ tab_input::on_move_track_up(wxCommandEvent &evt) {
 
   dont_copy_values_now = true;
 
-  t = tracks[selected_track - 1];
-  tracks[selected_track - 1] = tracks[selected_track];
-  tracks[selected_track] = t;
-  s = clb_tracks->GetString(selected_track - 1);
-  clb_tracks->SetString(selected_track - 1,
-                        clb_tracks->GetString(selected_track));
-  clb_tracks->SetString(selected_track, s);
-  clb_tracks->SetSelection(selected_track - 1);
-  clb_tracks->Check(selected_track - 1,
-                    tracks[selected_track - 1]->enabled);
-  clb_tracks->Check(selected_track, tracks[selected_track]->enabled);
-  selected_track--;
-  b_track_up->Enable(selected_track > 0);
+  int current_track = selected_track;
+
+  t = tracks[current_track - 1];
+  tracks[current_track - 1] = tracks[current_track];
+  tracks[current_track] = t;
+  s = clb_tracks->GetString(current_track - 1);
+  clb_tracks->SetString(current_track - 1,
+                        clb_tracks->GetString(current_track));
+  clb_tracks->SetString(current_track, s);
+  clb_tracks->SetSelection(current_track - 1);
+  clb_tracks->Check(current_track - 1,
+                    tracks[current_track - 1]->enabled);
+  clb_tracks->Check(current_track, tracks[current_track]->enabled);
+  current_track--;
+  b_track_up->Enable(current_track > 0);
   b_track_down->Enable(true);
 
+  selected_track       = current_track;
   dont_copy_values_now = false;
 }
 
@@ -781,21 +822,24 @@ tab_input::on_move_track_down(wxCommandEvent &evt) {
 
   dont_copy_values_now = true;
 
-  t = tracks[selected_track + 1];
-  tracks[selected_track + 1] = tracks[selected_track];
-  tracks[selected_track] = t;
-  s = clb_tracks->GetString(selected_track + 1);
-  clb_tracks->SetString(selected_track + 1,
-                        clb_tracks->GetString(selected_track));
-  clb_tracks->SetString(selected_track, s);
-  clb_tracks->SetSelection(selected_track + 1);
-  clb_tracks->Check(selected_track + 1,
-                    tracks[selected_track + 1]->enabled);
-  clb_tracks->Check(selected_track, tracks[selected_track]->enabled);
-  selected_track++;
-  b_track_up->Enable(true);
-  b_track_down->Enable(selected_track < (tracks.size() - 1));
+  int current_track = selected_track;
 
+  t = tracks[current_track + 1];
+  tracks[current_track + 1] = tracks[current_track];
+  tracks[current_track] = t;
+  s = clb_tracks->GetString(current_track + 1);
+  clb_tracks->SetString(current_track + 1,
+                        clb_tracks->GetString(current_track));
+  clb_tracks->SetString(current_track, s);
+  clb_tracks->SetSelection(current_track + 1);
+  clb_tracks->Check(current_track + 1,
+                    tracks[current_track + 1]->enabled);
+  clb_tracks->Check(current_track, tracks[current_track]->enabled);
+  current_track++;
+  b_track_up->Enable(true);
+  b_track_down->Enable(current_track < (tracks.size() - 1));
+
+  selected_track       = current_track;
   dont_copy_values_now = false;
 }
 
@@ -865,6 +909,7 @@ tab_input::on_track_selected(wxCommandEvent &evt) {
 
   selected_track = -1;
   new_sel = clb_tracks->GetSelection();
+
   if (0 > new_sel)
     return;
 
@@ -889,7 +934,7 @@ tab_input::on_track_selected(wxCommandEvent &evt) {
   ti_format->tc_stretch->SetValue(t->stretch);
   ti_format->cob_sub_charset->SetValue(t->sub_charset);
   ti_general->tc_tags->SetValue(t->tags);
-  ti_general->cb_default->SetValue(t->default_track);
+  ti_general->cob_default->SetSelection(t->default_track);
   ti_format->cb_aac_is_sbr->SetValue(t->aac_is_sbr);
   set_combobox_selection(ti_format->cob_aspect_ratio, t->aspect_ratio);
   ti_format->tc_display_width->SetValue(t->dwidth);
@@ -899,7 +944,7 @@ tab_input::on_track_selected(wxCommandEvent &evt) {
   ti_general->tc_timecodes->SetValue(t->timecodes);
   set_combobox_selection(ti_format->cob_fourcc, t->fourcc);
   set_combobox_selection(ti_format->cob_fps, t->fps);
-  ti_format->cob_nalu_size_length->SetSelection(t->nalu_size_length - 2);
+  ti_format->cob_nalu_size_length->SetSelection(t->nalu_size_length / 2);
   ti_general->tc_track_name->SetFocus();
   ti_format->cob_stereo_mode->SetSelection(t->stereo_mode);
   ti_extra->tc_user_defined->SetValue(t->user_defined);
@@ -976,7 +1021,7 @@ tab_input::save(wxConfigBase *cfg) {
       cfg->Write(wxT("id"), s);
       cfg->Write(wxT("enabled"), t->enabled);
       cfg->Write(wxT("content_type"), t->ctype);
-      cfg->Write(wxT("default_track"), t->default_track);
+      cfg->Write(wxT("default_track_2"), t->default_track);
       cfg->Write(wxT("aac_is_sbr"), t->aac_is_sbr);
       cfg->Write(wxT("language"), t->language);
       cfg->Write(wxT("track_name"), t->track_name);
@@ -999,6 +1044,7 @@ tab_input::save(wxConfigBase *cfg) {
       cfg->Write(wxT("track_name_was_present"), t->track_name_was_present);
       cfg->Write(wxT("appending"), t->appending);
       cfg->Write(wxT("user_defined"), t->user_defined);
+      cfg->Write(wxT("packetizer"), t->packetizer);
 
       cfg->SetPath(wxT(".."));
     }
@@ -1025,6 +1071,7 @@ tab_input::load(wxConfigBase *cfg,
   selected_file = -1;
   selected_track = -1;
   b_remove_file->Enable(false);
+  b_remove_all_files->Enable(false);
   b_append_file->Enable(false);
 
   files.clear();
@@ -1059,6 +1106,7 @@ tab_input::load(wxConfigBase *cfg,
 
     for (tidx = 0; tidx < (uint32_t)num_tracks; tidx++) {
       mmg_track_ptr tr(new mmg_track_t);
+      bool dummy = false;
 
       s.Printf(wxT("track %ld"), tidx);
       cfg->SetPath(s);
@@ -1075,7 +1123,10 @@ tab_input::load(wxConfigBase *cfg,
       }
       cfg->Read(wxT("enabled"), &tr->enabled);
       cfg->Read(wxT("content_type"), &tr->ctype);
-      cfg->Read(wxT("default_track"), &tr->default_track, false);
+      if (cfg->Read(wxT("default_track"), &dummy)) {
+        tr->default_track = dummy ? 1 : 0;
+      } else
+        cfg->Read(wxT("default_track_2"), &tr->default_track, 0);
       cfg->Read(wxT("aac_is_sbr"), &tr->aac_is_sbr, false);
       cfg->Read(wxT("language"), &tr->language);
       cfg->Read(wxT("track_name"), &tr->track_name);
@@ -1091,7 +1142,7 @@ tab_input::load(wxConfigBase *cfg,
       cfg->Read(wxT("display_height"), &tr->dheight);
       cfg->Read(wxT("fourcc"), &tr->fourcc);
       cfg->Read(wxT("fps"), &tr->fps);
-      cfg->Read(wxT("nalu_size_length"), &tr->nalu_size_length, 2);
+      cfg->Read(wxT("nalu_size_length"), &tr->nalu_size_length, 4);
       cfg->Read(wxT("stereo_mode"), &tr->stereo_mode, 0);
       cfg->Read(wxT("compression"), &tr->compression);
       cfg->Read(wxT("timecodes"), &tr->timecodes);
@@ -1099,6 +1150,7 @@ tab_input::load(wxConfigBase *cfg,
                 false);
       cfg->Read(wxT("appending"), &tr->appending, false);
       cfg->Read(wxT("user_defined"), &tr->user_defined);
+      cfg->Read(wxT("packetizer"), &tr->packetizer);
       tr->source = files.size();
       if (track_order.Length() > 0)
         track_order += wxT(",");
@@ -1178,6 +1230,7 @@ tab_input::load(wxConfigBase *cfg,
   st_tracks->Enable(tracks.size() > 0);
   clb_tracks->Enable(tracks.size() > 0);
   b_append_file->Enable(files.size() > 0);
+  b_remove_all_files->Enable(files.size() > 0);
 
   dont_copy_values_now = false;
 }
@@ -1270,10 +1323,10 @@ tab_input::validate_settings() {
       } else if ((FILE_TYPE_AVC_ES == f->container) &&
                  ('v' == t->type) &&
                  (t->ctype.Find(wxT("MPEG-4 part 10 ES")) >= 0) &&
-                 mdlg->settings_page->cb_warn_usage->GetValue()) {
+                 mdlg->options.warn_usage) {
         wxString message =
-          wxString::Format(wxT("You haven't selected a number of frames for "
-                               "track " LLD " of file '%s'. "
+          wxString::Format(wxT("You haven't selected a number of frames "
+                               "per second for track " LLD " of file '%s'. "
                                "mkvmerge cannot determine the number of "
                                "frames per second for such files itself. "
                                "Therefore you have to set this parameter "
@@ -1349,6 +1402,7 @@ IMPLEMENT_CLASS(tab_input, wxPanel);
 BEGIN_EVENT_TABLE(tab_input, wxPanel)
   EVT_BUTTON(ID_B_ADDFILE, tab_input::on_add_file)
   EVT_BUTTON(ID_B_REMOVEFILE, tab_input::on_remove_file)
+  EVT_BUTTON(ID_B_REMOVE_ALL_FILES, tab_input::on_remove_all_files)
   EVT_BUTTON(ID_B_APPENDFILE, tab_input::on_append_file)
   EVT_BUTTON(ID_B_TRACKUP, tab_input::on_move_track_up)
   EVT_BUTTON(ID_B_TRACKDOWN, tab_input::on_move_track_down)
