@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: tab_input_format.cpp 3423 2007-01-04 15:11:53Z mosu $
+   $Id: tab_input_format.cpp 3547 2007-06-30 17:50:41Z mosu $
 
    "input" tab -- "Format specific track options" sub-tab
 
@@ -118,7 +118,7 @@ tab_input_format::tab_input_format(wxWindow *parent,
   siz_fg->Add(cob_fourcc, 1, wxGROW | wxALIGN_CENTER_VERTICAL | wxALL,
               STDSPACING);
 
-  st_stereo_mode = new wxStaticText(this, -1, wxT("Stereo mode:"));
+  st_stereo_mode = new wxStaticText(this, -1, wxT("Stereoscopy:"));
   st_stereo_mode->Enable(false);
   siz_fg->Add(st_stereo_mode, 0, wxALIGN_CENTER_VERTICAL | wxALL, STDSPACING);
 
@@ -165,11 +165,10 @@ tab_input_format::tab_input_format(wxWindow *parent,
                    "This parameter is only available for AVC/h.264 elementary "
                    "streams read from AVC/h.264 ES files, AVIs or Matroska "
                    "files created with '--engage allow_avc_in_vwf_mode'. "
-                   "It defaults to 2 bytes, but there are files that contain "
-                   "frames or slices that are bigger than 65535 bytes. "
-                   "For such files you have to use this parameter and "
-                   "increase the size to 3 or mkvmerge will abort with an "
-                   "error."));
+                   "It defaults to 4 bytes, but there are files which do not "
+                   "contain a frame or slice that is bigger than 65535 bytes. "
+                   "For such files you can use this parameter and decrease "
+                   "the size to 2."));
   cob_nalu_size_length->SetSizeHints(0, -1);
   siz_fg->Add(cob_nalu_size_length, 1, wxGROW | wxALIGN_CENTER_VERTICAL | wxALL,
               STDSPACING);
@@ -277,9 +276,9 @@ tab_input_format::setup_control_contents() {
   cob_fps->Append(wxT("30000/1001"));
   cob_fps->Append(wxT("24000/1001"));
 
-  cob_nalu_size_length->Append(wxT("2"));
-  cob_nalu_size_length->Append(wxT("3"));
-  cob_nalu_size_length->Append(wxT("4"));
+  cob_nalu_size_length->Append(wxT("Default"));
+  cob_nalu_size_length->Append(wxT("2 bytes"));
+  cob_nalu_size_length->Append(wxT("4 bytes"));
 
   if (sorted_charsets.Count() == 0) {
     for (i = 0; sub_charsets[i] != NULL; i++)
@@ -307,8 +306,8 @@ tab_input_format::set_track_mode(mmg_track_t *t) {
   bool video = ('v' == type) && !appending;
   bool audio_app = ('a' == type);
   bool subs_app = ('s' == type);
-  bool avc_es = video && (ctype.Find(wxT("MPEG-4 part 10 ES")) >= 0) &&
-    (FILE_TYPE_AVC_ES == files[t->source].container);
+  bool avc_es = video && (t->packetizer == wxT("mpeg4_p10_es_video"));
+  bool avc    = video && (t->packetizer == wxT("mpeg4_p10_video"));
 
   ctype = ctype.Lower();
 
@@ -324,8 +323,8 @@ tab_input_format::set_track_mode(mmg_track_t *t) {
   cob_stereo_mode->Enable(video);
   st_fps->Enable(avc_es);
   cob_fps->Enable(avc_es);
-  st_nalu_size_length->Enable(avc_es);
-  cob_nalu_size_length->Enable(avc_es);
+  st_nalu_size_length->Enable(avc || avc_es);
+  cob_nalu_size_length->Enable(avc || avc_es);
   st_compression->Enable((ctype.Find(wxT("vobsub")) >= 0) && !appending);
   cob_compression->Enable((ctype.Find(wxT("vobsub")) >= 0) && !appending);
 
@@ -430,7 +429,7 @@ tab_input_format::on_nalu_size_length_changed(wxCommandEvent &evt) {
     return;
 
   tracks[input->selected_track]->nalu_size_length =
-    cob_nalu_size_length->GetSelection() + 2;
+    cob_nalu_size_length->GetSelection() * 2;
 }
 
 void
