@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_ac3.cpp 3024 2005-08-17 11:46:09Z mosu $
+   $Id: p_ac3.cpp 3503 2007-03-02 11:17:25Z mosu $
 
    AC3 output module
 
@@ -18,10 +18,11 @@
 #include <string.h>
 #include <errno.h>
 
-#include "pr_generic.h"
 #include "ac3_common.h"
-#include "p_ac3.h"
 #include "matroska.h"
+#include "output_control.h"
+#include "pr_generic.h"
+#include "p_ac3.h"
 
 using namespace libmatroska;
 
@@ -32,8 +33,9 @@ ac3_packetizer_c::ac3_packetizer_c(generic_reader_c *_reader,
                                    track_info_c &_ti)
   throw (error_c):
   generic_packetizer_c(_reader, _ti),
-  bytes_output(0), packetno(0),
-  samples_per_sec(_samples_per_sec), channels(_channels), bsid(_bsid) {
+  bytes_output(0), packetno(0), bytes_skipped(0),
+  samples_per_sec(_samples_per_sec), channels(_channels), bsid(_bsid),
+  first_packet(true) {
 
   set_track_type(track_audio);
   set_track_default_duration((int64_t)(1536000000000.0 *ti.async.linear /
@@ -63,39 +65,47 @@ ac3_packetizer_c::get_ac3_packet(unsigned long *header,
   if (packet_buffer == NULL)
     return NULL;
   pos = find_ac3_header(packet_buffer, size, ac3header);
-  if (pos < 0)
+  if (pos < 0) {
+    if (7 < size) {
+      bytes_skipped += size - 7;
+      byte_buffer.remove(size - 7);
+    }
     return NULL;
+  }
   if ((pos + ac3header->bytes) > size)
     return NULL;
 
-  if (pos > 0) {
+  bytes_skipped += pos;
+  if (0 < bytes_skipped) {
     bool warning_printed;
 
     warning_printed = false;
     if (packetno == 0) {
       int64_t offset;
 
-      offset = handle_avi_audio_sync(pos, false);
+      offset = handle_avi_audio_sync(bytes_skipped, false);
       if (offset != -1) {
-        mxinfo("The AC3 track %lld from '%s' contained %d bytes of non-AC3 "
-               "data at the beginning. This corresponds to a delay of %lldms. "
-               "This delay will be used instead of the non-AC3 data.\n",
-               ti.id, ti.fname.c_str(), pos, offset / 1000000);
+        mxinfo("The AC3 track " LLD " from '%s' contained " LLD " bytes of "
+               "non-AC3 data at the beginning. This corresponds to a delay "
+               "of " LLD "ms. This delay will be used instead of the non-AC3 "
+               "data.\n",
+               ti.id, ti.fname.c_str(), bytes_skipped, offset / 1000000);
         warning_printed = true;
       }
     }
     if (!warning_printed)
-      mxwarn("The AC3 track %lld from '%s' contained %d bytes of non-AC3 data "
-             "at the beginning which were skipped. The audio/video "
-             "synchronization may have been lost.\n", ti.id,
-             ti.fname.c_str(), pos);
+      mxwarn("The AC3 track " LLD " from '%s' contained " LLD " bytes of "
+             "non-AC3 data which were skipped. The audio/"
+             "video synchronization may have been lost.\n", ti.id,
+             ti.fname.c_str(), bytes_skipped);
     byte_buffer.remove(pos);
     packet_buffer = byte_buffer.get_buffer();
     size = byte_buffer.get_size();
     pos = 0;
+    bytes_skipped = 0;
   }
 
-  pins = 1536000000000.0 / samples_per_sec;
+  pins = ac3header->samples * 1000000000.0 / samples_per_sec;
 
   if (needs_negative_displacement(pins)) {
     /*
@@ -135,6 +145,8 @@ ac3_packetizer_c::set_headers() {
     id += "/BSID9";
   else if (bsid == 10)
     id += "/BSID10";
+  else if (bsid == 16)
+    id = MKV_A_EAC3;
   set_codec_id(id.c_str());
   set_audio_sampling_freq((float)samples_per_sec);
   set_audio_channels(channels);
@@ -151,17 +163,19 @@ ac3_packetizer_c::process(packet_cptr packet) {
 
   debug_enter("ac3_packetizer_c::process");
 
-  add_to_buffer(packet->memory->data, packet->memory->size);
+  add_to_buffer(packet->data->get(), packet->data->get_size());
   while ((ac3_packet = get_ac3_packet(&header, &ac3header)) != NULL) {
+    adjust_header_values(ac3header);
+
     if (packet->timecode == -1)
-      my_timecode = (int64_t)(1000000000.0 * packetno * 1536 /
+      my_timecode = (int64_t)(1000000000.0 * packetno * ac3header.samples /
                               samples_per_sec);
     else
       my_timecode = packet->timecode + ti.async.displacement;
     add_packet(new packet_t(new memory_c(ac3_packet, ac3header.bytes, true),
                             (int64_t)(my_timecode * ti.async.linear),
-                            (int64_t)(1000000000.0 * 1536 * ti.async.linear /
-                                      samples_per_sec)));
+                            (int64_t)(1000000000.0 * ac3header.samples *
+                                      ti.async.linear / samples_per_sec)));
     packetno++;
   }
 
@@ -170,6 +184,24 @@ ac3_packetizer_c::process(packet_cptr packet) {
   return FILE_STATUS_MOREDATA;
 }
 
+void
+ac3_packetizer_c::adjust_header_values(ac3_header_t &ac3header) {
+  if (!first_packet)
+    return;
+
+  first_packet = false;
+
+  bsid = ac3header.bsid;
+  if (16 == bsid)
+    set_codec_id(MKV_A_EAC3);
+
+  if (1536 != ac3header.samples)
+    set_track_default_duration((int64_t)(1000000000.0 * ac3header.samples *
+                                         ti.async.linear / samples_per_sec));
+
+  rerender_track_headers();
+}
+
 void
 ac3_packetizer_c::dump_debug_info() {
   mxdebug("ac3_packetizer_c: queue: %u; buffer size: %d\n",
