@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_avc.cpp 3437 2007-01-07 21:36:05Z mosu $
+   $Id: p_avc.cpp 3475 2007-01-16 19:37:34Z mosu $
 
    MPEG 4 part 10 ES video output module
 
@@ -58,6 +58,7 @@ mpeg4_p10_es_video_packetizer_c(generic_reader_c *n_reader,
 
   set_codec_private(m_avcc->get(), m_avcc->get_size());
   extract_aspect_ratio();
+  m_parser.set_keep_ar_info(false);
 }
 
 void
@@ -134,7 +135,7 @@ mpeg4_p10_es_video_packetizer_c::extract_aspect_ratio() {
     }
   }
 
-  if (priv != ti.private_data)
+  if (priv != hcodec_private)
     safefree(priv);
 }
 
