@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_dts.cpp 3024 2005-08-17 11:46:09Z mosu $
+   $Id: p_dts.cpp 3543 2007-06-30 15:27:47Z mosu $
 
    DTS output module
 
@@ -130,10 +130,19 @@ dts_packetizer_c::get_dts_packet(dts_header_t &dtsheader) {
     return get_dts_packet(dtsheader);
   }
 
-  if (verbose && (pos > 0) && !skipping_is_normal)
-    mxwarn("dts_packetizer: skipping %d bytes (no valid DTS header "
-           "found). This might make audio/video go out of sync, but this "
-           "stream is damaged.\n", pos);
+  if (verbose && (0 < pos) && !skipping_is_normal) {
+    int i;
+    bool all_zeroes = true;
+
+    for (i = 0; i < pos; ++i)
+      if (packet_buffer[i]) {
+        all_zeroes = false;
+        break;
+      }
+
+    if (!all_zeroes)
+      mxwarn("dts_packetizer: skipping %d bytes (no valid DTS header found). This might make audio/video go out of sync, but this stream is damaged.\n", pos);
+  }
 
   buf = (unsigned char *)safememdup(packet_buffer + pos,
                                     dtsheader.frame_byte_size);
@@ -176,7 +185,7 @@ dts_packetizer_c::process(packet_cptr packet) {
 
   debug_enter("dts_packetizer_c::process");
 
-  add_to_buffer(packet->memory->data, packet->memory->size);
+  add_to_buffer(packet->data->get(), packet->data->get_size());
   while ((dts_packet = get_dts_packet(dtsheader)) != NULL) {
     int64_t packet_len_in_ns =
       (int64_t)get_dts_packet_length_in_nanoseconds(&dtsheader);
@@ -214,7 +223,7 @@ dts_packetizer_c::can_connect_to(generic_packetizer_c *src,
   if (dsrc == NULL)
     return CAN_CONNECT_NO_FORMAT;
   if (get_first_header_later)
-    return CAN_CONNECT_NO_FORMAT;
+    return CAN_CONNECT_MAYBE_CODECPRIVATE;
   connect_check_a_samplerate(first_header.core_sampling_frequency,
                              dsrc->first_header.core_sampling_frequency);
   connect_check_a_channels(first_header.audio_channels,
