@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_passthrough.cpp 3024 2005-08-17 11:46:09Z mosu $
+   $Id: p_passthrough.cpp 3535 2007-06-24 08:16:04Z mosu $
 
    PASSTHROUGH output module
 
@@ -47,7 +47,7 @@ passthrough_packetizer_c::process(packet_cptr packet) {
 
   timecode = packet->timecode;
   packets_processed++;
-  bytes_processed += packet->memory->size;
+  bytes_processed += packet->data->get_size();
   if ((packet->duration > 0) &&
       needs_negative_displacement(packet->duration)) {
     displace(-packet->duration);
@@ -62,7 +62,7 @@ passthrough_packetizer_c::process(packet_cptr packet) {
   while ((packet->duration > 0) &&
          needs_positive_displacement(packet->duration)) {
     packet_t *new_packet =
-      new packet_t(packet->memory->clone(),
+      new packet_t(packet->data->clone(),
                    (int64_t)((timecode + ti.async.displacement) *
                              ti.async.linear),
                    packet->duration, packet->bref, packet->fref);
@@ -96,12 +96,14 @@ passthrough_packetizer_c::always_sync_complete_group(bool sync) {
 
 void
 passthrough_packetizer_c::dump_debug_info() {
-  mxdebug("passthrough_packetizer_c: packets processed: %lld, "
-          "bytes processed: %lld, packets in queue: %u\n",
+  mxdebug("passthrough_packetizer_c: packets processed: " LLD ", "
+          "bytes processed: " LLD ", packets in queue: %u\n",
           packets_processed, bytes_processed,
           (unsigned int)packet_queue.size());
 }
 
+#define CMP(member) (member != psrc->member)
+
 connection_result_e
 passthrough_packetizer_c::can_connect_to(generic_packetizer_c *src,
                                          string &error_message) {
@@ -110,6 +112,48 @@ passthrough_packetizer_c::can_connect_to(generic_packetizer_c *src,
   psrc = dynamic_cast<passthrough_packetizer_c *>(src);
   if (psrc == NULL)
     return CAN_CONNECT_NO_FORMAT;
+
   connect_check_codec_id(hcodec_id, psrc->hcodec_id);
-  return CAN_CONNECT_NO_PARAMETERS;
+
+  if (CMP(htrack_type) || CMP(hcodec_id))
+    return CAN_CONNECT_NO_PARAMETERS;
+
+  if (((ti.private_data == NULL) && (psrc->ti.private_data != NULL))
+      || ((ti.private_data != NULL) && (psrc->ti.private_data == NULL))
+      || (ti.private_size != psrc->ti.private_size)
+      || ((ti.private_data != NULL) && (psrc->ti.private_data != NULL)
+          && (ti.private_size == psrc->ti.private_size)
+          && memcmp(ti.private_data, psrc->ti.private_data, ti.private_size))) {
+    error_message = mxsprintf("The codec's private data does not match "
+                              "(lengths: %d and %d).", ti.private_size,
+                              psrc->ti.private_size);
+    return CAN_CONNECT_MAYBE_CODECPRIVATE;
+  }
+
+  switch (htrack_type) {
+    case track_video:
+      connect_check_v_width(hvideo_pixel_width, psrc->hvideo_pixel_width);
+      connect_check_v_height(hvideo_pixel_height, psrc->hvideo_pixel_height);
+      connect_check_v_dwidth(hvideo_display_width, psrc->hvideo_display_width);
+      connect_check_v_dheight(hvideo_display_height, psrc->hvideo_display_height);
+      if (CMP(htrack_min_cache) || CMP(htrack_max_cache) || CMP(htrack_default_duration))
+        return CAN_CONNECT_NO_PARAMETERS;
+      break;
+
+    case track_audio:
+      connect_check_a_samplerate(haudio_sampling_freq, psrc->haudio_sampling_freq);
+      connect_check_a_channels(haudio_channels, psrc->haudio_channels);
+      connect_check_a_bitdepth(haudio_bit_depth, psrc->haudio_bit_depth);
+      if (CMP(htrack_default_duration))
+        return CAN_CONNECT_NO_PARAMETERS;
+      break;
+
+    case track_subtitle:
+      break;
+
+    default:
+      break;
+  }
+
+  return CAN_CONNECT_YES;
 }
