@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_video.cpp 3458 2007-01-11 19:01:55Z mosu $
+   $Id: p_video.cpp 3536 2007-06-25 10:39:26Z mosu $
 
    video output module
 
@@ -183,6 +183,62 @@ mpeg1_2_video_packetizer_c(generic_reader_c *_reader,
     aspect_ratio_extracted = true;
 
   timecode_factory_application_mode = TFA_SHORT_QUEUEING;
+  if (hack_engaged(ENGAGE_USE_CODEC_STATE))
+      parser.SeparateSequenceHeaders();
+}
+
+int
+mpeg1_2_video_packetizer_c::process_framed(packet_cptr packet) {
+  if (0 == packet->data->get_size())
+    return FILE_STATUS_MOREDATA;
+
+  if (!hack_engaged(ENGAGE_USE_CODEC_STATE) ||
+      (4 > packet->data->get_size()))
+    return video_packetizer_c::process(packet);
+
+  unsigned char *buf = packet->data->get();
+  int pos = 4, marker, size = packet->data->get_size();
+
+  marker = get_uint32_be(buf);
+  while ((pos < size) && (MPEGVIDEO_SEQUENCE_START_CODE != marker)) {
+    marker <<= 8;
+    marker |= buf[pos];
+    ++pos;
+  }
+
+  if ((MPEGVIDEO_SEQUENCE_START_CODE != marker) ||
+      ((pos + 4) >= size))
+    return video_packetizer_c::process(packet);
+
+  int start = pos - 4;
+  marker = get_uint32_be(&buf[pos]);
+  pos += 4;
+
+  while ((pos < size) &&
+         ((MPEGVIDEO_EXT_START_CODE == marker) ||
+          (0x00000100 != (marker & 0xffffff00)))) {
+    marker <<= 8;
+    marker |= buf[pos];
+    ++pos;
+  }
+
+  if (pos >= size)
+    return video_packetizer_c::process(packet);
+
+  pos -= 4;
+  int sh_size = pos - start;
+
+  if ((NULL == seq_hdr.get()) ||
+      (sh_size != seq_hdr->get_size()) ||
+      memcmp(&buf[start], seq_hdr->get(), sh_size)) {
+    seq_hdr = clone_memory(&buf[start], sh_size);
+    packet->codec_state = clone_memory(&buf[start], sh_size);
+  }
+
+  memmove(&buf[start], &buf[pos], size - pos);
+  packet->data->set_size(size - sh_size);
+
+  return video_packetizer_c::process(packet);
 }
 
 int
@@ -196,12 +252,8 @@ mpeg1_2_video_packetizer_c::process(packet_cptr packet) {
   if (!aspect_ratio_extracted)
     extract_aspect_ratio(packet->data->get(), packet->data->get_size());
 
-  if (framed) {
-    if (0 != packet->data->get_size())
-      return video_packetizer_c::process(packet);
-    else
-      return FILE_STATUS_MOREDATA;
-  }
+  if (framed)
+    return process_framed(packet);
 
   state = parser.GetState();
   if ((state == MPV_PARSER_STATE_EOS) ||
@@ -231,15 +283,23 @@ mpeg1_2_video_packetizer_c::process(packet_cptr packet) {
       if (frame == NULL)
         break;
 
-//       if (hcodec_private == NULL)
-//         create_private_data();
-
       packet_t *new_packet =
         new packet_t(new memory_c(frame->data, frame->size, true),
                      frame->timecode, frame->duration,
                      frame->firstRef, frame->secondRef);
       new_packet->time_factor =
         MPEG2_PICTURE_TYPE_FRAME == frame->pictureStructure ? 1 : 2;
+
+      if ((NULL != frame->seqHdrData) &&
+          ((NULL == seq_hdr.get()) ||
+           (frame->seqHdrDataSize != seq_hdr->get_size()) ||
+           memcmp(frame->seqHdrData, seq_hdr->get(), frame->seqHdrDataSize))) {
+        seq_hdr = memory_cptr(new memory_c(frame->seqHdrData,
+                                           frame->seqHdrDataSize, true));
+        frame->seqHdrData = NULL;
+        new_packet->codec_state = clone_memory(seq_hdr);
+      }
+
       video_packetizer_c::process(packet_cptr(new_packet));
       frame->data = NULL;
       delete frame;
@@ -684,12 +744,15 @@ mpeg4_p10_video_packetizer_c(generic_reader_c *_reader,
                              int _width,
                              int _height,
                              track_info_c &_ti):
-  video_packetizer_c(_reader, MKV_V_MPEG4_AVC, _fps, _width, _height, _ti) {
+  video_packetizer_c(_reader, MKV_V_MPEG4_AVC, _fps, _width, _height, _ti),
+  m_nalu_size_len_src(0), m_nalu_size_len_dst(0), m_max_nalu_size(0) {
 
   relaxed_timecode_checking = true;
 
-  if ((ti.private_data != NULL) && (ti.private_size > 0))
+  if ((ti.private_data != NULL) && (ti.private_size > 0)) {
     extract_aspect_ratio();
+    setup_nalu_size_len_change();
+  }
 }
 
 void
@@ -736,6 +799,9 @@ mpeg4_p10_video_packetizer_c::process(packet_cptr packet) {
 
   ref_timecode = packet->timecode;
 
+  if (m_nalu_size_len_dst)
+    change_nalu_size_len(packet);
+
   add_packet(packet);
 
   return FILE_STATUS_MOREDATA;
@@ -766,3 +832,84 @@ mpeg4_p10_video_packetizer_c::can_connect_to(generic_packetizer_c *src,
   return CAN_CONNECT_YES;
 }
 
+void
+mpeg4_p10_video_packetizer_c::setup_nalu_size_len_change() {
+  if (!ti.private_data || (5 > ti.private_size))
+    return;
+
+  m_nalu_size_len_src = (ti.private_data[4] & 0x03) + 1;
+
+  if (!ti.nalu_size_length || (ti.nalu_size_length == m_nalu_size_len_src))
+    return;
+
+  m_nalu_size_len_dst = ti.nalu_size_length;
+  ti.private_data[4]  = (ti.private_data[4] & 0xfc) | (m_nalu_size_len_dst - 1);
+  m_max_nalu_size     = 1ll << (8 * m_nalu_size_len_dst);
+
+  set_codec_private(ti.private_data, ti.private_size);
+
+  mxverb(2, "mpeg4_p10: Adjusting NALU size length from %d to %d\n",
+         m_nalu_size_len_src, m_nalu_size_len_dst);
+}
+
+void
+mpeg4_p10_video_packetizer_c::change_nalu_size_len(packet_cptr packet) {
+  unsigned char *src = packet->data->get(), *dst;
+  int size           = packet->data->get_size();
+  int src_pos, dst_pos, nalu_size, i, shift;
+
+  if (!src || !size)
+    return;
+
+  vector<int> nalu_sizes;
+
+  src_pos = 0;
+
+  // Find all NALU sizes in this packet.
+  while (src_pos < size) {
+    if ((size - src_pos) < m_nalu_size_len_src)
+      break;
+
+    nalu_size = 0;
+    for (i = 0; i < m_nalu_size_len_src; ++i)
+      nalu_size = (nalu_size << 8) + src[src_pos + i];
+
+    if ((size - src_pos - m_nalu_size_len_src) < nalu_size)
+      nalu_size = size - src_pos - m_nalu_size_len_src;
+
+    if (nalu_size > m_max_nalu_size)
+      mxerror(FMT_TID "The chosen NALU size length of %d is too small. Try using '4'.\n",
+              ti.fname.c_str(), (int64_t)ti.id, m_nalu_size_len_dst);
+
+    src_pos += m_nalu_size_len_src + nalu_size;
+
+    nalu_sizes.push_back(nalu_size);
+  }
+
+  // Allocate memory if the new NALU size length is greater
+  // than the previous one. Otherwise reuse the existing memory.
+  if (m_nalu_size_len_dst > m_nalu_size_len_src) {
+    int new_size = size + nalu_sizes.size() * (m_nalu_size_len_dst - m_nalu_size_len_src);
+    packet->data = memory_cptr(new memory_c((unsigned char *)safemalloc(new_size), new_size, true));
+  }
+
+  // Copy the NALUs and write the new sized length field.
+  dst     = packet->data->get();
+  src_pos = 0;
+  dst_pos = 0;
+
+  for (i = 0; nalu_sizes.size() > i; ++i) {
+    nalu_size = nalu_sizes[i];
+
+    for (shift = 0; shift < m_nalu_size_len_dst; ++shift)
+      dst[dst_pos + shift] = (nalu_size >> (8 * (m_nalu_size_len_dst - 1 - shift))) & 0xff;
+
+    memmove(&dst[dst_pos + m_nalu_size_len_dst], &src[src_pos + m_nalu_size_len_src], nalu_size);
+
+    src_pos += m_nalu_size_len_src + nalu_size;
+    dst_pos += m_nalu_size_len_dst + nalu_size;
+  }
+
+  packet->data->set_size(dst_pos);
+}
+
